namespace de.osci.helper {

	using System;
	using System.Collections;
	using javax.xml.parsers;
	using de.osci.osci12.common;
	using de.osci.osci12.dotcommon; // Storestreams
	using de.osci.osci12.messagetypes;
	using org.apache.commons.logging;
	using System.Xml;
	using org.xml.sax;
	using System.Security.Cryptography;
	using org.xml.sax.helpers;
	using System.IO;

	/// <summary> <p>Handler fr die Kanonisierung.</p>
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: H. Tabrizi / N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	
	// Dieser Parser wird auch zum Parsen von zustzlichen SOAP-Headern verwendet.
	public class CanParser:DefaultHandler { //, LexicalHandler
		
		private static Log log;
		private static readonly System.String DEFAULT_ENCODING = Constants.CHAR_ENCODING;		
		protected internal int elementDepth;
		protected internal bool isFirstProcessing = true;
		private DigestStream outs = null;
		private XMLReader parser = null;
		private System.Collections.Stack sampleNSStack;
		protected internal bool useComment = false;
		private bool insideHeader = false;
        private bool insideContainerSignatureProps = false;
        private bool readNS; 
		private bool signedElement; 
		private System.Collections.SortedList tmpMap;
//		private SupportClass.MessageDigestSupport md;

        private System.Security.Cryptography.HashAlgorithm md;
        private bool insideTransportSignedInfo = false;
        private String transportSignatureMethod;
        private String currentTransportSignatureRef;
        private Hashtable transportDigestMethods;
        internal System.Collections.Hashtable messageDigests;

        private byte[] controlBlock;
        private String controlBlockID;

        private System.String id;
		internal System.Collections.Hashtable digestValues;
        internal System.Collections.ArrayList signedInfos;
        internal System.Collections.ArrayList signedProperties;

        internal System.Collections.ArrayList cocoNS; 
		private System.IO.Stream out_Renamed;
		private OSCIMessage msg;
		private ContentHandler parent;
		private System.String name;
		private System.Collections.ArrayList tag;
		/**
		 * SOAP-Namespace Identifier
		 */
		private String soapId;



		private StoreInputStream sis; // Storestreams
		private System.IO.TextWriter outWriter;
		private System.IO.TextWriter tmpWriter;

		// Constants
		/// <summary>Namespaces feature id (http://xml.org/sax/features/namespaces). 
		/// </summary>
		protected internal const System.String NAMESPACES_FEATURE_ID = "http://xml.org/sax/features/namespaces";
		/// <summary>Namespace-Prefix feature id (http://xml.org/sax/features/namespaces). 
		/// </summary>
		protected internal const System.String NAMESPACE_PREFIXES_FEATURE_ID = "http://xml.org/sax/features/namespace-prefixes";
		/// <summary>Validation feature id (http://xml.org/sax/features/validation). 
		/// </summary>
		protected internal const System.String VALIDATION_FEATURE_ID = "http://xml.org/sax/features/validation";
		/// <summary>Schema validation feature id (http://apache.org/xml/features/validation/schema). 
		/// </summary>
		protected internal const System.String SCHEMA_VALIDATION_FEATURE_ID = "http://apache.org/xml/features/validation/schema";
		/// <summary>Schema full checking feature id (http://apache.org/xml/features/validation/schema-full-checking). 
		/// </summary>
		protected internal const System.String SCHEMA_FULL_CHECKING_FEATURE_ID = "http://apache.org/xml/features/validation/schema-full-checking";
		/// <summary>Entity External General checking feature id (http://apache.org/xml/features/validation/schema-full-checking). 
		/// </summary>
		protected internal const System.String EXTERNAL_GENERAL_ENTITIES_FEATURE_ID = "http://xml.org/sax/features/external-general-entities";
		/// <summary>Entity External Parameter checking feature id (http://apache.org/xml/features/validation/schema-full-checking). 
		/// </summary>
		protected internal const System.String EXTERNAL_PARAMETER_ENTITIES_FEATURE_ID = "http://xml.org/sax/features/external-parameter-entities";
		/// <summary>Lexical handler property id (http://xml.org/sax/properties/lexical-handler). 
		/// </summary>
		protected internal const System.String crimson = "org.apache.crimson.parser.XMLReaderImpl"; 
    
		private const System.String LEXICAL_HANDLER_PROPERTY_ID = "http://xml.org/sax/properties/lexical-handler";

		private void  InitBlock() {
			sampleNSStack = new System.Collections.Stack();
		}

		public System.Collections.Hashtable DigestValues {
			get {
				return digestValues;
			}			
		}

		public CanParser(System.Collections.ArrayList tag, XMLReader xmlReader, ContentHandler parent, String name)
			:this(xmlReader, new System.IO.MemoryStream()){
			//			this(xmlReader, new ByteArrayOutputStream());
			this.tag = tag;
			this.parent = parent;
			this.name = name;
		}
		public CanParser(OSCIMessage msg, XMLReader xmlReader, ContentHandler parent, System.String name):this(xmlReader, new System.IO.MemoryStream()) {
			InitBlock();
			this.msg = msg;
			this.parent = parent;
			this.name = name;
		}

		public CanParser(System.IO.Stream out_Renamed, StoreInputStream sis):this(SAXParserFactory.newInstance().newSAXParser().XMLReader, out_Renamed) {
			this.sis = sis;
			InitBlock();
		}

		public CanParser(XMLReader xmlReader, System.IO.Stream out_Renamed) {
			InitBlock();
			this.out_Renamed = out_Renamed;
			parser = xmlReader;			
			parser.setContentHandler(this);
			//"SHA-1"
			outs = new DigestStream(out_Renamed);

//			md = SupportClass.MessageDigestSupport.GetInstance((System.String) Constants.JCA_JCE_MAP[DialogHandler.DigestAlgorithm]);
//			outs = new SupportClass.DigestOutputSupport(out_Renamed, md);
			outWriter = new System.IO.StreamWriter(outs, System.Text.Encoding.UTF8);
			outWriter.NewLine=null;
		}
		
		//Init()
		public override void  startDocument() {
			//			Console.WriteLine("START DOC");
			Hashtable sampleNSMap = new Hashtable();
			elementDepth = 0;
			sampleNSMap.Add("xmlns", "");
			this.sampleNSStack.Push(sampleNSMap);
			digestValues = new System.Collections.Hashtable();
            transportDigestMethods = new Hashtable();
            messageDigests = new Hashtable();
            currentTransportSignatureRef = "";
            if (signedInfos == null) signedInfos = new ArrayList();
            if (signedProperties == null) signedProperties = new ArrayList();
            if (cocoNS == null) cocoNS = new ArrayList(); 
//			md.Reset();
//			((DigestStream)outs).setActive();
			soapId = null;
		}
		
		//startDocument()
		/// <summary>Processing instruction. 
		/// </summary>
		public void  processingInstruction(System.String target, System.String data) {
			if ((elementDepth == 0) && isFirstProcessing) {
		//		byte [] ba=MoreSupport.ToByteArray("<?" + target + " " + this.normalizeProcessingInstruction(data) + "?>\n",DEFAULT_ENCODING);
		//		outs.Write(ba,0,ba.Length);
				outWriter.Write("<?" + target + " ");
				normalizeProcessingInstruction(data) ;
				outWriter.Write( "?>\n");
			}
				
			if ((elementDepth == 0) && !isFirstProcessing) {
				outWriter.Write("\n<?" + target + " ");
				normalizeProcessingInstruction(data) ;
				outWriter.Write( "?>");
			}
				
			if (isFirstProcessing)
				isFirstProcessing = false;
		}
		
		/// <summary>Start element. 
		/// </summary>
		public override void  startElement(System.String uri, 
			System.String localName, 
			System.String qName, 
			Attributes    attrs) {
			signedElement = false; 
			readNS = false; 
//			Console.WriteLine("*START ELEM "+qName);
//			System.Diagnostics.Debug.WriteLine("/START ELEM "+qName);
			int attCount = attrs.Length;
			System.Collections.SortedList outMap = new System.Collections.SortedList();
			Hashtable ht;
			if (sampleNSStack.Count>0) ht=(Hashtable)sampleNSStack.Peek();else ht=new Hashtable();
			Hashtable sampleNSMap = new Hashtable(ht);
			elementDepth++;

			// SOAP-Namespacedefinition suchen
			if (soapId == null) {
				soapId = "";
				for (int i = 0; i < attrs.Length; i++) {
						if (attrs.getValue(i).Equals("http://schemas.xmlsoap.org/soap/envelope/")) {
							soapId = attrs.getQName(i).Substring(attrs.getQName(i).IndexOf(':') + 1);

							break;
						}
					}

			}

			// --> StoreStreams
			// Entscheiden, ob der StoreStream die jetzt gepufferten Bytes wegwerfen oder
			// wegschreiben soll.
			if ((sis != null) && (qName.Equals(soapId + ":Envelope"))) 
			{
				String schemaValue = attrs.getValue("xsi:schemaLocation");
				sis.Save = schemaValue == null || schemaValue.IndexOf("soapMessageEncrypted.xsd") < 0;
			}
			// <-- StoreStreams

			if (qName.Equals(soapId + ":Header"))
				insideHeader = true;
            else if (insideHeader && localName.Equals("ControlBlock"))
            {
                signedElement = true;
                controlBlockID = "#" + attrs.getValue("Id");
                outWriter = new SplitWriter((StreamWriter)outWriter);
            }
			else if ((insideHeader && elementDepth == 3 && !qName.EndsWith("Signature")) || qName.Equals(soapId + ":Body")){
				id = attrs.getValue("Id");
                String mdm = (String)transportDigestMethods["#" + id];
                if (mdm == null)
                    mdm = Constants.DIGEST_ALGORITHM_SHA256;
                signedElement = true;
                try
                {
                    outWriter.Flush();
                }
                catch (Exception e)
                {
                    log.error(e);
                }
                md = (HashAlgorithm)messageDigests[mdm];
                if (md == null)
                {
                    md = MoreSupport.createMessageDigest(mdm);
                    messageDigests.Add(mdm, md);
                }

//                md.reset();
                //              outs.flush();
                outs.setMessageDigest(md);
                outs.on(true);

			}
			else if (localName.Equals("SignedInfo")
                        || (!insideHeader && localName.Equals("SignedProperties")))
			{ 
				signedElement = true; 
				readNS = true; 
				outWriter = new SplitWriter((StreamWriter)outWriter);
                if (insideHeader && localName.Equals("SignedInfo"))
                    insideTransportSignedInfo = true;
                else if (localName.Equals("SignedProperties"))
                    insideContainerSignatureProps = true;
            }
            else if (insideTransportSignedInfo)
            {
//                if (localName.Equals("SignatureMethod"))
//                    transportSignatureMethod = attrs.getValue("Algorithm");
//                else 
                if (localName.Equals("Reference"))
                    currentTransportSignatureRef = attrs.getValue("URI");
                else if (localName.Equals("DigestMethod"))
                {
                    transportDigestMethods.Add(currentTransportSignatureRef, attrs.getValue("Algorithm"));
                }
			}
            else if (localName.Equals("ContentContainer"))
            {
                signedElement = true;
                readNS = true;
            }
			
			//Sorting Attributes
			for (int i = 0; i < attCount; i++) 
			{
				String QName =attrs.getQName(i);
				String val = attrs.getValue(i);
				if (QName.StartsWith("xmlns:") || QName.Equals("xmlns")) 
				{
					if (!sampleNSMap.ContainsKey(QName) ) {
						outMap.Add(new MyString(" ," + QName), val);
						sampleNSMap.Add(QName, val);
					}
					else if (!val.Equals((String)sampleNSMap[QName])) {
						outMap.Add(new MyString(" ," + QName), val);
						sampleNSMap[QName]= val;
					}
				}
				else 
				{
					// looking for Attributes with Namespace Prefix
					if (!attrs.getURI(i).Equals(""))
						outMap.Add(new MyString(attrs.getURI(i) + "," + QName), val);
					else
						outMap.Add(new MyString("," + QName),val);
				}
			}
			
			this.sampleNSStack.Push(sampleNSMap);
			if (readNS) { 
				tmpMap = outMap; 
				outMap = new System.Collections.SortedList(); 
			} 
    
			outWriter.Write("<" + qName); 
    
			if (signedElement) { 
				String nskey, nsval;
				MyString mystring; 
				//for (Iterator iterator = sampleNSMap.keySet().iterator(); iterator.hasNext();) { 
				foreach (Object o in sampleNSMap.Keys){
					nskey = ((String)o);
					nsval = (String)sampleNSMap[o];
					if (nsval.Length > 0)
					{
						mystring = new MyString(" ,"+nskey);
						if (!outMap.ContainsKey(mystring))
							outMap.Add(mystring, nsval);
					}
				} 
			}
    
			if (readNS) { 
				tmpWriter = outWriter; 
				outWriter = new System.IO.StringWriter();
				outWriter.NewLine=null;

				foreach (DictionaryEntry entry in outMap){
					//while (itr.hasNext()) { 
//					Map.Entry entry = (Map.Entry) itr.next(); 
					String sKey = ((MyString) entry.Key).text; 
					outWriter.Write(" " + sKey.Substring(sKey.IndexOf(",") + 1) + "=\""); 
					this.normalizeAttr((String) entry.Value); 
					outWriter.Write("\""); 
				} 
    
				outMap = tmpMap; 
				if (localName.Equals("SignedInfo") || insideContainerSignatureProps) 
					((SplitWriter)tmpWriter).sb.Append(((System.IO.StringWriter)outWriter).GetStringBuilder()); 
				else {
						string s=(outWriter).ToString();
						cocoNS.Add(s); 
				}
				outWriter = tmpWriter; 
				tmpWriter = null; 
			} 
		
			// Normalize Attributes			
			//byte[] ba=MoreSupport.ToByteArray("<" + qName,DEFAULT_ENCODING);
			//pr dez4  outs.Write(ba,0,ba.Length);
	//		outWriter.Write("<" + qName);

			foreach( DictionaryEntry it in outMap) {
				String sKey = ((MyString) it.Key).text; 
				String s=" " + sKey.Substring(sKey.IndexOf(",") + 1) + "=\"" ;
				//+ this.normalizeAttr((System.String) it.Value) + "\"";
				outWriter.Write(s);
				normalizeAttr((System.String) it.Value);
				outWriter.Write("\"");
				//			ba=MoreSupport.ToByteArray(s,DEFAULT_ENCODING);
	//			outs.Write(ba,0,ba.Length);
			}
				
//			ba=MoreSupport.ToByteArray(">",DEFAULT_ENCODING);
//			outs.Write(ba,0,ba.Length);
			outWriter.Write(">");
		}
		
		/// <summary>Characters. 
		/// </summary>
		public override void  characters(char[] ch, int start, int length) {
//			byte [] ba=MoreSupport.ToByteArray(normalizeText(new System.String(ch, start, length)),DEFAULT_ENCODING);
//			outs.Write(ba,0,ba.Length);
//			outWriter.Write(normalizeText(new System.String(ch, start, length)));
			normalizeText(new System.String(ch, start, length));
		}
		
		/// <summary>Ignorable whitespace. 
		/// </summary>
		public void  ignorableWhiteSpace(char[] ch, int start, int length) {
			characters(ch, start, length);
		}
		
		/// <summary>End element. 
		/// </summary>
		public override void  endElement(System.String uri, System.String localName, System.String qName) 
        {
			elementDepth--;
            outWriter.Write("</" + qName + ">");
			if (localName.Equals("SignedInfo")) 
            { 
				// Keine Nachrichtenverschlsselung 
				if (!insideHeader && signedInfos.Count==0) 
					signedInfos.Add(null); 
				byte[] ba0=MoreSupport.ToByteArray(((SplitWriter)outWriter).sb.ToString(),DEFAULT_ENCODING);
				signedInfos.Add(ba0); 
				outWriter = ((SplitWriter)outWriter).wr;
                if (insideTransportSignedInfo)
                {
                    insideTransportSignedInfo = false;
                    String digAlgo = (String)transportDigestMethods[controlBlockID];
                    if (digAlgo != null)
                    {
                        md = MoreSupport.createMessageDigest(digAlgo);
                        messageDigests.Add(digAlgo, md);
                        if (digestValues.ContainsKey(controlBlockID))
                            throw new SAXException(DialogHandler.text.getString("unexpected_entry"));
                        //  id
                        digestValues.Add(controlBlockID, md.ComputeHash(controlBlock));
                    }
                }
            }
            else if (insideContainerSignatureProps && localName.Equals("SignedProperties"))
            {
                signedProperties.Add(((SplitWriter)outWriter).sb.ToString());
                outWriter = ((SplitWriter)outWriter).wr;
                insideContainerSignatureProps = false;
            }
            else if (insideHeader && localName.Equals("ControlBlock"))
            {
                outWriter.Flush();
                controlBlock = MoreSupport.ToByteArray(((SplitWriter)outWriter).sb.ToString());
                outWriter = ((SplitWriter)outWriter).wr;
                id = null;
  	 
			}
            else if (((insideHeader) && (elementDepth == 2) && !qName.EndsWith("Signature")) 
                || (qName.Equals(soapId + ":Body")))
            {
                outWriter.Flush();
                if (id != null)
				{
					if (digestValues.ContainsKey(id))
						throw new SAXException(DialogHandler.text.getString("unexpected_entry"));
//					SupportClass.PutElement(digestValues,"#" + id, md.DigestData());
                    else
					    SupportClass.PutElement(digestValues,"#" + id, outs.getDigest());
				}
//				md.Reset();
                outs.on(false);
                id = null;
			}
				
			if (!(this.sampleNSStack.Count == 0))
				this.sampleNSStack.Pop();
			
			if (qName.Equals(soapId + ":Header"))
				insideHeader = false;
			
			if ((name != null) && (qName.Equals(name)))
				endDocument();
		}
		
		/// <summary>End document. 
		/// </summary>
		public override void  endDocument() {
			isFirstProcessing = true;
			outs.Flush();
			outWriter.Close();
			if (tag != null) {
				out_Renamed.Position=0;				
				byte[] tmp = ((MemoryStream)out_Renamed).ToArray();
				System.String uf = System.Text.UTF8Encoding.UTF8.GetString(tmp, 1, tmp.Length - 1);
//				System.String uf = MoreSupport.NewString(out_Renamed);
				tag.Add(uf);
//				out_Renamed.
				parser.setContentHandler((DefaultHandler)parent);
			}
		}
		
		public override void  warning(SAXParseException ex) {
			log.debug("Warning: " + ex);
			throw ex;
		}
		
		/// <summary>Error. 
		/// </summary>
		public override void  error(SAXParseException ex) {
			log.debug("Error: " + ex);
			throw ex;
		}
		
		/// <summary>Fatal error. 
		/// </summary>
		public override void  fatalError(SAXParseException ex) {
			log.debug("Fatal Error: " + ex);
			throw ex;
		}
		
		public void  startDTD(System.String name, System.String publicId, System.String systemId) {
		}
		
		/// <summary>End DTD. 
		/// </summary>
		public void  endDTD() {
		}
		
		/// <summary>Start entity. 
		/// </summary>
		public void  startEntity(System.String name) {
		}
		
		/// <summary>End entity. 
		/// </summary>
		public void  endEntity(System.String name) {
		}
		
		/// <summary>Start CDATA section. 
		/// </summary>
		public void  startCDATA() {
			log.debug("CDATA");
		}
		
		/// <summary>End CDATA section. 
		/// </summary>
		public void  endCDATA() {
			log.debug("CDATA");
		}
		
		/// <summary>Comment. 
		/// </summary>
		public void  comment(char[] ch, int start, int length) {
			if (useComment) {
				byte [] ba;
				if (elementDepth == 0) {
	//				ba=MoreSupport.ToByteArray("\n",DEFAULT_ENCODING);
	//				outs.Write(ba,0,ba.Length);
					outWriter.Write("\n");
				}
				
//				ba=MoreSupport.ToByteArray("<!--" + 
//					normalizeText(new System.String(ch, start, length)) + "-->",DEFAULT_ENCODING);
//				outs.Write(ba,0,ba.Length);
				outWriter.Write("<!--");
				normalizeText(new System.String(ch, start, length)) ;
				outWriter.Write("-->");
			}
		}
		
		/// <summary>Normalize Attribute 
		/// </summary>
		protected internal System.String normalizeAttr(System.String s) {
			System.Text.StringBuilder stringbuffer = new System.Text.StringBuilder();
			int i = (s == null)?0:s.Length;
			
			for (int j = 0; j < i; j++) {
				char c = s[j];
				
				switch (c) {
					case '&': 
						stringbuffer.Append("&amp;");
						break;
					
					case '<': 
						stringbuffer.Append("&lt;");
						break;
						
					case '>':
						  stringbuffer.Append("&gt;");
						  break;
					case '"': 
						stringbuffer.Append("&quot;");
						break;
					
					case (char) (0x09): 
						stringbuffer.Append("&#x9;");
						break;
					
					case (char) (0x0A): 
//						stringbuffer.Append("&#xA;");
						outWriter.NewLine=null;
						outWriter.Write(s.Substring(j,1));

						break;
					
					case (char) (0x0D): 
						stringbuffer.Append("&#xD;");
						break;
					
					default: 
						stringbuffer.Append(c);
						break;					
				}
			}
			this.outWriter.Write(stringbuffer.ToString());
			return stringbuffer.ToString();
		}
		
		/// <summary>Normalize Processing Instruction
		/// </summary>
		protected internal System.String normalizeProcessingInstruction(System.String s) {
			System.Text.StringBuilder stringbuffer = new System.Text.StringBuilder();
			int i = (s == null)?0:s.Length;
			
			for (int j = 0; j < i; j++) {
				char c = s[j];
				
				switch (c) {
					case (char) (0x0D): 
						stringbuffer.Append("&#xD;");
						break;					
					
					default: 
						stringbuffer.Append(c);
						break;
				}
			}
			this.outWriter.Write(stringbuffer.ToString());
			return stringbuffer.ToString();
		}
		
		/// <summary>Normalize Text
		/// </summary>
		protected internal void normalizeText(System.String s) {
			System.Text.StringBuilder stringbuffer = new System.Text.StringBuilder();
			int i = (s == null)?0:s.Length;
			
			for (int j = 0; j < i; j++) {
				char c = s[j];
				
				switch (c) {
					case '&': 
						outWriter.Write("&amp;");
						break;
					case '<': 
						outWriter.Write("&lt;");
						break;
					case '>': 
						outWriter.Write("&gt;");
						break;				
					case (char) (0xD): 
// JAVA-Version ignores '\r', too
//						outWriter.Write("&#xD;");
						break;
					case (char) (0xA): 
						outWriter.NewLine=null;
						outWriter.Write(s.Substring(j,1));
						break;
					default: 
						outWriter.Write(s.Substring(j,1));
						break;
				}
			}
	//		outWriter.Write(stringbuffer.ToString());
	//		return stringbuffer.ToString();
		}
		
		/// <summary>Start Canonicalization
		/// </summary>
		public void  startCanonicalization(System.IO.Stream is_Renamed, bool withComment) {

			System.IO.BufferedStream input = new System.IO.BufferedStream(is_Renamed);
			this.useComment = withComment;
			parser.parse(input);
		}
		
		static CanParser() {
			log = LogFactory.getLog(typeof(Canonizer));
		}
	}
	internal class SplitWriter : System.IO.StreamWriter { 
			internal	StreamWriter  wr; 
			internal	System.Text.StringBuilder sb = new System.Text.StringBuilder(); 
    
	 public SplitWriter(System.IO.StreamWriter w):base(new System.IO.MemoryStream())
	 { 
			wr = w; 
	} 
    
	public override void Write(String s) { 
	 sb.Append(s); 
		wr.Write(s); 
	} 
    
	public override void Close() { 
	wr.Close(); 
} 
    
	public override void Flush() { 
	wr.Flush(); 
} 
    
	public override void Write(int c){ 
	sb.Append((char)c); 
	wr.Write(c); 
} 
    
	public override void Write(char[] cbuf, int off, int len)  { 
	this.Write(new String(cbuf, off, len)); 
} 
    
    
} 
    

	/// <summary>
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class MyString:IComparable {
		public String text;

		public MyString(String s) {
			text = s;
		}

		public int CompareTo(Object o) {
			return String.CompareOrdinal(text, ((MyString)o).text);
		}
	}
}