namespace de.osci.helper {
	
	using System;
	using System.IO;
	using javax.xml.parsers;
	using de.osci.osci12.common;
	using de.osci.osci12.dotcommon;
	using de.osci.osci12.messagetypes;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;

	/// <summary> Diese Klasse fhrt die Kanonisierung gem der Spezifikation
	/// http://www.w3.org/TR/xml-c14n durch. Die Funktion beschrnkt
	/// sich auf die Anforderungen der OSCI 1.2 Transportbibliothek.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: H. Tabrizi / N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.extinterfaces.DialogFinder">
	/// </seealso>
	public class Canonizer:MemoryStream {

		private static Log log;

		// Constants
		/// <summary>Namespaces feature id (http://xml.org/sax/features/namespaces). 
		/// </summary>
		protected internal const System.String NAMESPACES_FEATURE_ID = "http://xml.org/sax/features/namespaces";
		/// <summary>Namespace-Prefix feature id (http://xml.org/sax/features/namespaces). 
		/// </summary>
		protected internal const System.String NAMESPACE_PREFIXES_FEATURE_ID = "http://xml.org/sax/features/namespace-prefixes";
		/// <summary>Validation feature id (http://xml.org/sax/features/validation). 
		/// </summary>
		protected internal const System.String VALIDATION_FEATURE_ID = "http://xml.org/sax/features/validation";
		/// <summary>Schema validation feature id (http://apache.org/xml/features/validation/schema). 
		/// </summary>
		protected internal const System.String SCHEMA_VALIDATION_FEATURE_ID = "http://apache.org/xml/features/validation/schema";
		/// <summary>Schema full checking feature id (http://apache.org/xml/features/validation/schema-full-checking). 
		/// </summary>
		protected internal const System.String SCHEMA_FULL_CHECKING_FEATURE_ID = "http://apache.org/xml/features/validation/schema-full-checking";
		/// <summary>Entity External General checking feature id (http://apache.org/xml/features/validation/schema-full-checking). 
		/// </summary>
		protected internal const System.String EXTERNAL_GENERAL_ENTITIES_FEATURE_ID = "http://xml.org/sax/features/external-general-entities";
		/// <summary>Entity External Parameter checking feature id (http://apache.org/xml/features/validation/schema-full-checking). 
		/// </summary>
		protected internal const System.String EXTERNAL_PARAMETER_ENTITIES_FEATURE_ID = "http://xml.org/sax/features/external-parameter-entities";
		/// <summary>Lexical handler property id (http://xml.org/sax/properties/lexical-handler). 
		/// </summary>
		private const System.String LEXICAL_HANDLER_PROPERTY_ID = "http://xml.org/sax/properties/lexical-handler";
		/// <summary>Default parser name. 
		/// </summary>
		//  protected static final String DEFAULT_PARSER_NAME = "org.apache.xerces.parsers.SAXParser";
		private static readonly System.String DEFAULT_ENCODING = Constants.CHAR_ENCODING;
		
		// Data
		private MemoryStream pis;
		private System.IO.IOException ioex;
		private SAXException saxex;
		private System.IO.Stream input;
		private CanParser cp;

		virtual public System.Collections.Hashtable DigestValues 
        {			
			get 
            {
				return cp.digestValues;
			}
		}

        virtual public System.Collections.ArrayList SignedInfos
        {
            get
            {
                return cp.signedInfos;
            }
        }

        virtual public System.Collections.ArrayList SignedProperties
        {
            get
            {
                return cp.signedProperties;
            }
        }

        virtual public System.Collections.ArrayList ContainerNS 
        {			
			get 
            {
				return cp.cocoNS; 
			} 
		}
		// --> Storestreams:
		public Canonizer(System.IO.Stream in_Renamed, StoreInputStream sis) {
			input = in_Renamed;

			cp = new CanParser(this, sis);
			cp.startCanonicalization(in_Renamed, false);
			// <-- Storestreams:

			this.Position = 0;

			ioex = null;
			saxex = null;
			
			if (ioex != null)
				throw ioex;
			if (saxex != null)
				throw saxex;
		}

		static Canonizer() {
			log = LogFactory.getLog(typeof(Canonizer));
		}
	}
}