namespace de.osci.osci12 {

	using System;
	using dothelpers;

	// Diese Exception wird fr die in der OSCI-Spezifikation definierten Fehlermeldungen
	// sowie fr allgemeine eigene Fehlermeldungen der Bibliothek, die keiner besonderen Kategorie
	// zugeordnet werden knnen. Auerdem stellt sie die Superklasse aller brigen bibliothekseigenen
	// Exceptions dar. 
	/// <summary> Diese Klasse stellt die Superklasse aller bibliothekseigenen
	/// Exceptions dar.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p>
	/// <p>Version: 1.5</p>
	/// </summary>
	public class OSCIException:System.Exception {

		protected String errorCode = "null"; 
		public static System.Resources.ResourceManager us_text;
		//		public static ResourceBundle us_text = ResourceBundle.getBundle("de.osci.osci12.extinterfaces.language.Text", 
//			java.util.Locale.US); 
		/// <summary> Liefert den OSCI-Fehlercode.
		/// </summary>
		/// <value> Code
		/// </value>
		virtual public System.String ErrorCode {
			get {
				return errorCode;
			}			
		}

		/// <summary> Liefert die Exception-Nachricht in der jeweiligen Sprache (Default-Locale).
		/// </summary>
		/// <returns> lokale Nachricht
		/// </returns>
		public String getLocalizedMessage() {
			try {return de.osci.osci12.common.DialogHandler.text.getString(errorCode);}
			catch (NullReferenceException ex) {return errorCode + " - "+Message;}
		}

		// <overloads>Diese Methode hat berladungen</overloads>
		// <summary> Initialisiert eine neue Instanz der OSCIException Klasse.
		// </summary>
		public OSCIException() {
		}
		
		/// <summary> Erzeugt ein OSCIException-Objekt mit einem Fehlercode. 
		/// Als Fehlercode kann jeder String verwendet werden, fr den
		/// ein entsprechender Eintrag in der zum gesetzten default-Locale gehrenden
		/// Sprachdatei (s. Package de.osci.osci12.extinterfaces.language) vorhanden ist.
		/// </summary>
		/// <param name="errorCode">errorCode
		/// </param>
		public OSCIException(System.String errorCode):base() {
			this.errorCode = errorCode;
		}

		/// <summary> Erzeugt ein OSCIException-Objekt mit einem erklrenden String und
		/// einem Fehlercode. Als Fehlercode kann jeder String verwendet werden, fr den
		/// ein entsprechender Eintrag in der zum gesetzten default-Locale gehrenden
		/// Sprachdatei (s. Package de.osci.osci12.extinterfaces.language) vorhanden ist.
		/// </summary>
		/// <param name="message">message
		/// </param>
		/// <param name="errorCode">errorCode
		/// </param>
		public OSCIException(System.String message, System.String errorCode):base(message) 
		{
			this.errorCode = errorCode;
		}
		
		static OSCIException() {
			//UPGRADE_TODO: Stellen Sie sicher, dass in dieser Klasse verwendete Ressourcen gltige Ressourcendateien sind. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1078_3"'
			System.Threading.Thread.CurrentThread.CurrentUICulture = new System.Globalization.CultureInfo("en-US");
			us_text = System.Resources.ResourceManager.CreateFileBasedResourceManager("de.osci.osci12.extinterfaces.language.Text", "", null);
		}
	}
}