namespace de.osci.osci12.encryption {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	using de.osci.osci12.extinterfaces;
	using de.osci.osci12.messageparts;

	/// <exclude/>
	/// <summary>
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class CipherValue 
	{

		private static Log log;
		private System.String value_Renamed;
		private OSCIDataSource swapBuffer = null;
		private ContentContainer coco = null;
		private bool base64Encoding_Renamed_Field = true;
		
		private bool usageSecretKey_Renamed_Field = true;
		private javax.crypto.SecretKey key = null;
		private byte[] ivVector = null;
		public const int STATE_DECRYPTED = 0;
		public const int STATE_ENCRYPTED = 2;
		internal int stateOfObject;

		private void  InitBlock() {
			stateOfObject = STATE_DECRYPTED;
		}

		virtual internal javax.crypto.SecretKey SecretKey {
			set {
				if (stateOfObject == STATE_ENCRYPTED) {
					throw new System.SystemException("Das Object befindet sich schon in verschlsseltem Zustand");
				}		
				this.key = value;
			}			
		}

		virtual public OSCIDataSource CipherValueBuffer {
			get {
				log.debug("This Object:" + this);
				
				if (coco != null) {
					log.debug("########Es handlet sich um ein COCO");
					
					OSCIDataSource tempSwap = DialogHandler.NewDataBuffer;
					this.writeCipherValue(tempSwap.OutputStream);
					
					return tempSwap;
				} else {
					log.debug("########Es handlet sich um ein byte data. Lnge des SwapBuffers: " + swapBuffer.Length + " State ist:" + stateOfObject);
					
					if (stateOfObject == STATE_DECRYPTED) {
						this.writeCipherValue(null);
					}
					swapBuffer.InputStream.Seek(0,System.IO.SeekOrigin.Begin);
					return swapBuffer;
				}
			}	
		}

		virtual public byte[] CipherValueBytes {
			get {
				System.IO.MemoryStream bos = new System.IO.MemoryStream();
				System.IO.Stream is_Renamed = swapBuffer.InputStream;
				byte[] tmp = new byte[128];
				int i;
				
				while ((i = SupportClass.ReadInput(is_Renamed, ref tmp, 0, tmp.Length)) > 0) {
					bos.Write(tmp, 0, i);
				}
				is_Renamed.Seek(0,System.IO.SeekOrigin.Begin);
				bos.Close();
				
				return bos.ToArray();
			}	
		}

		/// <summary> Erstellt ein neues CipherValue-Objekt.
		/// </summary>
		/// <param name="data"> data
		/// </param>
		public CipherValue(System.String data) {
			InitBlock();
			swapBuffer = DialogHandler.NewDataBuffer;
			log.debug("Konstruktor String Data");
			
			System.IO.Stream out_Renamed = swapBuffer.OutputStream;
			MoreSupport.ToStream(out_Renamed,data,Constants.CHAR_ENCODING);
			out_Renamed.Flush();
		}
		
		/// <summary> Erstellt ein neues CipherValue-Objekt.
		/// </summary>
		/// <param name="dataBytes">dataBytes
		/// </param>
		public CipherValue(byte[] dataBytes) {
			InitBlock();
			swapBuffer = DialogHandler.NewDataBuffer;
			log.debug("Konstruktor bytes. Lnge: " + dataBytes.Length + " swap: " + swapBuffer);
				
			System.IO.Stream out_Renamed = swapBuffer.OutputStream;
			byte[] temp_byteArray;
			temp_byteArray = dataBytes;
			out_Renamed.Write(temp_byteArray, 0, temp_byteArray.Length);;;;
			out_Renamed.Flush();
			out_Renamed.Close();
		}
		
		internal void setSecretKey(javax.crypto.SecretKey key) {
			if (stateOfObject == STATE_ENCRYPTED)
				throw new System.SystemException("Das Object befindet sich schon in verschlsseltem Zustand");
			this.key = key;
		}
		
		internal virtual void  setSecretKey(javax.crypto.SecretKey key, System.String encryptionMethodAlgorithm) {
			if (!Constants.SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC_JCA_JCE.Equals(encryptionMethodAlgorithm)) {
				throw new System.ArgumentException("Es wird derzeit nur: " + Constants.SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC_JCA_JCE + " untersttzt.");
			}
			if (stateOfObject == STATE_ENCRYPTED) {
				throw new System.SystemException("Das Object befindet sich schon in verschlsseltem Zustand");
			}
			
			this.key = key;
		}
		
		/// <summary> Erstellt ein neues CipherValue-Objekt.
		/// </summary>
		/// <param name="contentContainer">contentContainer
		/// </param>
		public CipherValue(de.osci.osci12.messageparts.ContentContainer contentContainer) {
			InitBlock();
			coco = contentContainer;
		}
		
		/// <summary> Erstellt ein neues CipherValue-Objekt.
		/// </summary>
		/// <param name="dataSource">dataSource
		/// </param>
		public CipherValue(de.osci.osci12.extinterfaces.OSCIDataSource dataSource) {
			InitBlock();
			swapBuffer = dataSource;
		}
		
		/// <summary> Erstellt ein neues CipherValue-Objekt.
		/// </summary>
		/// <param name="dataStream">dataStream
		/// </param>
		public CipherValue(System.IO.Stream dataStream) {
			InitBlock();
			swapBuffer = DialogHandler.NewDataBuffer;
			log.debug("Konstruktor InputStream");
			
			byte[] bytes = new byte[1024];
			int anz = 0;
			
			while ((anz = SupportClass.ReadInput(dataStream, ref bytes, 0, bytes.Length)) > 0) {
				swapBuffer.OutputStream.Write(bytes, 0, anz);
			}
			swapBuffer.OutputStream.Flush();
		}

		public virtual void  useSecretKey(bool usage) {
			this.usageSecretKey_Renamed_Field = usage;
		}
		
		public virtual bool usageSecretKey() {
			return usageSecretKey_Renamed_Field;
		}
		
		public virtual bool base64Encoding() {
			return base64Encoding_Renamed_Field;
		}
		
		public virtual void  doBase64Encoding(bool encodingOn) {
			base64Encoding_Renamed_Field = encodingOn;
		}
		
		private void  writeCipherValue(System.IO.Stream oriOut) {
			log.debug("(start) writeCipherValue state:" + stateOfObject + "base64: " + base64Encoding_Renamed_Field + " ist contentData: " + coco);
			
			System.IO.Stream returnOut = null;
			OSCIDataSource tempSwap = null;
			
			if (coco != null) {
				returnOut = oriOut;
			} else {
				log.debug("SwapBuffer wird gesetzt!");
				tempSwap = DialogHandler.NewDataBuffer;
				returnOut = tempSwap.OutputStream;
			}
			
			if ((coco != null) || ((stateOfObject == STATE_DECRYPTED) && (key != null) && usageSecretKey_Renamed_Field)) {
				log.debug("base64 und verschlsselung");
				returnOut = new de.osci.osci12.common.Base64OutputStream(returnOut, true);
				de.osci.helper.SymCipherOutputStream tdesOut = new de.osci.helper.SymCipherOutputStream(returnOut, key, ivVector);
				ivVector = tdesOut.Iv;
				returnOut = tdesOut;
			} else if ((base64Encoding_Renamed_Field == true) && (stateOfObject == STATE_DECRYPTED)) {
				returnOut = new de.osci.osci12.common.Base64OutputStream(returnOut, true);
			}
				
			if (coco != null) {
				coco.writeXML(returnOut);
				oriOut.Flush();
				if (returnOut is de.osci.helper.SymCipherOutputStream) {
					log.debug("der Outputstream wird geclosed" + this);
					returnOut.Close();
				}
			} else if (stateOfObject == STATE_DECRYPTED) {
				// es handles sich um StreamData oder um Byte data (kein COCO) verschlsselung und / oder base64 wurde noch nicht durchgefhrt
				System.IO.Stream in_Renamed = swapBuffer.InputStream;
				in_Renamed.Seek(0,System.IO.SeekOrigin.Begin);

				byte[] bytes = new byte[1024];
				int anz = 0;
				System.IO.MemoryStream my = new System.IO.MemoryStream();
					
				while ((anz = SupportClass.ReadInput(in_Renamed, ref bytes, 0, bytes.Length)) > 0) {
					returnOut.Write(bytes, 0, anz);
				}
				stateOfObject = STATE_ENCRYPTED;
				this.swapBuffer = tempSwap;
				if (returnOut is de.osci.helper.SymCipherOutputStream) {
					log.debug("der Outputstream wird geclosed ohne coco");
					returnOut.Close();
				}
					
				returnOut.Flush();
				// und nun noch einmal die Bytes herausschreiben
				if (oriOut != null) {
					in_Renamed = swapBuffer.InputStream;
					in_Renamed.Seek(0,System.IO.SeekOrigin.Begin);
					anz = 0;
					while ((anz = SupportClass.ReadInput(in_Renamed, ref bytes, 0, bytes.Length)) > 0) {
						oriOut.Write(bytes, 0, anz);
						my.Write(bytes, 0, anz);
					}
				}
				// interner Buffer ist auf dem neustem Stand herausschreiben des Buffers
			} else if (oriOut != null) {
				System.IO.Stream in_Renamed = swapBuffer.InputStream;
				in_Renamed.Seek(0,System.IO.SeekOrigin.Begin);
				int anz = 0;
				byte[] bytes = new byte[1024];
					
				while ((anz = SupportClass.ReadInput(in_Renamed, ref bytes, 0, bytes.Length)) > 0) {
					oriOut.Write(bytes, 0, anz);
				}
				in_Renamed.Seek(0,System.IO.SeekOrigin.Begin);
			}
		}
		
		public virtual void  writeXML(System.IO.Stream out_Renamed, String xenc) {
			log.debug("Wert base64 " + base64Encoding_Renamed_Field + " do encryption: " + usageSecretKey_Renamed_Field);
			MoreSupport.ToStream(out_Renamed,"<"+ xenc + ":CipherValue>",Constants.CHAR_ENCODING);
			writeCipherValue(out_Renamed);
			MoreSupport.ToStream(out_Renamed,"</"+ xenc + ":CipherValue>",Constants.CHAR_ENCODING);
		}

		static CipherValue() {
			log = LogFactory.getLog(typeof(CipherValue));
		}
	}
}