namespace de.osci.osci12.encryption {

	using System;
	using javax.crypto;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	using System.Security.Cryptography;

	/// <exclude/>
	/// <summary>
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class Crypto 
	{

		private static String hex = "0123456789abcdef";
		private static Log log;
		
		// Asymetrische Verfahren
		//public final static String RSA15 = "http://www.w3.org/2001/04/xmlenc#rsa-1_5";
		
		// Symetrische Verfahen
		//public final static String TDES ="http://www.w3.org/2001/04/xmlenc#tripledes-cbc";
		//public final static String AES128 ="http://www.w3.org/2001/04/xmlenc#aes128-cbc";
		//public final static String AES192 ="http://www.w3.org/2001/04/xmlenc#aes192-cbc";
		//public final static String AES256 ="http://www.w3.org/2001/04/xmlenc#aes256-cbc";

		public static System.Security.Cryptography.SymmetricAlgorithm createSymmetricAlgorithm(String keyname) {
			System.Security.Cryptography.SymmetricAlgorithm sa;
			if (keyname.Equals("http://www.w3.org/2001/04/xmlenc#tripledes-cbc")) {
			 sa = new TripleDESCryptoServiceProvider();}
			else if (keyname.Equals("http://www.w3.org/2001/04/xmlenc#aes128-cbc")) {
			 sa = new RijndaelManaged();sa.KeySize=128;}
			else if (keyname.StartsWith("http://www.w3.org/2001/04/xmlenc#aes192-cbc")) {
			 sa = new RijndaelManaged();sa.KeySize=192;}
			else if (keyname.StartsWith("http://www.w3.org/2001/04/xmlenc#aes256-cbc")) {
			 sa = new RijndaelManaged();sa.KeySize=256;}
			else{throw(new Exception());}
			return sa;
		}
		public static SecretKey createSymKey(String keyname) {
			System.Security.Cryptography.SymmetricAlgorithm sa=Crypto.createSymmetricAlgorithm(keyname);
			sa.GenerateKey();
			return new SecretKey(sa.Key,keyname);
		} 

		public static javax.crypto.SecretKey createSymKey(byte[] data, String algorithm) {
			return new SecretKey(data, algorithm);
		}

		public static String toHex(byte[] b) {
			System.Text.StringBuilder s = new System.Text.StringBuilder();
			int j;
			for (int i = 0; i < b.Length; i++) {
				// hi-nibble
				j = (b[i] >> 4) & 0xf;
				s.Append(hex[j]);
				// lo-nibble
				j = b[i] & 0xf;
				s.Append(hex[j]);
			}
			return s.ToString();
		}

		public static byte[] doRSADecryption(SupportClass.PrivateKeySupport key, byte[] data) {
			if (data.Length <= 0) 
            {
				log.error("Es wurden keine Daten bergeben.");
				return null;
			}

//			RSACryptoServiceProvider rsacsp;
//			de.osci.osci12.dotcommon.RSAEncryption.GetRSACryptoServiceProviderPrivateKey(out rsacsp, key);
            RSA rsa = new de.osci.osci12.dotcommon.RSA_Cipher(key);
            rsa.DecryptValue(data);
			byte[] cipheredData = rsa.DecryptValue(data);
			return cipheredData;
		}
		
		public static byte[] doRSAEncryption(java.security.cert.X509Certificate encryptionCert, byte[] data) 
        {
//            RSACryptoServiceProvider rsacsp;
//            de.osci.osci12.dotcommon.RSAEncryption.GetRSACryptoServiceProviderPublicKey(out rsacsp, encryptionCert);
            RSA rsa = new de.osci.osci12.dotcommon.RSA_Cipher(encryptionCert);
            byte[] cipheredData = rsa.EncryptValue(data);
			return cipheredData;
		}

		public static System.String createMIMEBoundary() {
			byte[] boundery = new byte[20];
			SupportClass.SecureRandomSupport.getInstance("SHA1PRNG").nextBytes(boundery);
			return de.osci.helper.Base64.encode(boundery);
		}

    }
}