namespace de.osci.osci12.encryption {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	
	// <p><b>de.osci.osci12.messageparts.EncryptedDataOSCI</b></p>
	// <p>Die EncryptedData-Klasse stellt einen Datencontainer fr verschlsselte
	// Daten in einer OSCI-Nachricht dar. Ein EncryptedData-Objekt wird in ein
	// ContentContainer-Objekt eingestellt und  kann selbst entweder ein Attachment-
	// Objekt enthalten (in Form einer Referenz auf ein verschlsseltes Attachment,
	// welches in dem Parent-Nachrichtenobjekt liegt) oder aber wiederum ein
	// ContentContainer- oder EncryptedData-Objekt.</p>
	/// <exclude/>
	/// <summary>
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class EncryptedDataBuilder:DefaultHandler 
	{

		private void  InitBlock() {
			transformer = new System.Collections.ArrayList();
			keyInfoStack = new System.Collections.Stack();
			cipherDataStack = new System.Collections.Stack();
		}

		virtual public EncryptedData EncryptedData {
			get {
				return encrytedDataObject;
			}
		}

		private static Log log;
		internal DefaultHandler parentHandler;
		private System.Collections.ArrayList transformer;
		
		internal XMLReader xmlReader;
		internal System.Collections.Stack keyInfoStack;
		internal System.Collections.Stack cipherDataStack;
		private EncryptedData encrytedDataObject;
		private System.String edId = "";
		private EncryptedKey encKey = null;
		private System.String cipherRefId = "";
		private System.String currentElement = null;
		protected  static String DS_XMLNS = "http://www.w3.org/2000/09/xmldsig#"; 
		protected static String XENC_XMLNS = "http://www.w3.org/2001/04/xmlenc#"; 

		public EncryptedDataBuilder(XMLReader xmlReader) 
		{
			InitBlock();
			this.xmlReader = xmlReader;
			encrytedDataObject = new EncryptedData("http://www.w3.org/2001/04/xmlenc#tripledes-cbc", "");
		}
		
		public EncryptedDataBuilder(XMLReader xmlReader, DefaultHandler parentHandler, Attributes attributes) {
			InitBlock();
			this.xmlReader = xmlReader;
			this.parentHandler = parentHandler;
			encrytedDataObject = new EncryptedData("http://www.w3.org/2001/04/xmlenc#tripledes-cbc", "");
			if ((attributes != null) && (attributes.getValue("Id") != null)) {
				encrytedDataObject.setId(attributes.getValue("Id"));
			}
		}
		
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			log.debug("Start-Element: " + localName);
			if (localName.ToUpper().Equals("EncryptedData".ToUpper())&& uri.Equals(XENC_XMLNS)) {
				log.debug("encryptedData");
				if (encrytedDataObject == null) {
					encrytedDataObject = new EncryptedData("http://www.w3.org/2001/04/xmlenc#tripledes-cbc", edId);
				}
				log.debug("Konstruktor durchlaufen");
				if (attributes.getValue("Id") != null) {
					encrytedDataObject.setId(attributes.getValue("Id"));
				}
			}
			else if (localName.Equals("EncryptionMethod")&& uri.Equals(XENC_XMLNS)) {
				encrytedDataObject.EncryptionMethodAlgorithm=attributes.getValue("Algorithm");
			}
			else if (localName.Equals("KeyInfo")&& uri.Equals(DS_XMLNS)) {
				DefaultHandler childBuilder = new de.osci.osci12.signature.KeyInfoBuilder(xmlReader, this, attributes);
				xmlReader.setContentHandler(childBuilder);
			}
			else if (localName.Equals("CipherData")&& uri.Equals(XENC_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("CipherReference")&& uri.Equals(XENC_XMLNS)) {
				if (encKey == null) {
					cipherRefId = attributes.getValue("URI");
				}
			}
			else if (localName.Equals("Transforms")&& uri.Equals(XENC_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("Transform")&& uri.Equals(DS_XMLNS)) {
				transformer.Add(attributes.getValue("Algorithm"));
			}
			else if (localName.Equals("CipherValue")&& uri.Equals(XENC_XMLNS)) {
				currentElement = "";
			}
			else {
				throw new org.xml.sax.SAXException("Unerwartetes Element im EncryptedData-Builder: " + localName);
			}
		}
		
		public override void  characters(char[] ch, int start, int length) {
			log.debug("Character: " + new System.String(ch, start, length));
			if (currentElement == null) {
				for (int i = 0; i < length; i++) {
					if (ch[start + i] > ' ')
						throw new SAXException(de.osci.osci12.common.DialogHandler.text.getString("unexpected_char"));
				}
			}
			else currentElement += new System.String(ch, start, length);
		}
		
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("Ende-Element: " + localName);
			if (localName.ToUpper().Equals("EncryptedData".ToUpper()) && uri.Equals(XENC_XMLNS)) {
				xmlReader.setContentHandler(parentHandler);
				parentHandler.endElement(uri, localName, qName);
			}
			else if (localName.Equals("EncryptionMethod")&& uri.Equals(XENC_XMLNS)) {
				; // nothing  to do
			}
			else if (localName.Equals("CipherData")&& uri.Equals(XENC_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("EncryptedKey")&& uri.Equals(XENC_XMLNS)) {
				;
			}
				// nothing to do
			else if (localName.Equals("CipherData")&& uri.Equals(XENC_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("CipherReference")&& uri.Equals(XENC_XMLNS)) {
				CipherReference cr = new CipherReference(cipherRefId);
				System.Collections.IEnumerator enum_Renamed = transformer.GetEnumerator();
				while (enum_Renamed.MoveNext()) {
					cr.addTransform((System.String) enum_Renamed.Current);
				}
				encrytedDataObject.CipherData = new CipherData(cr);
			}
			else if (localName.Equals("Transforms")&& uri.Equals(XENC_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("Transform")&& uri.Equals(DS_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("CipherValue")&& uri.Equals(XENC_XMLNS)) {
				CipherValue cv = new CipherValue(currentElement);
				encrytedDataObject.CipherData = new CipherData(cv);
				cv.stateOfObject = CipherValue.STATE_ENCRYPTED;
			}
			else {
				throw new org.xml.sax.SAXException("Unerwartetes Element im EncryptedData: " + localName);
			}
			currentElement = null;
		}
		
		static EncryptedDataBuilder() {
			log = LogFactory.getLog(typeof(EncryptedDataBuilder));
		}
	}
}