namespace de.osci.osci12.encryption {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	using de.osci.osci12.common;
	using de.osci.osci12.signature;
	
	/// <exclude/>
	/// <summary>
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class EncryptedKeyBuilder:DefaultHandler 
	{
		protected  static String DS_XMLNS = "http://www.w3.org/2000/09/xmldsig#"; 
		protected  static String XENC_XMLNS = "http://www.w3.org/2001/04/xmlenc#"; 

		private void  InitBlock() 
		{
			transformer = new System.Collections.ArrayList();
		}

		virtual public EncryptedKey EncKey {
			get {
				return encKey;
			}	
		}

		private static Log log;
		internal EncryptedKey encKey = null;
		private System.Collections.ArrayList transformer;
		private CipherReference cipherRef = null;
		private CipherValue cipherValue = null;
		private CipherData cipherData = null;
		private System.String currentElement = null;
		private de.osci.osci12.signature.KeyInfoBuilder parentHandler = null;
		private XMLReader xmlReader = null;

		public EncryptedKeyBuilder(XMLReader xmlReader, DefaultHandler parentHandler, Attributes attributes) {
			InitBlock();
			if (parentHandler is KeyInfoBuilder) {
				this.parentHandler = ((KeyInfoBuilder) parentHandler);
			}
			else {
				throw new SAXException("Encrypted Data darf nur als Unterelement von KeyInfo auftreten.");
			}
			this.xmlReader = xmlReader;
			encKey = new EncryptedKey(Constants.ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5);
			if (attributes != null) {
				if (attributes.getValue("Recipient") != null) {
					encKey.Recipient=attributes.getValue("Recipient");
				}
				if (attributes.getValue("Id") != null) {
					encKey.Id=attributes.getValue("Id");
				}
				if (attributes.getValue("Type") != null) {
					encKey.Type=attributes.getValue("Type");
				}
				if (attributes.getValue("MimeType") != null) {
					encKey.MimeType=attributes.getValue("MimeType");
				}
				if (attributes.getValue("Encoding") != null) {
					encKey.Encoding=attributes.getValue("Encoding");
				}
			}
		}
		
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			log.debug("Start-Element: " + localName);
			if (localName.Equals("EncryptedKey")&& uri.Equals(XENC_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("KeyInfo")&& uri.Equals(DS_XMLNS)) {
				xmlReader.setContentHandler(new KeyInfoBuilder(xmlReader, this, attributes));
			}
			else if (localName.Equals("EncryptionMethod")&& uri.Equals(XENC_XMLNS)) {
				encKey.EncryptionMethodAlgorithm=attributes.getValue("Algorithm");
			}
			else if (localName.Equals("CipherData")&& uri.Equals(XENC_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("CipherValue")&& uri.Equals(XENC_XMLNS)) {
				currentElement = "";
			}
			else if (localName.Equals("CipherReference")&& uri.Equals(XENC_XMLNS)) {
				String cp = attributes.getValue("URI");
				cipherRef = new CipherReference(cp); 
			}
			else if (localName.Equals("Transforms")&& uri.Equals(DS_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("Transform")&& uri.Equals(DS_XMLNS)) {
				if (attributes.getValue("Algorithm") == null) {
					throw new SAXException("Das Attribute Algorithm ist beim Element 'Transform' erforderlich.");
				}
				transformer.Add(attributes.getValue("Algorithm"));
			}
			else if (localName.Equals("EncryptionProperties")&& uri.Equals(XENC_XMLNS)) {
				throw new SAXException("Element EncryptionProperties wird nicht untersttzt");
			}
			else if (localName.Equals("RefernceList")&& uri.Equals(XENC_XMLNS)) {
				throw new SAXException("Element RefernceList wird nicht untersttzt");
			}
			else if (localName.Equals("CarriedKeyName")&& uri.Equals(XENC_XMLNS)) {
				currentElement = "";
			}
			else {
				throw new org.xml.sax.SAXException("Unerwartetes Element im EncryptedKey-Builde: " + localName);
			}
		}
		
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("End-Element: " + localName);
			if (localName.Equals("EncryptedKey") && uri.Equals(XENC_XMLNS)) {
				parentHandler.KeyInfo.addEncryptedKey(encKey);
				xmlReader.setContentHandler(parentHandler);
			}
			else if (localName.Equals("KeyInfo")&& uri.Equals(DS_XMLNS)) {
				;
			}
			else if (localName.Equals("CipherData")&& uri.Equals(XENC_XMLNS)) {
				if (cipherRef != null) {
					cipherData = new CipherData(cipherRef);
				}
				else {
					cipherData = new CipherData(cipherValue);
					cipherValue.stateOfObject = CipherValue.STATE_ENCRYPTED;
				}
				encKey.CipherData = cipherData;
			}
			else if (localName.Equals("CipherReference")&& uri.Equals(XENC_XMLNS)) {
				System.Collections.IEnumerator transformers  = transformer.GetEnumerator();
				while (transformers.MoveNext()) {
					cipherRef.addTransform((System.String) transformers.Current);
				}
				transformer.Clear();
			}
			else if (localName.Equals("Transforms")&& uri.Equals(XENC_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("Transform")&& uri.Equals(DS_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("CipherValue")&& uri.Equals(XENC_XMLNS)) {
				cipherValue = new CipherValue(currentElement);
			}
			else if (localName.Equals("EncryptionMethod")&& uri.Equals(XENC_XMLNS)) {
				; // not supported
			}
			else if (localName.Equals("EncryptionProperties")&& uri.Equals(XENC_XMLNS)) {
				; // not supported
			}
			else if (localName.Equals("RefernceList")&& uri.Equals(XENC_XMLNS)) {
				; // not supported
			}
			else if (localName.Equals("CarriedKeyName")&& uri.Equals(XENC_XMLNS)) {
				encKey.CarriedKeyName = currentElement;
			}
			else {
				throw new org.xml.sax.SAXException("Unerwartetes Element im EncryptedKey-Builder: " + localName);
			}
			currentElement = null;
		}
		
		public override void  characters(char[] ch, int start, int length) {
			log.debug("Character: " + new System.String(ch, start, length));
			if (currentElement == null) {
				for (int i = 0; i < length; i++) {
					if (ch[start + i] > ' ')
						throw new SAXException(DialogHandler.text.getString("unexpected_char"));
				}
			}
			else currentElement += new System.String(ch, start, length);
		}

		static EncryptedKeyBuilder() {
			log = LogFactory.getLog(typeof(EncryptedKeyBuilder));
		}
	}
}