namespace de.osci.osci12.extinterfaces.crypto {

	using System;
	using java.security.cert;
	using de.osci.osci12.encryption;

	/// <summary>Diese abstrakte Klasse stellt die Schnittstelle der Bibliothek fr
	/// die Anbindung von Entschlsselungs-Modulen (Crypto-Token) dar. Anwendungen, die
	/// OSCI-Nachrichten entschlsseln wollen, mssen an das entschlsselnde Rollenobjekt
	/// eine Implementation dieser Klasse bergeben.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public abstract class Decrypter {

		/// <summary> Liefert die Versionsnummer.
		/// </summary>
		/// <value> Versionsnummer
		/// </value>
		public abstract System.String Version {
			get;
		}

		/// <summary> Liefert den Namen des Herstellers.
		/// </summary>
		/// <value> Herstellername
		/// </value>
		public abstract System.String Vendor {
			get;
		}

		/// <summary> Die Implementierung dieser Methode muss das Verschlsselungszertifikat zurckgeben.
		/// </summary>
		/// <value> Verschlsselungszertifikat
		/// </value>
		public abstract X509Certificate Certificate {
			get;
		}
		
		/// <summary> Die Implementierung dieser Methode muss das bergebene Byte-Array entschlsseln.
		/// </summary>
		/// <param name="data">Zu entschlsselndes Byte-Array.
		/// </param>
		/// <returns> Entschlsseltes Byte-Array.
		/// </returns>
		public abstract byte[] decrypt(byte[] data);
	}
}