namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	using de.osci.osci12.soapheader;

	/// <summary> Diese Klasse stellt einen OSCI-Body (soap:Body) dar. 
	/// Sie muss von Anwendungen nie direkt verwendet werden.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	// Body ist nicht wirklich ein soapheader, aber es ist ganz praktisch...
	public class Body:HeaderEntry {

		/// <summary> Liefert die ID im soap-Dokument des Bodys (hier momentan statisch 'soap:Body')
		/// </summary>
		/// <value> 'soap:Body'
		/// </value>
		override public System.String RefID {
			get {
				return id;
			}
		}

		new private static Log log;
		internal System.String data;
		private ContentContainer[] cc;
		private EncryptedDataOSCI[] encData;

		/// <summary> Setzt den Body initial mit einem selber vorbereiteten String.
		/// </summary>
		/// <param name="data">Inhalt
		/// </param>
		public Body(System.String data) {
			this.data = data;
			id = "body";
		}

		/// <summary>Legt ein Body-Objekt mit Inhaltsdaten an.
		/// </summary>
		/// <param name="cc">ContentContainer mit unverschlsselten Inhaltsdaten.
		/// </param>
		/// <param name="encData">EncryptedData mit unverschlsselten Inhaltsdaten.
		/// </param> 
		public Body(ContentContainer[] cc, EncryptedDataOSCI[] encData) {
			this.cc = cc;
			this.encData = encData;
			id = "body";
			System.Collections.Hashtable nsp = new System.Collections.Hashtable(); 
			nsp.Add(osciNSPrefix, MessagePartParser.OSCI_XMLNS); 
			nsp.Add(soapNSPrefix, MessagePartParser.SOAP_XMLNS); 
			nsp.Add(dsNSPrefix, MessagePartParser.DS_XMLNS); 
			nsp.Add(xencNSPrefix, MessagePartParser.XENC_XMLNS); 
			nsp.Add(xsiNSPrefix, MessagePartParser.XSI_XMLNS); 
    
			for (int i = 0; i < cc.Length; i++) 
			{ 
				if (!nsp.Contains(cc[i].osciNSPrefix)) nsp.Add(cc[i].osciNSPrefix, MessagePartParser.OSCI_XMLNS); 
				if (!nsp.Contains(cc[i].osciNSPrefix)) nsp.Add(cc[i].soapNSPrefix, MessagePartParser.SOAP_XMLNS); 
				if (!nsp.Contains(cc[i].osciNSPrefix)) nsp.Add(cc[i].dsNSPrefix, MessagePartParser.DS_XMLNS); 
				if (!nsp.Contains(cc[i].osciNSPrefix)) nsp.Add(cc[i].xencNSPrefix, MessagePartParser.XENC_XMLNS); 
				if (!nsp.Contains(cc[i].osciNSPrefix)) nsp.Add(cc[i].xsiNSPrefix, MessagePartParser.XSI_XMLNS); 
			} 
    
			for (int i = 0; i < encData.Length; i++) 
			{ 
				if (!nsp.Contains(encData[i].osciNSPrefix)) nsp.Add(encData[i].osciNSPrefix, MessagePartParser.OSCI_XMLNS); 
				if (!nsp.Contains(encData[i].osciNSPrefix)) nsp.Add(encData[i].soapNSPrefix, MessagePartParser.SOAP_XMLNS); 
				if (!nsp.Contains(encData[i].osciNSPrefix)) nsp.Add(encData[i].dsNSPrefix, MessagePartParser.DS_XMLNS); 
				if (!nsp.Contains(encData[i].osciNSPrefix)) nsp.Add(encData[i].xencNSPrefix, MessagePartParser.XENC_XMLNS); 
				if (!nsp.Contains(encData[i].osciNSPrefix)) nsp.Add(encData[i].xsiNSPrefix, MessagePartParser.XSI_XMLNS); 
			} 
    
			System.Collections.ArrayList keySet = new System.Collections.ArrayList(nsp.Keys); 
			keySet.Sort();
			System.Text.StringBuilder sb = new System.Text.StringBuilder(); 
			for (int i = 0; i < keySet.Count; i++) 
			{ 
				sb.Append(" xmlns:"); 
				sb.Append(keySet[i]); 
				sb.Append("=\""); 
				sb.Append(nsp[keySet[i]]); 
				sb.Append("\""); 
			} 
			ns=MoreSupport.ToByteArray(sb.ToString(),Constants.CHAR_ENCODING); 
			log.debug("NS: "+sb.ToString()); 
		}
		
		/// <summary> Diese Methode wird intern aufgerufen.
		/// Es wird der eingestellte Body serialisiert.
		/// </summary>
		/// <param name="out_Renamed">Outputstream, in den geschrieben werden soll
		/// </param>
		public override void  writeXML(System.IO.Stream out_Renamed) {
			if (tmpBuffer != null) {
				byte[] temp_byteArray;
				temp_byteArray = tmpBuffer;
				out_Renamed.Write(temp_byteArray, 0, temp_byteArray.Length);;;;
			} else {
				MoreSupport.ToStream(out_Renamed,"<"+soapNSPrefix+":Body",Constants.CHAR_ENCODING);
				out_Renamed.Write(ns,0,ns.Length);
				MoreSupport.ToStream(out_Renamed," Id=\"" + id + "\">",Constants.CHAR_ENCODING);

				if (data != null) {
					MoreSupport.ToStream(out_Renamed,data,Constants.CHAR_ENCODING);
				} else {
					MoreSupport.ToStream(out_Renamed,"<"+osciNSPrefix+":ContentPackage>",Constants.CHAR_ENCODING);
					for (int i = 0; i < encData.Length; i++) 
					{
						encData[i].writeXML(out_Renamed, false);
					}
					for (int i = 0; i < cc.Length; i++) 
					{
						cc[i].writeXML(out_Renamed);
					}
					MoreSupport.ToStream(out_Renamed,"</"+osciNSPrefix+":ContentPackage>",Constants.CHAR_ENCODING);
				}
				MoreSupport.ToStream(out_Renamed,"</"+soapNSPrefix+":Body>",Constants.CHAR_ENCODING);
			}
		}

		static Body() {
			log = LogFactory.getLog(typeof(Body));
		}
	}
}