namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	using de.osci.osci12.common;
	using de.osci.osci12.messagetypes;
	using de.osci.helper;
	/// <summary> <H4>ContentContainer-Parser</H4> 
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class ContentContainerBuilder:MessagePartParser {

		/// <summary> Gets the contentContainer attribute of the ContentContainerBuilder object
		/// </summary>
		/// <value> The contentContainer value
		/// </value>
		virtual public ContentContainer ContentContainer {
			get {
				return coco;
			}
		}

		private static Log log;
		private bool readContent = false;
		private bool underlayingCoco = false;
		private ContentContainerBuilder cocoBuilder = null;
		private de.osci.osci12.extinterfaces.OSCIDataSource swapBuffer = null;
		private Attachment att = null;
		private System.String contentID = null;
		private System.String cocoId = null;
		private ContentContainer coco;
		private de.osci.osci12.encryption.EncryptedDataBuilder encDataBuilder = null;
		private OSCISignatureBuilder sigBuilder;
		private System.IO.StreamWriter osw;
		private System.IO.Stream b64;
		// Canonizer, aus dem der Parser liest 
		internal Canonizer can; 
		internal String cocoNS; 
		internal String soapNSPrefix, osciNSPrefix, dsNSPrefix, xencNSPrefix, xsiNSPrefix;	
		/// <summary>  Constructor for the ContentContainerBuilder object
		/// </summary>
		/// <param name="xmlReader">
		/// </param>
		/// <param name="parentHandler">
		/// </param>
		/// <param name="parentMessage">
		/// </param>
		/// <param name="atts">
		/// </param>
		public ContentContainerBuilder(XMLReader xmlReader, DefaultHandler parentHandler, OSCIMessage parentMessage, Attributes atts,Canonizer can):base(xmlReader, parentHandler) {
			msg = parentMessage;
			coco = new ContentContainer();
			cocoNS = (String)can.ContainerNS[0]; 
			coco.ns = MoreSupport.ToByteArray(cocoNS,Constants.CHAR_ENCODING); 
			String []ns=cocoNS.Substring(1).Split(" =".ToCharArray());
			String prefix, uri; 
			for(int i=0;i<ns.Length;i+=2)
			{ 
				prefix = ns[i];
				prefix = prefix.Substring(prefix.IndexOf(":")+1); 
				uri = ns[i+1]; 
				uri = uri.Substring(1, uri.Length-2); 
				if (uri.Equals(SOAP_XMLNS)) 
					soapNSPrefix = prefix; 
				else if (uri.Equals(OSCI_XMLNS)) 
					osciNSPrefix = prefix; 
				else if (uri.Equals(DS_XMLNS)) 
					dsNSPrefix = prefix; 
				else if (uri.Equals(XENC_XMLNS)) 
					xencNSPrefix = prefix; 
				else if (uri.Equals(XSI_XMLNS)) 
					xsiNSPrefix = prefix; 
			} 
			coco.setNSPrefixes(soapNSPrefix, osciNSPrefix, dsNSPrefix, xencNSPrefix, xsiNSPrefix);
			can.ContainerNS.RemoveAt(0); 
			coco.stateOfObject = ContentContainer.STATE_OF_OBJECT_PARSING; 
			if (atts.getValue("Id") != null) {
				coco.RefID=atts.getValue("Id");
			}
			log.debug("coco ID: " + atts.getValue("Id"));
			this.can = can;
		}
		
		/// <summary>
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <param name="attributes">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			log.debug("start-Element: " + qName);
 			if (localName.Equals("Signature") && uri.Equals(DS_XMLNS)) {
				sigBuilder = new OSCISignatureBuilder(this.xmlReader, this, attributes);
				sigBuilder.sig.setNSPrefixes(soapNSPrefix, osciNSPrefix, dsNSPrefix, xencNSPrefix, xsiNSPrefix);
				this.xmlReader.setContentHandler(sigBuilder);
			}
			else if (localName.Equals("Content") && uri.Equals(OSCI_XMLNS)) {
				contentID = attributes.getValue("Id");
				if (attributes.getValue("href") != null) {
					String href = attributes.getValue("href").Substring(4); 
					if (msg.attachments.ContainsKey(href)) {
						att = (Attachment) msg.attachments[href];
						if (log.DebugEnabled)log.debug("########## Attachment gefunden. ");
					}
					else {
						if (log.DebugEnabled)
							log.debug("Attchment wird der Nachricht hinzugef\u00FCgt.");

						try 
                        {
                            if (msg.isSigned())
                                att = new Attachment(null, href, 0, (String)msg.signatureHeader.getDigestMethods()["cid:" + href]);
                            else
							    att = new Attachment(null, href, 0, null);
						}
						catch (Exception ex1) {
							throw new SAXException(ex1);
						}
						SupportClass.PutElement(msg.attachments,att.RefID, att);
						att.stateOfAttachment = Attachment.STATE_OF_ATTACHMENT_PARSING;
					}
				}
			} 
			else if (localName.Equals("Base64Content") && uri.Equals(OSCI_XMLNS)){ 
				contentID = attributes.getValue("Id"); 
				readContent = true;
				try { 
					swapBuffer = de.osci.osci12.common.DialogHandler.NewDataBuffer;
					osw = new System.IO.StreamWriter(swapBuffer.OutputStream);
				} 
				catch (Exception ex) { 
					throw new SAXException(ex); 
				} 
			}
 			   else if (localName.ToUpper().Equals("EncryptedData".ToUpper())&& uri.Equals(XENC_XMLNS)) {
				if (encDataBuilder != null) {
					log.debug("Encrypted-Data wird hinzugefgt.");
					EncryptedDataOSCI enc = new EncryptedDataOSCI(encDataBuilder.EncryptedData, msg);
					enc.NS=cocoNS; 
					enc.setNSPrefixes(soapNSPrefix, osciNSPrefix, dsNSPrefix, xencNSPrefix, xsiNSPrefix);
					coco.addEncryptedData(enc); 
				}
				encDataBuilder = new de.osci.osci12.encryption.EncryptedDataBuilder(this.xmlReader, this, attributes);
				this.xmlReader.setContentHandler(encDataBuilder);
			}
			else if (localName.Equals("ContentContainer")&& uri.Equals(OSCI_XMLNS)) {
				readContent = false;
				underlayingCoco = true;
				cocoBuilder = new ContentContainerBuilder(this.xmlReader, this, msg, attributes, can);
				this.xmlReader.setContentHandler(cocoBuilder);
			}
			else {
				throw new org.xml.sax.SAXException("Unerwartetes Element im ContentContainer: " + localName);
			}
		}
		
		/// <summary> 
		/// </summary>
		/// <param name="parm1">
		/// </param>
		/// <param name="parm2">
		/// </param>
		/// <param name="parm3">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  characters(char[] ch, int start, int length) {
			log.debug("readContent: " + readContent + " " + new System.String(ch,start,length));
			if (readContent) {
				osw.Write(new System.String(ch,start,length));
			}
		   else
			{
				for (int i = 0; i < length; i++)
				{
				if (ch[start + i] > ' ')
					throw new SAXException(DialogHandler.text.getString("unexpected_char"));
				}
			}
		}
		
		/// <summary>  
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("End-Element: " + qName);
			// Ein Content Element wird beendet und dem coco zugefgt
			if (localName.Equals("Content")&& uri.Equals(OSCI_XMLNS)) {
				if (att != null) {
					Content co = new Content(att);
					co.transformers[0]= "<" + dsNSPrefix + ":Transform Algorithm=\"" + Constants.TRANSFORM_CANONICALIZATION + "\"></" + dsNSPrefix + ":Transform>";
					co.coNS = cocoNS;
					co.setNSPrefixes(soapNSPrefix, osciNSPrefix, dsNSPrefix, xencNSPrefix, xsiNSPrefix);
					co.RefID = contentID;
					log.debug("RefID: " + contentID);
					coco.addContent(co);
					att = null;
				}
			}
			else if (localName.Equals("Base64Content")&& uri.Equals(OSCI_XMLNS))
			{
				osw.Close();
				Content co = new Content(swapBuffer);
				co.coNS = cocoNS;
				co.setNSPrefixes(soapNSPrefix, osciNSPrefix, dsNSPrefix, xencNSPrefix, xsiNSPrefix);
				co.RefID = contentID;
				coco.addContent(co);
				readContent = false;
				swapBuffer = null;
			}
			else if (localName.Equals("ContentContainer")&& uri.Equals(OSCI_XMLNS)) {
				if (underlayingCoco) {
					ContentContainer newCoco = cocoBuilder.ContentContainer;
					Content tmpCnt = new Content(newCoco); 
					tmpCnt.transformers[0]= "<" + cocoBuilder.dsNSPrefix + ":Transform Algorithm=\"" +  Constants.TRANSFORM_CANONICALIZATION + "\"></" + cocoBuilder.dsNSPrefix + ":Transform>";
					Content cocoTmp = new Content(newCoco);
					cocoTmp.coNS = cocoBuilder.cocoNS;
					tmpCnt.RefID=contentID; 
					tmpCnt.setNSPrefixes(cocoBuilder.soapNSPrefix, cocoBuilder.osciNSPrefix, cocoBuilder.dsNSPrefix, cocoBuilder.xencNSPrefix, cocoBuilder.xsiNSPrefix); 
					this.coco.addContent(tmpCnt); 
					underlayingCoco = false;
				}
				else {
					// gibt es noch weitere encryptedData Elemente die dem coco noch nicht hinzugefgt wurden
					if (encDataBuilder != null) {
						log.debug("Encrypted-Data wird hinzugefgt.Parent Message: ");
						EncryptedDataOSCI enc = new EncryptedDataOSCI(encDataBuilder.EncryptedData, msg);
						enc.setNSPrefixes(soapNSPrefix, osciNSPrefix, dsNSPrefix, xencNSPrefix, xsiNSPrefix);
						enc.NS=cocoNS;  
						coco.addEncryptedData(enc); 
						encDataBuilder = null;
					}
					if (coco.signerList.Count > 0) {
						OSCISignatureReference[] refs = ((OSCISignature) coco.signerList[0]).getReferences();
						Content[] cnt = coco.Contents;
						String refKey;
						for (int i = 0; i < cnt.Length; i++) {
							if (cnt[i].ContentType==Content.ATTACHMENT_REFERENCE)
								refKey = "cid:" + cnt[i].RefID;
							else
								refKey = "#" + cnt[i].RefID;

							for (int j = 0; j < refs.Length; j++) {
								if (refKey.Equals(refs[j].RefID))
									cnt[i].transformers = refs[j].transformerAlgorithms;
							}
						}
					}
					// zurck zum Parent
						this.xmlReader.setContentHandler(this.parentHandler);
						log.debug("parentHandler: " + parentHandler);
						parentHandler.endElement(uri, localName, qName);
				}
			}
			else if (localName.Equals("EncryptedData")&& uri.Equals(XENC_XMLNS)) {
				if (encDataBuilder != null) {
					EncryptedDataOSCI enc = new EncryptedDataOSCI(encDataBuilder.EncryptedData, msg);
					enc.setNSPrefixes(soapNSPrefix, osciNSPrefix, dsNSPrefix, xencNSPrefix, xsiNSPrefix);
					enc.NS=cocoNS;  
					coco.addEncryptedData(enc); 
				}
				encDataBuilder = null;
			}
			else {
				log.debug("Name des Elemtes: " + localName);
				throw new SAXException("Nicht erwartetes Element im ContentContainer");
			}
		}
		
		static ContentContainerBuilder() {
			log = LogFactory.getLog(typeof(ContentContainerBuilder));
		}
	}
}