namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using de.osci.osci12.encryption;
	using de.osci.osci12.messagetypes;
	using de.osci.helper;

	/// <summary><H4>ContentPackage-Parser</H4>
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class ContentPackageBuilder:MessagePartParser {

		virtual public System.Object LastCreatedObject {
			get {
				return lastObject;
			}
		}
/*
        public static void setDigestValue(Content c, byte[] digest) { 
			c.digestValue = digest; 
		} 
*/  

		new private static Log log;
		private System.Object lastObject = null;
		private de.osci.osci12.encryption.EncryptedDataBuilder encDataBuilder = null;
		private ContentContainerBuilder cocoBuilder = null;
		private String soapNSPrefix, osciNSPrefix, dsNSPrefix, xencNSPrefix, xsiNSPrefix;
		// Canonizer, aus dem der Parser liest 
		private Canonizer can; 
		
		public ContentPackageBuilder(XMLReader reader, OSCIMessage msg, Canonizer can):base(reader,null) {
			log.debug("############## Konstruktor: " + msg.GetType());
			this.xmlReader = reader;
			this.msg = msg;
			this.can = can;
		}
		
		public ContentPackageBuilder(OSCIMessageBuilder parentBuilder):base(parentBuilder) {
			can = parentBuilder.CanStream;
		}
		
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) 
		{
			log.debug("Start-Element: " + qName);

			if (localName.Equals("ContentPackage")&& uri.Equals(OSCI_XMLNS)) 
			{
				; // nothing to do
			}
			else if (localName.Equals("EncryptedData")&& uri.Equals(XENC_XMLNS)) {
				encDataBuilder = new EncryptedDataBuilder(this.xmlReader, this, attributes);
				this.xmlReader.setContentHandler(encDataBuilder);
			}
			else if (localName.Equals("ContentContainer")&& uri.Equals(OSCI_XMLNS)) {
				cocoBuilder = new ContentContainerBuilder(this.xmlReader, this, msg, attributes, can);
				this.xmlReader.setContentHandler(cocoBuilder);
			}
			else {
				new org.xml.sax.SAXException("Nicht vorgesehens Element! Element Name:" + localName);
			}
		}
		
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("End-Element: " + qName);
			if (localName.Equals("ContentPackage")&& uri.Equals(OSCI_XMLNS)) {
				xmlReader.setContentHandler(parentHandler);
			}
			else if (localName.Equals("ContentContainer")&& uri.Equals(OSCI_XMLNS)) {
				lastObject = this.cocoBuilder.ContentContainer;
				if (parentHandler != null)
					SupportClass.PutElement(msg.contentContainer, ((ContentContainer) lastObject).RefID, (ContentContainer) lastObject);
			}
			else if (localName.Equals("EncryptedData")&& uri.Equals(XENC_XMLNS)) {
				log.debug("Encrypted-Data wird hinzugefgt.");
				lastObject = new EncryptedDataOSCI(encDataBuilder.EncryptedData, msg);
				((EncryptedDataOSCI)lastObject).RefID=encDataBuilder.EncryptedData.Id;
				if (parentHandler != null)
				SupportClass.PutElement(msg.encryptedData, ((EncryptedDataOSCI) lastObject).RefID, lastObject);
			}
			else {
				new org.xml.sax.SAXException("Nicht vorgesehens Element! Element Name:" + localName);
			}
		}
		
		static ContentPackageBuilder() {
			log = LogFactory.getLog(typeof(ContentPackageBuilder));
		}
	}
}