namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	using System.Text;
	using de.osci.osci12.common;

	/// <summary><H4>Feedback-Parser</H4>
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	

	class FeedbackBuilder : MessagePartParser {

		private static Log log = LogFactory.getLog(typeof(FeedbackBuilder));
		private System.Collections.ArrayList feedbacks = new System.Collections.ArrayList();
		private String[] entry;

		
		/// <summary> Constructor for the FeedbackBuilder object
		/// </summary>
		public FeedbackBuilder(de.osci.osci12.messagetypes.OSCIMessageBuilder parent):base(parent) {
	   ;}

		public override void startElement(String uri, String localName, String qName, Attributes attributes) {
			if (log.DebugEnabled)
				log.debug("Start Element in Feedback: " + localName);

			if (localName.Equals("Entry")&& uri.Equals(OSCI_XMLNS)) {
				entry = new String[3];
				entry[0] =attributes.getValue("xml:lang");
			}
			else if (localName.Equals("Code")&& uri.Equals(OSCI_XMLNS)) {
				currentElement = new  StringBuilder();
			}
			else if (localName.Equals("Text")&& uri.Equals(OSCI_XMLNS)) {
				currentElement = new  StringBuilder();
			}
			else {
				throw new SAXException(DialogHandler.text.getString("unexpected_entry")+": "+localName);
			}
		}

		public override void endElement(String uri, String localName, String qName) {
			if (log.DebugEnabled)
				log.debug("End Element in Feedback: " + localName);

			if (localName.Equals("Feedback")&& uri.Equals(OSCI_XMLNS)) {
				String[][] fd = new String[feedbacks.Count][];
				for (int i=0 ;i<feedbacks.Count;i++){
					fd[i] = new String[3];
					fd[i][0] = ((String[])feedbacks[i])[0];
					fd[i][1] = ((String[])feedbacks[i])[1];
					fd[i][2] = ((String[])feedbacks[i])[2];
				}
				((de.osci.osci12.messagetypes.OSCIResponseTo)msg).feedBack = fd;

				xmlReader.setContentHandler(parentHandler);
			}
			else if (localName.Equals("Entry")&& uri.Equals(OSCI_XMLNS)) {
				feedbacks.Add(entry);
			}
			else if (localName.Equals("Code")&& uri.Equals(OSCI_XMLNS)) {
				entry[1] = currentElement.ToString();
			}
			else if (localName.Equals("Text")&& uri.Equals(OSCI_XMLNS)) {
				entry[2] = currentElement.ToString();
			}
			else {
				throw new SAXException(DialogHandler.text.getString("unexpected_entry")+": "+localName);
			}

			currentElement = null;
		}
	}
}