namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	using System.Text;

	/// <summary><H4>Inspection-Parser</H4>
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class InspectionBuilder:MessagePartParser {

		virtual public Inspection InspectionObject {
			get {
				return inspection;
			}
			
		}

		private static Log log;
		internal Inspection inspection = null;
		private TimestampBuilder timestampBuilder = null;
		private bool isInsideOnlineResult = false;
		private System.Collections.ArrayList onlineChecks = new System.Collections.ArrayList(); 
		private System.Collections.ArrayList onlineCheckNames = new System.Collections.ArrayList(); 
    
		public InspectionBuilder(XMLReader xmLReader, DefaultHandler parenTHandler):base(xmLReader, parenTHandler)
		{	
			inspection = new Inspection();
			ProcessCardBundle pcb = ((ProcessCardBundleBuilder)parentHandler).processCard;
			inspection.setNSPrefixes(pcb.soapNSPrefix, pcb.osciNSPrefix, pcb.dsNSPrefix, pcb.xencNSPrefix, pcb.xsiNSPrefix);
		}
	

		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) 
		{
			log.debug("Start-Element: " + qName);
			if (localName.Equals("Timestamp") && uri.Equals(OSCI_XMLNS)) {
				timestampBuilder = new TimestampBuilder(xmlReader, this);
				xmlReader.setContentHandler(timestampBuilder);
			}
			else if (localName.Equals("X509IssuerName") && uri.Equals(OSCI_XMLNS)) {
				currentElement = new System.Text.StringBuilder();
			}
			else if (localName.Equals("X509SerialNumber") && uri.Equals(OSCI_XMLNS)) {
				currentElement = new System.Text.StringBuilder();
			}
			else if (localName.Equals("X509SubjectName") && uri.Equals(OSCI_XMLNS)) {
				currentElement = new System.Text.StringBuilder();
			}
			else if (localName.Equals("CertType") && uri.Equals(OSCI_XMLNS)) {
				inspection.CertType=attributes.getValue("Type");
			}
			else if (localName.Equals("MathResult") && uri.Equals(OSCI_XMLNS)) {
				inspection.MathResult=(attributes.getValue("Result")).ToLower();
				if (!inspection.mathResult.Equals("ok") && !inspection.mathResult.Equals("corrupted") && 
					!inspection.mathResult.Equals("indeterminate")) { 
					throw new dothelpers.IllegalArgumentException(); }
			}
			else if (localName.Equals("OfflineResult") && uri.Equals(OSCI_XMLNS)) {
				inspection.OfflineResult=(attributes.getValue("Result")).ToLower();
				if (!inspection.offlineResult.Equals("valid") && 
					!inspection.offlineResult.Equals("invalid") && 
					!inspection.offlineResult.Equals("indeterminate")) { 
					throw new dothelpers.IllegalArgumentException(); 
				} 
			}
			else if (localName.Equals("OnlineResult") && uri.Equals(OSCI_XMLNS)) {
				isInsideOnlineResult = true;
				inspection.isOnlineChecked = true;
				inspection.onlineResult=(attributes.getValue("Result").Equals("ok"));
			}
			else if (isInsideOnlineResult){
				if (localName.Equals("OCSP") && uri.Equals(OSCI_XMLNS)) {
					  currentElement = new System.Text.StringBuilder();
					onlineCheckNames.Add("OCSP");  
				  }
				  else if (localName.Equals("CRL") && uri.Equals(OSCI_XMLNS)) {
					  currentElement = new System.Text.StringBuilder();
					 onlineCheckNames.Add("CRL");
				  }
				  else if (localName.Equals("LDAP") && uri.Equals(OSCI_XMLNS)) {
					  currentElement = new System.Text.StringBuilder();
					 onlineCheckNames.Add("LDAP");
				  }
				  else {
					  throw new SAXException("Das Element " + localName + " ist im Content von Inspection nicht gltig");
				  }
			}
		}
		
		
		public override void endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("End-Element: " + qName);
			if (localName.Equals("Inspection") && uri.Equals(OSCI_XMLNS)) {
				if ((inspection.mathResult==null) || (inspection.offlineResult==null))
					throw new SAXException(de.osci.osci12.common.DialogHandler.text.getString("missing_entry") + ".");
				inspection.OnlineCheckNames=new string[onlineCheckNames.Count];
				for(int i=0 ;i<onlineCheckNames.Count;i++)inspection.OnlineCheckNames[i]=onlineCheckNames[i].ToString();
				inspection.OnlineChecks=new string[onlineChecks.Count];
				for(int i=0 ;i<onlineChecks.Count;i++)inspection.OnlineChecks[i]=onlineChecks[i].ToString();
				parentHandler.endElement(uri, localName, qName);
				xmlReader.setContentHandler(parentHandler);
			}
			else if (localName.Equals("Timestamp") && uri.Equals(OSCI_XMLNS)) {
				inspection.TimeStamp = timestampBuilder.TimestampObject;
				// hier sollte ein X509Certificate oder eine Ref auf ein bestehendes Zertifikat als Ersatz eingefgt werden
			}
			else if (localName.Equals("X509IssuerName") && uri.Equals(OSCI_XMLNS)) {
				inspection.X509IssuerName = currentElement.ToString();
			}
			else if (localName.Equals("X509SerialNumber") && uri.Equals(OSCI_XMLNS)) {
				inspection.X509SerialNumber = currentElement.ToString();
			}
			else if (localName.Equals("X509SubjectName") && uri.Equals(OSCI_XMLNS)) { 
				inspection.X509SubjectName=currentElement.ToString(); }
			else if (localName.Equals("CertType")&& uri.Equals(OSCI_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("MathResult") && uri.Equals(OSCI_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("OfflineResult") && uri.Equals(OSCI_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("OnlineResult") && uri.Equals(OSCI_XMLNS)) {
				// dieser gilt fr OCSP und CRL
				isInsideOnlineResult = false;
			}
			else if (isInsideOnlineResult && ((localName.Equals("OCSP") && uri.Equals(OSCI_XMLNS))
				|| (localName.Equals("CRL") && uri.Equals(OSCI_XMLNS)) || (localName.Equals("LDAP") && uri.Equals(OSCI_XMLNS)))) {
				onlineChecks.Add(currentElement.ToString());
			}
			else {
				throw new SAXException("Das Element " + localName + " ist im Content von Inspection nicht gltig");
			}
			currentElement = null;
		}
		
		static InspectionBuilder() {
			log = LogFactory.getLog(typeof(InspectionBuilder));
		}
	}
}