namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	using de.osci.osci12.messagetypes;

	/// <summary> Interne Klasse, wird von Anwendungen nicht direkt bentigt.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class MessagePartParser:DefaultHandler {

		private static Log log;
		protected internal DefaultHandler parentHandler = null;
		protected internal XMLReader xmlReader = null;
		protected internal System.Text.StringBuilder currentElement;
		protected internal OSCIMessage msg = null;
   
		protected internal static String SOAP_XMLNS = "http://schemas.xmlsoap.org/soap/envelope/"; 
		protected internal static String OSCI_XMLNS = "http://www.osci.de/2002/04/osci"; 
		protected internal static String DS_XMLNS = "http://www.w3.org/2000/09/xmldsig#"; 
		protected internal static String XENC_XMLNS = "http://www.w3.org/2001/04/xmlenc#"; 
		protected internal static String XSI_XMLNS = "http://www.w3.org/2001/XMLSchema-instance";
        protected internal static String XADES_XMLNS = "http://uri.etsi.org/01903/v1.3.2#";

		/// <summary> Dieser Konstruktur ist nur fr den Fall das ein Encrypted ContentContainer
		/// decrypted wird, er wird nur von ContentPackageBuilder aufgerufen
		/// </summary>
		protected internal MessagePartParser() 
		{
			// nothing to do
		}
		
		protected internal MessagePartParser(OSCIMessageBuilder parentHandler) {
			
			this.parentHandler = parentHandler;
			msg = parentHandler.OSCIMessage;
			this.xmlReader = parentHandler.parentBuilder.XmlReader;
		}
		protected MessagePartParser(XMLReader xmlReader, DefaultHandler parentHandler) {
			this.parentHandler = parentHandler;
			this.xmlReader = xmlReader;
		}
		
		/// <summary>
		/// </summary>
		/// <param name="ch">
		/// </param>
		/// <param name="start">
		/// </param>
		/// <param name="length">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  characters(char[] ch, int start, int length) {
			log.debug("Character: " + new System.String(ch, start, length));
			if (currentElement == null) {
				for (int i = 0; i < length; i++) {
					if (ch[start + i] > ' ')
						throw new SAXException(de.osci.osci12.common.DialogHandler.text.getString("unexpected_char"));
				}
			}
			else currentElement.Append(ch, start, length);
		}

		static MessagePartParser() {
			log = LogFactory.getLog(typeof(MessagePartParser));
		}
	}
}