namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	using de.osci.osci12.common;
	using de.osci.osci12.messagetypes;
//	using de.osci.osci12.roles;
	using de.osci.helper; 
	using de.osci.osci12.common; 
	using de.osci.osci12.messagetypes; 
	using de.osci.osci12.roles; 
	using de.osci.osci12.signature; 
  

	/// <summary><H4>Signature-Parser</H4>
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	// Transformationen werden hier nicht untersttzt
	public class OSCISignatureBuilder:MessagePartParser {

		private static Log log;
		internal OSCISignature sig;
		private OSCISignatureReference sr;
        private String signingTime;
        private String signingPropsId;
        //		private SupportClass.MessageDigestSupport messageDigest;
		
		/// <summary> Constructor for the OSCISignatureBuilder object
		/// </summary>
		/// <param name="xmlReader">
		/// </param>
		/// <param name="parentHandler">
		/// </param>
		/// <param name="atts">
		/// </param>
		public OSCISignatureBuilder(XMLReader xmlReader, DefaultHandler parentHandler, Attributes atts):base(xmlReader, parentHandler) {
			if (parentHandler is OSCIMessageBuilder) {
				OSCIMessage msg = ((OSCIMessageBuilder)parentHandler).OSCIMessage;
				String name = "<" + msg.osciNSPrefix +":";     
				String soap = msg.soapNSPrefix; 
				name += ((((de.osci.osci12.messagetypes.OSCIMessageBuilder) parentHandler).OSCIMessage is OSCIRequest)?"ClientSignature Id=\"":"SupplierSignature Id=\"");
				name += (atts.getValue("Id") + "\" ");
				name += soap + ":actor=\"http://schemas.xmlsoap.org/soap/actor/next\" " + soap +":mustUnderstand=\"1\">";
				sig = new OSCISignature(name);
				sig.setNSPrefixes(soap, msg.osciNSPrefix, msg.dsNSPrefix, msg.xencNSPrefix, msg.xsiNSPrefix);
			}
			else {
				sig = new OSCISignature();
			}
		}
		
		/// <summary> 
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <param name="attributes">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) 
        {

			log.debug("Start Element" + localName);
            if (uri.Equals(DS_XMLNS))
            {

                if (localName.Equals("Signature") 
                    || localName.Equals("SignedInfo")
                    || localName.Equals("CanonicalizationMethod")
                    || localName.Equals("Object"))
                {
                    ;
                }
                else if (localName.Equals("DigestMethod"))
                {
                    sr.DigestMethodAlgorithm = attributes.getValue("Algorithm");
                }
                else if (localName.Equals("SignatureValue"))
                {
                    currentElement = new System.Text.StringBuilder();
                }
                else if (localName.Equals("KeyInfo"))
                {
                    currentElement = new System.Text.StringBuilder();
                }
                else if (localName.Equals("SignatureMethod"))
                {
                    sig.signatureAlgorithm = attributes.getValue("Algorithm");
                    switch (attributes.getValue("Algorithm"))
                    {
                        case Constants.SIGNATURE_ALGORITHM_RSA_SHA1:
                        case Constants.SIGNATURE_ALGORITHM_RSA_SHA256:
                        case Constants.SIGNATURE_ALGORITHM_RSA_SHA512:
                        case Constants.SIGNATURE_ALGORITHM_RSA_RIPEMD160:
                        case Constants.SIGNATURE_ALGORITHM_ECDSA_SHA256:
                        case Constants.SIGNATURE_ALGORITHM_ECDSA_SHA512:
                            break;
                        default:
                            throw new SAXException(DialogHandler.text.getString("invalid_signature_algorithm") + " " +
                                attributes.getValue("Algorithm"));
                            break;
                    }
                }
                else if (localName.Equals("Reference"))
                {
                    sr = new OSCISignatureReference();
                    String rf = attributes.getValue("URI");
                    sr.RefID = rf;
                    log.debug("############ Anzahl refs Start:" + sig.refs.Count);
                    sig.refs.Add(sr);
                }
                else if (localName.Equals("Transforms"))
                {
                    // do nothing
                }
                else if (localName.Equals("Transform"))
                {
                    try
                    {
                        de.osci.helper.CanParser cp = new de.osci.helper.CanParser(
                            sr.transformerAlgorithms, xmlReader, this, qName);
                        xmlReader.setContentHandler(cp);
                        cp.startDocument();
                        cp.startElement(uri, localName, qName, attributes);
                    }
                    catch (Exception ex)
                    {
                        throw new SAXException(DialogHandler.text.getString("sax_exception_transfomation"), ex);
                    }
                }
                else if (localName.Equals("DigestMethod "))
                {
                    sr.DigestMethodAlgorithm = attributes.getValue("Algorithm");
                }
                else if (localName.Equals("DigestValue") || localName.Equals("SignatureValue"))
                {
                    currentElement = new System.Text.StringBuilder();
                }
                else if (localName.Equals("RetrievalMethod"))
                {
                    {
                        String rf = attributes.getValue("URI");
                        sig.signerId = rf;
                    }

                }
                else if (!localName.Equals("CanonicalizationMethod") &&
                    !localName.Equals("SignedInfo"))
                {
                    throw new org.xml.sax.SAXException("Unerwartetes Element im Signature-Header: " + localName);
                }
            }
            else if (uri.Equals(XADES_XMLNS))
            {
              if (localName.Equals("QualifyingProperties")
                || localName.Equals("SignedSignatureProperties"))
                ;
              else if (localName.Equals("SigningTime"))
                  currentElement = new System.Text.StringBuilder();
              else if (localName.Equals("SignedProperties"))
              {
                signingPropsId = attributes.getValue("Id");
              }
              else
                throw new SAXException(DialogHandler.text.getString("unexpected_entry") + ": " + localName);
            }
            else
              throw new SAXException(DialogHandler.text.getString("unexpected_entry") + ": " + localName);
		}
		
		/// <summary>
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("End-Element: " + localName);

            if (uri.Equals(DS_XMLNS))
            {
                if (localName.Equals("Reference"))
                {
                    log.debug("############ Anzahl refs End :" + sig.refs.Count);
                    sr = null;
                }
                else if (localName.Equals("DigestValue"))
                {
                    log.debug("STARTE B64DEC " + currentElement);
                    log.debug("ID " + sr.RefID);

                    byte[] dv = de.osci.helper.Base64.decode(currentElement.ToString());
                    sr.digestValue = dv;
                }
                else if (localName.Equals("SignatureValue"))
                {
                    sig.signatureValue = de.osci.helper.Base64.decode(currentElement.ToString());
                }
                else if (localName.Equals("SignedInfo"))
                {
                }
                else if (localName.Equals("Signature"))
                {
                    if (parentHandler is ContentContainerBuilder)
                    {
                        log.debug("##########" + sig.signerId);
                        String sigId = sig.signerId;
                        if (sigId[0] == '#')
                            sigId = sigId.Substring(1);
                        Role signer = ((ContentContainerBuilder)parentHandler).msg.getRoleForRefID(sigId);

                        if (signer == null)
                        {
                            log.error("Das Referenziert Signer Object konnte nicht gefunden werden.");
                            throw new org.xml.sax.SAXException("Das Referenziert Signer Object konnte nicht gefunden werden.");
                        }
                        sig.signer = signer;
                        sig.RefID = "BALLA-BALLA";
                        ((ContentContainerBuilder)parentHandler).ContentContainer.addSignature(sig);
                        sig.signedInfo = (byte[])((ContentContainerBuilder)parentHandler).can.SignedInfos[1];
                        ((ContentContainerBuilder)parentHandler).can.SignedInfos.RemoveAt(1);
                        if (signingPropsId != null)
                        {
                            sig.signingProperties = (String)((ContentContainerBuilder)parentHandler).can.SignedProperties[0];
                            ((ContentContainerBuilder)parentHandler).can.SignedProperties.RemoveAt(0);
                            sig.signingPropsId = signingPropsId;
                            sig.signingTime = signingTime;
                        }
                    }
                    else if (parentHandler is OSCIMessageBuilder)
                    {
                        ((de.osci.osci12.messagetypes.OSCIMessageBuilder)parentHandler).OSCIMessage.signatureHeader = sig;
                    }

                    this.xmlReader.setContentHandler(this.parentHandler);
                }
            }
            else if (localName.Equals("SigningTime")
                && uri.Equals(XADES_XMLNS)
                && (parentHandler is ContentContainerBuilder))
            {
                signingTime = currentElement.ToString();
            }
			currentElement = null;
		}


		static OSCISignatureBuilder() 
        {
			log = LogFactory.getLog(typeof(OSCISignatureBuilder));
		}
	}
}