namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;

	/// <summary> Diese Klasse bildet eine XML-Signature Referenze in der OSCI-Bibliothek ab.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class OSCISignatureReference:MessagePart {

		new private static Log log;
		internal System.String digestMethodAlgorithm;
		new internal byte[] digestValue = null;
		internal System.Collections.ArrayList transformerAlgorithms = new System.Collections.ArrayList();

		private void  InitBlock() {
			transformerAlgorithms = new System.Collections.ArrayList();
		}

		/// <summary> Liefert den Hashalgorithmus.
		/// </summary>
		/// <value> String-Id des Hashalgorithmus
		/// </value>
		/// <seealso cref="de.osci.osci12.common.Constants">
		/// </seealso>
		virtual public System.String DigestMethodAlgorithm {
			get {
				return digestMethodAlgorithm;
			}
			set {
				this.digestMethodAlgorithm = value;
			}
		}

/*
		/// <summary> Liefert den Hashwert.
		/// </summary>
		/// <returns> den Hashwert oder null, falls dieser noch nicht ermittelt wurde.
		/// </returns>
		new internal virtual byte[] DigestValue {
			get {
				return digestValue;
			}
			set {
				this.digestValue = value;
			}
		}
*/
		virtual public System.String[] TransformerAlgorithms {
			get {
				return (System.String[]) transformerAlgorithms.ToArray(typeof(String));
			}
		}

        internal OSCISignatureReference(MessagePart MessagePart, String digestAlgorithm)
        {
			InitBlock();
			id = MessagePart.RefID;
			if (MessagePart is Attachment)
				id = "cid:" + MessagePart.RefID;
			else
				id = "#" + MessagePart.RefID;
			setNSPrefixes(MessagePart.soapNSPrefix, MessagePart.osciNSPrefix, MessagePart.dsNSPrefix, 
				MessagePart.xencNSPrefix, MessagePart.xsiNSPrefix); 
			digestMethodAlgorithm = digestAlgorithm;
			digestValue = MessagePart.getDigestValue(digestMethodAlgorithm);
			transformerAlgorithms = MessagePart.transformers;
		}
		
		// Konstruktor fr den Parser
		internal OSCISignatureReference() {
			InitBlock();
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed) {
			MoreSupport.ToStream(out_Renamed,"<" + dsNSPrefix + ":Reference URI=\"" + id + "\">\n",Constants.CHAR_ENCODING);
			if (transformerAlgorithms.Count > 0) {
				MoreSupport.ToStream(out_Renamed, "<" + dsNSPrefix + ":Transforms>\n",Constants.CHAR_ENCODING);
				for (int j = 0; j < transformerAlgorithms.Count; j++) {
					MoreSupport.ToStream(out_Renamed,(String)transformerAlgorithms[j],Constants.CHAR_ENCODING);
				}
				MoreSupport.ToStream(out_Renamed, "</" + dsNSPrefix + ":Transforms>\n",Constants.CHAR_ENCODING);
			}
			MoreSupport.ToStream(out_Renamed,"<" + dsNSPrefix + ":DigestMethod Algorithm=\"" + digestMethodAlgorithm + "\"></" + dsNSPrefix + ":DigestMethod>\n<" + dsNSPrefix + ":DigestValue>" + de.osci.helper.Base64.encode(digestValue) + "</" + dsNSPrefix + ":DigestValue>\n</" + dsNSPrefix + ":Reference>\n",Constants.CHAR_ENCODING);
		}
		
		public override System.String ToString() {
			try {
				System.IO.MemoryStream out_Renamed = new System.IO.MemoryStream();
				this.writeXML(out_Renamed);
				char[] tmpChar;
				byte[] tmpByte;
				tmpByte = out_Renamed.GetBuffer();
				tmpChar = new char[out_Renamed.Length];
				System.Array.Copy(tmpByte, 0, tmpChar, 0, tmpChar.Length);
				return new System.String(tmpChar);
			}
			catch (System.IO.IOException ex) {
				log.error("Fehler", ex);
				return null;
			}
		}
		static OSCISignatureReference() {
			log = LogFactory.getLog(typeof(OSCISignatureReference));
		}
	}
}