namespace de.osci.osci12.messageparts {
	
	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	using de.osci.osci12.common;
	using de.osci.osci12.messagetypes;
	
	/// <summary><p><H4>Laufzettelparser</H4></p>Wird von Anwendungen nie direkt bentigt.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class ProcessCardBundleBuilder:MessagePartParser {

		private void  InitBlock() {
			inspections = new System.Collections.ArrayList();
		}

		virtual public ProcessCardBundle ProcessCardBundleObject {
			get {
				return processCard;
			}
			
		}

		private static Log log;
		internal ProcessCardBundle processCard = null;
		private System.String
			name;
		private bool isInProcessCard = false;
		internal TimestampBuilder timeBuilder = null;
		private XMLReader xmlReader = null;
		private DefaultHandler parent = null;
		private InspectionBuilder inspectionBuilder = null;
		private int[] check;

		private System.Collections.ArrayList inspections;

		/// <summary>
		/// </summary>
		/// <param name="xmlReader">
		/// </param>
		/// <param name="parentHandler">
		/// </param>
		/// <param name="atts">
		/// </param>
		/// <param name="check">Array von gltigen Elementen
		/// -1 Element drfen beliebig oft auftreten
		/// 0 niemals
		/// 1 Einmal
		/// Es werden die Elemente der ProcessCard beschieben
		/// Der Erste Wert im Array bechreibt das Creation-Element
		/// 2. Das Forward Element
		/// 3. das Reception - Element
		/// 4. Das Subject Element
		/// </param>
		public ProcessCardBundleBuilder(System.String name, XMLReader xmlReader, DefaultHandler parentHandler, Attributes atts, int[] check) {
			InitBlock();
			this.name = name;
			processCard = new ProcessCardBundle(name);
			this.check = check;
			OSCIMessage msg = ((OSCIMessageBuilder)parentHandler).OSCIMessage; 
			processCard.setNSPrefixes(msg); 
			this.xmlReader = xmlReader;
			this.parent = parentHandler;
		}
		
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			log.debug("Start Element: " + localName);
			if (localName.Equals("MessageId") && uri.Equals(OSCI_XMLNS)) {
				currentElement = new System.Text.StringBuilder();
			}
			else if (localName.Equals("ProcessCardBundle")&& uri.Equals(OSCI_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("ProcessCard")&& uri.Equals(OSCI_XMLNS)) {
				processCard.RecentModification=attributes.getValue("RecentModification");
			}
			else if (localName.Equals("Creation")&& uri.Equals(OSCI_XMLNS)
				|| localName.Equals("Forwarding")&& uri.Equals(OSCI_XMLNS)
				|| localName.Equals("Reception")) {
				timeBuilder = new TimestampBuilder(this.xmlReader, this);
				xmlReader.setContentHandler(timeBuilder);
			}
			else if (localName.Equals("Subject")&& uri.Equals(OSCI_XMLNS)) {
				currentElement = new System.Text.StringBuilder();
			}
			else if (localName.Equals("InspectionReport")&& uri.Equals(OSCI_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("Inspection")&& uri.Equals(OSCI_XMLNS)) {
				inspectionBuilder = new InspectionBuilder(xmlReader, this);
//				inspections.Add(inspectionBuilder.InspectionObject);
				xmlReader.setContentHandler(inspectionBuilder);
			}
			else {
				throw new SAXException("Das Element " + localName + " ist im Content von Process-Card nicht gltig");
			}
		}
		
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("End-Element: " + localName);
			if (localName.Equals(name)&& uri.Equals(OSCI_XMLNS)) {
				if (inspections.Count > 0) {
					processCard.Inspections = (Inspection[]) inspections.ToArray(typeof(Inspection));
				}
				processCard.Name = name;
				xmlReader.setContentHandler(parent);
			}
			else if (localName.Equals("MessageId")&& uri.Equals(OSCI_XMLNS)) {
				processCard.MessageId =  MoreSupport.NewString(de.osci.helper.Base64.decode(currentElement.ToString()), Constants.CHAR_ENCODING);
			}
			else if (localName.Equals("Creation")&& uri.Equals(OSCI_XMLNS)) {
				if (check[0] != 0) {
					processCard.Creation = timeBuilder.TimestampObject;
					check[0]--;
				}
				else {
					throw new SAXException("Das Element  " + timeBuilder.TimestampObject.Name + " ist im Content dieser Process-Card nicht gltig");
				}
			}
			else if (localName.Equals("Forwarding")&& uri.Equals(OSCI_XMLNS)) {
				if (check[1] != 0) {
					processCard.Forwarding = timeBuilder.TimestampObject;
					check[1]--;
				}
				else {
					throw new SAXException("Das Element  " + timeBuilder.TimestampObject.Name + " ist im Content dieser Process-Card nicht gltig");
				}
			}
			else if (localName.Equals("Reception")&& uri.Equals(OSCI_XMLNS)) {
				if (check[2] != 0) {
					processCard.Reception = timeBuilder.TimestampObject;
					check[2]--;
				}
				else {
					throw new SAXException("Das Element " + timeBuilder.TimestampObject.Name + " ist im Content dieser Process-Card nicht gltig");
				}
			}
			else if (localName.Equals("Subject")&& uri.Equals(OSCI_XMLNS)) {
				if (check[3] != 0) {
					processCard.Subject = currentElement.ToString();
					check[3]--;
				}
				else {
					throw new SAXException("Das Element Subject ist im Content dieser Process-Card nicht gltig");
				}
			}
			else if (localName.Equals("InspectionReport")&& uri.Equals(OSCI_XMLNS)) {
				; // nothing to do
			}
			else if (localName.Equals("Inspection")&& uri.Equals(OSCI_XMLNS)) {
				inspections.Add(inspectionBuilder.InspectionObject);
			}
			else if (localName.Equals("ProcessCard")&& uri.Equals(OSCI_XMLNS)) {
				; // nothing to do
			}
			else {
				throw new SAXException("Das End-Element " + localName + " ist im Content von Process-Card nicht gltig");
			}
			currentElement = null;
		}
		
		
		static ProcessCardBundleBuilder() {
			log = LogFactory.getLog(typeof(ProcessCardBundleBuilder));
		}
	}
}