namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;

	/// <summary>Diese Klasse reprsentiert das OSCI-Timestamp-Element, es enthlt
	/// Zeitstempelinformationen
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class Timestamp:MessagePart {

		internal System.String Algorithm;
private static  long serialVersionUID = 3258130267062415434L;
		/// <summary> Liefert den Algorithmus (fr kryptographische Zeitstempel)
		/// </summary>
		/// <value> der Algorithmus
		/// </value>
		virtual public System.String algorithm {
			get {
				return Algorithm;
			}
			
		}

		/// <summary> Liefert den Typ des Zeitstempels als String. Mgliche Werte sind
		/// "Creation", "Forwarding", "Reception" und "Timestamp".
		/// </summary>
		/// <value> id
		/// </value>
		virtual public System.String Name {
			get {
				return TIMESTAMP_NAMES[nameID];
			}
			
		}

		internal System.String TimeStamp;

		/// <summary> Liefert den Zeitstempelstring selbst.
		/// </summary>
		/// <value> Zeitstempel
		/// </value>
		virtual public System.String timeStamp {
			get {
				return TimeStamp;
			}
			
		}

		internal int NameID;
		
		/// <summary> Liefert den Identifier des Zeitstempeltyps. Mgliche Werte sind
		/// PROCESS_CARD_CREATION, PROCESS_CARD_FORWARDING, PROCESS_CARD_RECEPTION
		/// und PROCESS_CARD_TIMESTAMP.
		/// </summary>
		/// <value> id Typ
		/// </value>
		virtual public int nameID {
			get {
				return NameID;
			}
			
		}

		new private static Log log;
		private static int idNr = - 1;
		public static System.String[] TIMESTAMP_NAMES = new System.String[]{"Creation", "Forwarding", "Reception", "Timestamp"};
		public const int PROCESS_CARD_CREATION = 0;
		public const int PROCESS_CARD_FORWARDING = 1;
		public const int PROCESS_CARD_RECEPTION = 2;
		public const int PROCESS_CARD_TIMESTAMP = 3;
		
		internal Timestamp() {
			idNr++;
			this.id = "timestamp" + idNr;
		}
		
		// Bei algorithm == null oder Leerstring ist der Typ "plain"
		public Timestamp(int nameID, System.String algorithm, System.String timeStamp) {
			idNr++;
			this.id = "timestamp" + idNr;
			this.NameID = nameID;
			this.Algorithm = algorithm;
			this.TimeStamp = timeStamp;
		}
		
		/// <summary>Diese Methode wird intern aufgerufen.
		/// </summary>
		/// <param name="out_Renamed">Outputstream in den das Ergebnis serialisiert wird
		/// </param>
		public override void  writeXML(System.IO.Stream out_Renamed) {
			MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix + ":" + TIMESTAMP_NAMES[nameID] + ">",Constants.CHAR_ENCODING);
			if ((algorithm == null) || (algorithm.Length == 0)) 
			{
				MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix + ":Plain>" + timeStamp + "</" + osciNSPrefix + ":Plain>",Constants.CHAR_ENCODING);
			}
			else 
			{
				MoreSupport.ToStream(out_Renamed,
					"<" + osciNSPrefix + ":Cryptographic Algorithm=\"" + algorithm + "\">"
					+ de.osci.helper.Base64.encode(timeStamp,Constants.CHAR_ENCODING)
					+ "</" + osciNSPrefix + ":Cryptographic>"
					,Constants.CHAR_ENCODING);
			}
			MoreSupport.ToStream(out_Renamed,"</" + osciNSPrefix + ":" + TIMESTAMP_NAMES[nameID] + ">",Constants.CHAR_ENCODING);
		}
		
		static Timestamp() {
			log = LogFactory.getLog(typeof(Timestamp));
		}
	}
}