namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	
	/// <summary><p><H4>Zeitstempelparser</H4></p>Wird von Anwendungen nie direkt bentigt.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class TimestampBuilder:MessagePartParser {

		virtual internal Timestamp TimestampObject {
			get {
				return timestamp;
			}
			
		}

		private static Log log;
		private Timestamp timestamp = null;
		private XMLReader xmlReader = null;
		private DefaultHandler parent = null;
		public TimestampBuilder(XMLReader xmlReader, DefaultHandler parentHandler) {
			MessagePart mp; 
			timestamp = new Timestamp();
			if (parentHandler is InspectionBuilder) 
				mp = ((InspectionBuilder)parentHandler).inspection; 
			else 
			{
				mp = ((ProcessCardBundleBuilder)parentHandler).processCard; 
			}
			
			timestamp.setNSPrefixes(mp.soapNSPrefix, mp.osciNSPrefix, mp.dsNSPrefix, mp.xencNSPrefix, mp.xsiNSPrefix); 
			this.xmlReader = xmlReader;
			this.parent = parentHandler;
			timestamp = new Timestamp();
		}
		
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			log.debug("Start-Element: " + localName);
			if (localName.Equals("Plain")&& uri.Equals(OSCI_XMLNS)) {
				currentElement = new System.Text.StringBuilder();
			}
			else if (localName.Equals("Cryptographic")&& uri.Equals(OSCI_XMLNS)) {
				currentElement = new System.Text.StringBuilder();
				System.String algo = attributes.getValue("Algorithm");
				if (algo == null || algo.ToUpper().Equals("".ToUpper())) {
					throw new SAXException("Das Attribut Algorthm fehlt im Element Timestamp!");
				}
				timestamp.Algorithm = algo;
			}
			else {
				throw new SAXException("Das Element " + localName + " ist im Content von Timestamp nicht gltig");
			}
		}
		
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("End-Element: " + localName);
			if (localName.Equals("Plain")&& uri.Equals(OSCI_XMLNS)) {
				timestamp.TimeStamp = currentElement.ToString();
			}
			else if (localName.Equals("Cryptographic")&& uri.Equals(OSCI_XMLNS)) {
				timestamp.TimeStamp = MoreSupport.NewString(de.osci.helper.Base64.decode(currentElement.ToString()), de.osci.osci12.common.Constants.CHAR_ENCODING);
			}
			else if (localName.Equals("Timestamp")&& uri.Equals(OSCI_XMLNS)) {
				timestamp.NameID = Timestamp.PROCESS_CARD_TIMESTAMP;
				parent.endElement(uri, localName, qName);
				xmlReader.setContentHandler(parent);
			}
			else if (localName.Equals("Creation")&& uri.Equals(OSCI_XMLNS)) {
				timestamp.NameID = Timestamp.PROCESS_CARD_CREATION;
				parent.endElement(uri, localName, qName);
				xmlReader.setContentHandler(parent);
			}
			else if (localName.Equals("Forwarding")&& uri.Equals(OSCI_XMLNS)) {
				timestamp.NameID = Timestamp.PROCESS_CARD_FORWARDING;
				parent.endElement(uri, localName, qName);
				xmlReader.setContentHandler(parent);
			}
			else if (localName.Equals("Reception")&& uri.Equals(OSCI_XMLNS)) {
				timestamp.NameID = Timestamp.PROCESS_CARD_RECEPTION;
				parent.endElement(uri, localName, qName);
				xmlReader.setContentHandler(parent);
			}
			else {
				throw new SAXException("Das Element " + localName + " ist im Content von TimeStamp nicht gltig");
			}
			currentElement = null;
		}
		
		static TimestampBuilder() {
			log = LogFactory.getLog(typeof(TimestampBuilder));
		}
	}
}