namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using de.osci.osci12.soapheader;
	
	/// <summary><p><H4>MessageId-Anforderungsauftrag</H4></p>
	/// Mit dieser Klasse werden Nachrichtenobjekte zur Anforderung einer
	/// MessageId angelegt. Clients erhalten als Antwort auf diese Nachricht
	/// vom Intermedir ein Nachrichtenobjekt, welches eine Rckmeldung ber den
	/// Erfolg der Operation und ggf. die angeforderte MessageId enthlt.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p>
	/// <p>Version: 1.5</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.ResponseToGetMessageId">
	/// </seealso>
	public class GetMessageId:OSCIRequest {

		new private static Log log;
		
		/// <summary> Legt ein Nachrichtenobjekt zur Anforderung einer MessageId an.
		/// </summary>
		/// <param name="dh">DialogHandler-Objekt des Dialogs, innerhalb dessen die Nachricht
		/// versendet werden soll.
		/// </param>
		/// <seealso cref="de.osci.osci12.common.DialogHandler">
		/// </seealso>
		public GetMessageId(DialogHandler dh):base(dh) {
			messageType = GET_MESSAGE_ID;
			originator = ((Originator) dh.Client);
			if (!dialogHandler.explicitDialog)	dialogHandler.resetControlBlock();
			dialogHandler.Controlblock.Response = dialogHandler.prevChallenge;
			dialogHandler.Controlblock.Challenge = Tools.createRandom(10);
			dialogHandler.Controlblock.SequenceNumber = dialogHandler.Controlblock.SequenceNumber + 1;
		}
		
		internal GetMessageId():base() {
			messageType = GET_MESSAGE_ID;
		}
		
		public override void  compose() 
		{
			base.compose();
			if (dialogHandler.Controlblock.Challenge == null) 
			{
				throw new System.SystemException("Kein Challenge-Wert in DialogHandler eingestellt.");
			}
			if (dialogHandler.Controlblock.SequenceNumber == - 1) {
				throw new System.SystemException("Kein Squenznummer in DialogHandler eingestellt.");
			}
			if (dialogHandler.Client.hasCipherCertificate()) {				
				Object dumm=dialogHandler.Client.CipherCertificate;
				if (nonIntermediaryCertificatesH == null) {
					nonIntermediaryCertificatesH = new NonIntermediaryCertificatesH();
				}
				nonIntermediaryCertificatesH.CipherCertificateOriginator = (Originator) dialogHandler.Client;
			}
			
			body = new Body("<" +  osciNSPrefix + ":getMessageId></" +  osciNSPrefix + ":getMessageId>");
			stateOfMsg |= STATE_COMPOSED;
		}
		
		/// <summary> Versendet die Nachricht und liefert die Antwortnachricht zurck.
		/// Diese Methode wirft eine Exception, wenn beim Aufbau oder Versand der
		/// Nachricht ein Fehler auftritt. Fehlermeldungen vom Intermedir mssen
		/// dem Feedback der Antwortnachricht entnommen werden.
		/// </summary>
		/// <returns> das Antwortnachricht-Objekt
		/// </returns>
		// --> Storestreams:
		public virtual ResponseToGetMessageId send(System.IO.Stream outp, System.IO.Stream inp)
		{
			return (ResponseToGetMessageId) transmit(outp, inp);
		}
		public virtual ResponseToGetMessageId send() 
		{
			return (ResponseToGetMessageId) transmit(null, null);
		}
		// <-- Storestreams:

		public override void  writeXML(System.IO.Stream out_Renamed) {
			base.writeXML(out_Renamed);
			// ClientSignatur
			if (signatureHeader != null) {
				signatureHeader.writeXML(out_Renamed);
			}

			// DesiredLanguage
			desiredLanguagesH.writeXML(out_Renamed);
			
			if (nonIntermediaryCertificatesH != null) {
				nonIntermediaryCertificatesH.writeXML(out_Renamed);
			}
			completeMessage(out_Renamed);
		}
		
		static GetMessageId() {
			log = LogFactory.getLog(typeof(GetMessageId));
		}
	}
}