namespace de.osci.osci12.messagetypes {

	using System;
	using javax.xml.parsers;
	using org.apache.commons.logging;
	using org.xml.sax;
	using de.osci.osci12.common;
	using de.osci.osci12.dotcommon; // Storestreams
	using de.osci.osci12.encryption;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using de.osci.helper;

	/// <summary><H4>Streamparser</H4>
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public abstract class IncomingMSGParser {

		internal static System.String dir = "./../../../osci-bibliothek-net-test/Daten/Testdaten/";		
		internal abstract OSCIEnvelopeBuilder getParser(XMLReader reader, DialogHandler dh);
		protected static Addressee[] defaultSupplier;
		private static Log log;
		public static System.String test;

		public IncomingMSGParser() {
		}
		
		private static void  collectZerts(System.Collections.Hashtable hash, Role[] r) 
		{
			for (int i = 0; i < r.Length; i++) 
			{
				if (r[i].hasCipherCertificate())
					SupportClass.PutElement(hash, r[i].CipherCertificateId, r[i].CipherCertificate);
				
				if (r[i].hasSignatureCertificate())
					SupportClass.PutElement(hash, r[i].SignatureCertificateId, r[i].SignatureCertificate);
			}
		}

		private static void  writeToFile(System.String name, byte[] data) {
			System.IO.FileStream out_Renamed = new System.IO.FileStream(dir + name + ".osci", System.IO.FileMode.Create);
			out_Renamed.Write(data,0,data.Length);
			out_Renamed.Close();
			log.debug("Aufbau-Ergebnis: " + name + ": " + new System.String(SupportClass.ToCharArray(data)));
		}
		
		// --> Storestreams:
		internal virtual OSCIMessage parse(System.IO.Stream is_Renamed, 
			DialogHandler dh, 
			StoreInputStream storeInStream) {
			XMLReader reader = null;
			OSCIEnvelopeBuilder builder = null;
	
			javax.xml.parsers.SAXParserFactory saxFactory = javax.xml.parsers.SAXParserFactory.newInstance();
			SAXParser parser = saxFactory.newSAXParser();
			//saxFactory.setNamespaceAware(true);
			reader = parser.XMLReader;
			builder = getParser(reader, dh);
			reader.setFeature("http://xml.org/sax/features/validation", false);
			reader.setContentHandler(builder);
			reader.setErrorHandler(builder);
			builder.hashNCanStream = new de.osci.helper.Canonizer(is_Renamed, storeInStream);
			reader.parse(builder.hashNCanStream);
			return builder.childBuilder.msg;
		}
		
		internal virtual OSCIMessage parseStream(System.IO.Stream in_Renamed, 
			DialogHandler dial, 
			bool request, 
			System.IO.Stream storeStream) {
      return parseStream(in_Renamed, dial, request, false, storeStream); 
     } 
    
     internal virtual OSCIMessage parseStream(System.IO.Stream  in_Renamed, DialogHandler dial, bool request, 
                             bool decryptedStream, 
                             System.IO.Stream storeStream)
     { 

			OSCIMessage msg = null;
			MIMEParser incomingMsg = null; 
			MIMEPartInputStream mimeStream = null; 

			StoreInputStream sis = null;
			log.debug("IN "+in_Renamed);

			if (storeStream != null) {
				sis = new StoreInputStream(in_Renamed, storeStream);
				incomingMsg = new MIMEParser(sis);
			}
			else
				incomingMsg = new MIMEParser(in_Renamed);

			mimeStream = incomingMsg.getNextStream();
			msg = parse(mimeStream, dial, sis);
		    msg.boundary_string = incomingMsg.boundary;

			log.debug("Fertig mit Parsen des Transport Objektes. Msgtype: " + msg.GetType().ToString());
			// <-- Storestreams:
			
			if (msg.MessageType == OSCIMessage.SOAP_MESSAGE_ENCRYPTED) {
				de.osci.osci12.encryption.EncryptedData ed = ((SOAPMessageEncrypted) msg).encryptedData;
				mimeStream = incomingMsg.getNextStream();
					
				System.String s = null;
				log.debug("Mime ID:  " + mimeStream.ContentID + " Encrypted ID: " + ed.CipherData.CipherReference.URI);
					
				if (!(s = ("cid:"+mimeStream.ContentID)).Equals(ed.CipherData.CipherReference.URI)) {
					throw new System.ArgumentException("Falsche Attachment-Id: " + s);
				}
					
				de.osci.osci12.roles.Role role = null;
				java.security.cert.X509Certificate cert = null;
					
				if (dial == null) 
				{
					role = null;
					for (int i=0; i < defaultSupplier.Length; i++)
					{
						if (defaultSupplier[i].CipherCertificate.Equals(ed.KeyInfo.EncryptedKeys[0].KeyInfo.X509Data.X509Certificate))
						{
							role = defaultSupplier[i];
							break;
						}
					}
					if (role == null)
						throw new SoapClientException("9200");
				} 
				else 
				{
					try {
						if (request) {
							cert = dial.Supplier.CipherCertificate;
						} else {
							cert = dial.Client.CipherCertificate;
						}
						String s1=MoreSupport.NewString(cert.getEncoded(), Constants.CHAR_ENCODING);
						String s2=MoreSupport.NewString(ed.KeyInfo.EncryptedKeys[0].KeyInfo.X509Data.X509Certificate.getEncoded(), Constants.CHAR_ENCODING);
						if (s1.Equals(s2)) {
							if (request) {
								role = dial.Supplier;
							} else {
								role = dial.Client;
							}
						}
					} catch (OSCIRoleException ex) {
						// wird unten abgefangen
					}
				}
					
				if (role == null) {
					throw new OSCIRoleException("Kein Rollenobjekt mit dem Verschlsselungszertifikat der Nachricht im DialogHandler gefunden.");
				}
					
				if (!ed.KeyInfo.EncryptedKeys[0].EncryptionMethodAlgorithm.Equals(Constants.ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5)) {
					throw new System.Exception("Falscher asymmetrischer Verschlsselungsalgorithmus: " + ed.KeyInfo.EncryptedKeys[0].EncryptionMethodAlgorithm);
				}
					
				log.debug("\n\nENC-ALGO: "+ ed.EncryptionMethodAlgorithm + "\n\n");
					
				System.IO.Stream inKey = ed.KeyInfo.EncryptedKeys[0].CipherData.CipherValue.CipherValueBuffer.InputStream;
				inKey.Seek(0,System.IO.SeekOrigin.Begin);
					
				Base64InputStream base64inKey = new Base64InputStream(inKey);
				System.IO.MemoryStream bos2 = new System.IO.MemoryStream();
				int anz = 0;
				byte[] bytes = new byte[256];
					
				while ((anz = SupportClass.ReadInput(base64inKey, ref bytes, 0, bytes.Length)) > - 1) {
					bos2.Write(bytes, 0, anz);
				}
				inKey.Close();
//                String bumm = SupportClass.NewString(bos2.ToArray(), "UTF-8");
//                String bumm = SupportClass.NewString(;
                byte[] decryptedKey = role.Decrypter.decrypt(bos2.ToArray());
				log.debug("Symetrischer Schlssel: " + MoreSupport.NewString(decryptedKey, "UTF-8"));
					
				System.IO.Stream inStream;
				bool b64 = mimeStream.encoding.ToLower().Equals("base64") || mimeStream.contentType.ToLower().Equals("text/base64");
				if (b64)
					inStream = new de.osci.osci12.common.Base64InputStream(mimeStream);
				else
					inStream = mimeStream;
				SymCipherInputStream cin = new SymCipherInputStream(inStream, Crypto.createSymKey(decryptedKey,ed.EncryptionMethodAlgorithm), false);
				log.debug("#################### Encrypted OSCI-Msg wurde komplett verarbeitet, nun wird der Transportumschlag geffnet und die eigentliche OSCI-Nachricht betrachtet ####################");
				// --> Storestreams:
				msg = parseStream(cin, dial, request,true, storeStream);
				msg.Base64Encoding = b64;
				msg.dialogHandler.Encryption = true;
                msg.dialogHandler.SymmetricCipherAlgorithm = ed.EncryptionMethodAlgorithm;
				// <-- Storestreams:
	
				return msg;
			} else {
				// Check, ob eine Antwort auf einen verschlsselten request auch verschlsselt war. 
				if (!request && msg.dialogHandler.Encryption && !decryptedStream && 
					!(msg.MessageType == OSCIMessage.SOAP_FAULT_MESSAGE)) 
					throw new OSCICipherException(DialogHandler.text.getString("unencrypted_msg"), null); 

				msg.dialogHandler.Encryption = false;
				// es handelt sich um eine nicht verschlsselte Nachricht
				// die Attachments werden der Nachricht hinzugefgt
				log.debug("Es handelte sich um eine nicht Verschlsselte Nachricht");
				// ## Lesen der Attachments
				log.debug("Nchster Schritt: Verarbeitung der Attachments");
				readAttachment(msg, incomingMsg);
				log.debug("Die Signaturen werden berprft.");
				
				// Nachricht signiert?
				if ((msg.dialogHandler.CheckSignatures) && (msg.signatureHeader != null)) {
					bool sigErg = checkMsgHashes(msg);
					System.String str = null;
					
					if (sigErg) {
						log.debug("Die Signaturen der XML-OSCI-Daten sind in Ordnung.");
						
						System.Collections.IEnumerator hashes = msg.hashableMsgPart.Keys.GetEnumerator();
						str = "Noch nicht geprfte Signature: ";
						
						while (hashes.MoveNext()) {
							str += (((System.String) hashes.Current) + ", ");
						}
						log.debug(str);
					} else {
						log.error("Die Signaturen der XML-OSCI-Daten sind fehlerhaft.");
						throw new OSCIErrorException("9601", msg);
					}
				} else {
					log.debug("Unsignierte-Nachricht");
				}
			}
			
			log.debug("Alles ist Fertig");			
			return msg;
		}
		
		internal  bool checkMsgHashes(OSCIMessage msg) 
        {
			try 
            {
				OSCISignature sig = msg.signatureHeader;
				
					System.Collections.Hashtable refsHash = sig.getDigests();
					System.Collections.IEnumerator keys = refsHash.Keys.GetEnumerator();
					
					// Auswertung der Hashes
					while (keys.MoveNext()) {
						System.String keyName = (System.String) keys.Current;
						System.Object object_Renamed = msg.hashableMsgPart[keyName];
						if (object_Renamed == null) {
							log.error("Element zur Signatur-Referenz '" + keyName + "' nicht in Nachricht gefunden.");
							return false;
						}
						else if (!Tools.compareByteArrays((byte[]) msg.hashableMsgPart[keyName], (byte[]) refsHash[keyName]))
						{
							log.error("Hashwerte der Signatur-Referenz '" + keyName + "' stimmen nicht berein.");
                            log.error(MoreSupport.NewString((byte[])msg.hashableMsgPart[keyName], "UTF-8") + "\n" + MoreSupport.NewString((byte[])refsHash[keyName], "UTF-8"));
							return false;						
                        }					        
						else
						{
							msg.hashableMsgPart.Remove(keyName);
						}
					}
					if (msg.hashableMsgPart.Count > 0) 
					{
							log.error("Nachricht enthlt " + msg.hashableMsgPart.Count + "unsignierte Elemente.");
							 throw new OSCIErrorException("9602", msg);
					}

                    java.security.cert.X509Certificate c;
                    if (msg is OSCIRequest)
                        c = msg.dialogHandler.Client.SignatureCertificate;
                  else
                    c = msg.dialogHandler.Supplier.SignatureCertificate;
                  
                  msg.signerCert = c;

                  if ((c.GetKeyUsage() != null) && !c.GetKeyUsage()[0] && !c.GetKeyUsage()[1])
                  {
    	            log.error("Signature certificate has wrong key usage.");
                    return false;
                  }


                  bool res = MoreSupport.checkSignature(c, sig.SignedInfoBytes, sig.signatureValue, sig.signatureAlgorithm);
                  
                  if (!res) 
						return false;
	
				log.debug("Nach check Signature");
				return true;
			} 
			catch (System.Exception ex) 
			{
				log.error("Es ist ein Fehler beim berprfen der Hashwerte aufgetreten.", ex);		
				return false;
			}
		}

		private static bool cmpByteArrays(sbyte[] a, sbyte[] b) {
			if (a.Length != b.Length) {
				return false;
			}
			for (int i = 0; i < a.Length; i++) {
				if (a[i] != b[i]) {
					return false;
				}
			}
			
			return true;
		}

		private static bool cmpByteArrays(byte[] a, byte[] b) {
			if (a.Length != b.Length) {
				return false;
			}
			for (int i = 0; i < a.Length; i++) {
				if (a[i] != b[i]) {
					return false;
				}
			}			
			return true;
		}

		private static void  readAttachment(OSCIMessage msg, MIMEParser incomingMsg) {
			MIMEPartInputStream mimeStream = null;
			Attachment[] atts = msg.Attachments;
			while ((mimeStream = incomingMsg.getNextStream()) != null) {
				Attachment att = null;
				System.String refId = mimeStream.ContentID;
				log.debug("Attachment RefId: " + refId);
				bool b64 = mimeStream.encoding.ToLower().Equals("base64") || mimeStream.contentType.ToLower().Equals("text/base64");
					
				for (int i = 0; i < atts.Length; i++) 
				{
					if (atts[i].RefID.Equals(refId)) {
						log.debug("Vorbereitetes Attachment gefunden. Der Stream wird nun hinzugefgt.");
						att = atts[i];
						att.base64 = b64;
						att.boundary_string = incomingMsg.boundary;
						MessagePartsEntry.attachmentSetState(att, Attachment.STATE_OF_ATTACHMENT_PARSING, false);
						break;
					}
				}
					
                String digestMethod = null;
                if (msg.isSigned())
                    digestMethod = (String)msg.signatureHeader.getDigestMethods()["cid:"+refId];
   
                if (att == null) 
				{
					// ## Verschlsselte Attachachments
					log.debug("Verschlsseltes Attachment gefunden.");
					if (b64)
						att = MessagePartsEntry.attachment(new Base64InputStream(mimeStream), refId, mimeStream.Length, digestMethod);
					else
						att = MessagePartsEntry.attachment(mimeStream, refId, mimeStream.Length, digestMethod);
					att.base64 = b64;
					att.boundary_string = incomingMsg.boundary;
					msg.addAttachment(att);
					MessagePartsEntry.attachmentSetState(att, Attachment.STATE_OF_ATTACHMENT_ENCRYPTED, true);

				} 
				else 
				{
					// ## Attachments wurden der Nachricht bereits hinzugefgt
					log.debug("Unverschlsseltes Attachment gefunden.");
					if (b64)
						MessagePartsEntry.attachmentSetStream(att, new Base64InputStream(mimeStream), false, mimeStream.Length, digestMethod);
					else
						MessagePartsEntry.attachmentSetStream(att, mimeStream, false, mimeStream.Length, digestMethod);
				}
					
				// ## Allgemeiner Part fr Attachments
				log.debug("Es wurde ein Attachment hinzugefgt!RefID: " + att.RefID); // + " digest: " + new System.String(SupportClass.ToCharArray(att.encryptedDigestValues)));
				att.MimeHeaders = mimeStream.mime_headers;
                if (msg.isSigned())
				    SupportClass.PutElement(msg.hashableMsgPart,"cid:" + att.RefID, att.encryptedDigestValues[digestMethod]);
			}
		}

		static IncomingMSGParser() 
		{
			log = LogFactory.getLog(typeof(IncomingMSGParser));
		}
	}
}