namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	using de.osci.osci12.common;

	/// <summary><H4>Nachrichtenparser</H4> 
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class OSCIEnvelopeBuilder:DefaultHandler {

		virtual public XMLReader XmlReader {
			get {
				return xmlReader;
			}
		}

		internal const String XSD_ENC_SIG = "http://www.w3.org/2000/09/xmldsig# oscisig.xsd http://www.w3.org/2001/04/xmlenc# oscienc.xsd";
		// Deklaration der XSD fr die Nachrichtentypen
		internal const System.String XSD_INIT_DIALOG          = "http://schemas.xmlsoap.org/soap/envelope/ soapInitDialog.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_RSP_INIT_DIALOG      = "http://schemas.xmlsoap.org/soap/envelope/ soapResponseToInitDialog.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_END_DIALOG           = "http://schemas.xmlsoap.org/soap/envelope/ soapExitDialog.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_RSP_END_DIALOG       = "http://schemas.xmlsoap.org/soap/envelope/ soapResponseToExitDialog.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_GET_MSG_ID           = "http://schemas.xmlsoap.org/soap/envelope/ soapGetMessageId.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_RSP_GET_MSG_ID       = "http://schemas.xmlsoap.org/soap/envelope/ soapResponseToGetMessageId.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_STORE_DELIVERY       = "http://schemas.xmlsoap.org/soap/envelope/ soapStoreDelivery.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_RSP_STORE_DELIVERY   = "http://schemas.xmlsoap.org/soap/envelope/ soapResponseToStoreDelivery.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_FETCH_DELIVERY       = "http://schemas.xmlsoap.org/soap/envelope/ soapFetchDelivery.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_RSP_FETCH_DELIVERY   = "http://schemas.xmlsoap.org/soap/envelope/ soapResponseToFetchDelivery.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_FETCH_PROCESS_CARD   = "http://schemas.xmlsoap.org/soap/envelope/ soapFetchProcessCard.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_RSP_PROCESS_CARD     = "http://schemas.xmlsoap.org/soap/envelope/ soapResponseToFetchProcessCard.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_FORWARD_DELIVERY     = "http://schemas.xmlsoap.org/soap/envelope/ soapForwardDelivery.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_RSP_FORWARD_DELIVERY = "http://schemas.xmlsoap.org/soap/envelope/ soapResponseToForwardDelivery.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_ACCEPT_DELIVERY      = "http://schemas.xmlsoap.org/soap/envelope/ soapAcceptDelivery.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_RSP_ACCEPT_DELIVERY  = "http://schemas.xmlsoap.org/soap/envelope/ soapResponseToAcceptDelivery.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_MEDIATE_DELIVERY     = "http://schemas.xmlsoap.org/soap/envelope/ soapMediateDelivery.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_RSP_MEDIATE_DELIVERY = "http://schemas.xmlsoap.org/soap/envelope/ soapResponseToMediateDelivery.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_PROCESS_DELIVERY     = "http://schemas.xmlsoap.org/soap/envelope/ soapProcessDelivery.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_RSP_PROCESS_DELIVERY = "http://schemas.xmlsoap.org/soap/envelope/ soapResponseToProcessDelivery.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_MESSAGE_FAULT        = "http://schemas.xmlsoap.org/soap/envelope/ soapMessageFault.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_ENCRYPTED_DATA       = "http://schemas.xmlsoap.org/soap/envelope/ soapMessageEncrypted.xsd " + XSD_ENC_SIG;
		internal const System.String XSD_RSP_FETCH_PROCESS_CARD = "http://schemas.xmlsoap.org/soap/envelope/ soapResponseToFetchProcessCard.xsd " + XSD_ENC_SIG;
		
		// Allgemeines
		private static Log log;
		protected internal OSCIMessage msg = null;
		internal XMLReader xmlReader = null;
		protected internal de.osci.helper.Canonizer hashNCanStream = null;
		protected internal DialogHandler dhHandler = null;		
		internal OSCIMessageBuilder childBuilder = null;
		internal String soapNSPrefix, osciNSPrefix, dsNSPrefix, xencNSPrefix; 
		internal System.Text.StringBuilder ns; 

		/// <summary> Constructor for the OSCIMessageBuilder object
		/// </summary>
		/// <param name="xmlReader">
		/// </param>
		/// <param name="dh">
		/// </param>
		public OSCIEnvelopeBuilder(XMLReader xmlReader, DialogHandler dh) 
		{
			this.xmlReader = xmlReader;
			this.dhHandler = dh;
			ns = new System.Text.StringBuilder();
		}
		 public override void startPrefixMapping(String prefix, String uri)
		 { 
			 if (uri.Equals(OSCIMessageBuilder.SOAP_XMLNS)) 
				 soapNSPrefix = prefix; 
			 else if (uri.Equals(OSCIMessageBuilder.OSCI_XMLNS)) 
				 osciNSPrefix = prefix; 
			 else if (uri.Equals(OSCIMessageBuilder.DS_XMLNS)) 
				 dsNSPrefix = prefix; 
			 else if (uri.Equals(OSCIMessageBuilder.XENC_XMLNS)) 
				 xencNSPrefix = prefix; 
			 ns.Append(" xmlns:" + prefix + "=\"" + uri + "\"" ); 
		 }    

		protected internal OSCIEnvelopeBuilder() 
{
		}
		
		/// <summary> Start element.
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <param name="attributes">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			log.debug("Start-Element: " + qName);
			
			// SOAP-Namespacedefinition suchen
			if ((soapNSPrefix == null) || !localName.Equals("Envelope")) 
					throw new SAXException(DialogHandler.text.getString("msg_format_error") + ": " + localName); 

			System.String schemaLocation = attributes.getValue("xsi:schemaLocation");
			log.debug("Nachrichtentyp: " + schemaLocation);
			
			if (schemaLocation == null) {
				childBuilder = new SOAPFaultBuilder(this);
			}
			else if (schemaLocation.Equals(OSCIEnvelopeBuilder.XSD_ENCRYPTED_DATA)) {
				childBuilder = new SOAPMessageEncryptedBuilder(this);
			}
			else if (schemaLocation.Equals(OSCIEnvelopeBuilder.XSD_ACCEPT_DELIVERY)) {
				childBuilder = new AcceptDeliveryBuilder(this);
			}
			else if (schemaLocation.Equals(OSCIEnvelopeBuilder.XSD_RSP_FORWARD_DELIVERY)) {
				childBuilder = new ResponseToForwardDeliveryBuilder(this);
			}
			else if (schemaLocation.Equals(OSCIEnvelopeBuilder.XSD_RSP_INIT_DIALOG)) {
				childBuilder = new ResponseToInitDialogBuilder(this);
			}
			else if (schemaLocation.Equals(OSCIEnvelopeBuilder.XSD_PROCESS_DELIVERY)) {
				childBuilder = new ProcessDeliveryBuilder(this);
			}
			else if (schemaLocation.Equals(OSCIEnvelopeBuilder.XSD_RSP_MEDIATE_DELIVERY)) {
				childBuilder = new ResponseToMediateDeliveryBuilder(this);
			}
			else if (schemaLocation.Equals(OSCIEnvelopeBuilder.XSD_RSP_GET_MSG_ID)) {
				childBuilder = new ResponseToGetMessageIdBuilder(this);
			}
			else if (schemaLocation.Equals(OSCIEnvelopeBuilder.XSD_RSP_END_DIALOG)) {
				childBuilder = new ResponseToExitDialogBuilder(this);
			}
			else if (schemaLocation.Equals(OSCIEnvelopeBuilder.XSD_RSP_STORE_DELIVERY)) {
				childBuilder = new ResponseToStoreDeliveryBuilder(this);
			}
			else if (schemaLocation.Equals(OSCIEnvelopeBuilder.XSD_RSP_FETCH_DELIVERY)) {
				childBuilder = new ResponseToFetchDeliveryBuilder(this);
			}
			else if (schemaLocation.Equals(OSCIEnvelopeBuilder.XSD_RSP_PROCESS_CARD)) {
				childBuilder = new ResponseToFetchProcessCardBuilder(this);
			}
			else {
				log.error("Ergebnis Schema Test: " + schemaLocation.Equals(OSCIEnvelopeBuilder.XSD_FORWARD_DELIVERY));
				log.error("Falsche OSCI-Nachricht. Nachrichtentyp nicht bekannt! Nachrichtentypt ist:\n" + schemaLocation + "\n" + (OSCIEnvelopeBuilder.XSD_RSP_PROCESS_CARD));
				throw new SAXException("Falsche OSCI-Nachricht. Nachrichtentyp nicht bekannt! Nachrichtentypt ist: " + schemaLocation);
			}
			this.xmlReader.setContentHandler(childBuilder); 
			childBuilder.OSCIMessage.soapNSPrefix = soapNSPrefix; 
			childBuilder.OSCIMessage.osciNSPrefix = osciNSPrefix; 
			childBuilder.OSCIMessage.dsNSPrefix = dsNSPrefix; 
			childBuilder.OSCIMessage.xencNSPrefix = xencNSPrefix; 
			childBuilder.OSCIMessage.ns = ns.ToString ();
			}
		
		public override void  endDocument() {
			childBuilder.OSCIMessage.hashableMsgPart = hashNCanStream.DigestValues;
			if (childBuilder.OSCIMessage.signatureHeader != null) {
				childBuilder.OSCIMessage.signatureHeader.signedInfo =  (byte[])hashNCanStream.SignedInfos[0];//MoreSupport.NewString((byte[])hashNCanStream.SignedInfos[0],"UTF-8")
				hashNCanStream.SignedInfos.RemoveAt(0); 
			}
			childBuilder.OSCIMessage.stateOfMsg |= OSCIMessage.STATE_PARSED;
		}
		
		static OSCIEnvelopeBuilder() {
			log = LogFactory.getLog(typeof(OSCIEnvelopeBuilder));
		}
	}
}