namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	using de.osci.helper;


	/// <summary> Diese Klasse ist die Superklasse der Nachrichtenparser. Wird vom Anwender nicht bentigt.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N.Bngener</p>
	/// <p>Version: 1.5</p>
	/// </summary>
	public class OSCIMessageBuilder:DefaultHandler {

		/// <summary> Gets the OSCIMessage attribute of the OSCIMessageBuilder object
		/// </summary>
		/// <value> The OSCIMessage value
		/// </value>
		virtual public OSCIMessage OSCIMessage {
			get {
				return msg;
			}
		}
		virtual public Canonizer CanStream {
			get {
				return parentBuilder.hashNCanStream; 
			}
		}

		public static String SOAP_XMLNS = "http://schemas.xmlsoap.org/soap/envelope/"; 
		public const String OSCI_XMLNS = "http://www.osci.de/2002/04/osci"; 
		public const String DS_XMLNS = "http://www.w3.org/2000/09/xmldsig#"; 
		public const  String XENC_XMLNS = "http://www.w3.org/2001/04/xmlenc#"; 
		public const  String XSI_XMLNS = "http://www.w3.org/2001/XMLSchema-instance"; 
		
		private static Log log;
		
		protected internal System.Text.StringBuilder currentElement = null;
		protected internal OSCIMessage msg = null;
		protected internal bool insideHeader = false;
		protected internal bool insideBody = false;

		/// <summary>SOAP-Namespace Identifier
		/// </summary>
		public OSCIEnvelopeBuilder parentBuilder = null;
		protected System.Collections.ArrayList customSoapHeader = new System.Collections.ArrayList();
		
		/// <summary> Constructor for the OSCIMessageBuilder object
		/// </summary>
		public OSCIMessageBuilder(OSCIEnvelopeBuilder eb) {
			this.parentBuilder = eb;
		}
		
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			if (localName.Equals("Header") && uri.Equals(SOAP_XMLNS)) {
				insideBody = false;
				insideHeader = true;
			}
			else if (localName.Equals("Body")&& uri.Equals(SOAP_XMLNS)) {
				insideBody = true;
				insideHeader = false;
				msg.bodyId = attributes.getValue("Id");
			}
			else
				new org.xml.sax.SAXException("Nicht vorgesehens Element! Element Name:" + localName);
		}

		protected void startCustomSoapHeader(String uri, String localName, String qName, Attributes attributes) {
			try {
				de.osci.helper.CanParser cp = new de.osci.helper.CanParser(customSoapHeader, parentBuilder.xmlReader,
					parentBuilder.xmlReader.getContentHandler(), qName);
				parentBuilder.xmlReader.setContentHandler(cp);
				cp.startDocument();
				cp.startElement(uri, localName, qName, attributes);
			}
			catch (Exception ex) {
				throw new SAXException(de.osci.osci12.common.DialogHandler.text.getString("sax_exception_customheader"), ex);
			}
		}

		public override void endElement(String uri, String localName, String qName){
			if (localName.Equals("Header")&& uri.Equals(SOAP_XMLNS))
				insideHeader = false;
			else if (localName.Equals("Body")&& uri.Equals(SOAP_XMLNS)){
				insideBody = false;
			}
			else if (localName.Equals("Envelope")&& uri.Equals(SOAP_XMLNS)) {
				parentBuilder.xmlReader.setContentHandler(parentBuilder);
				for (int i = 0; i < customSoapHeader.Count; i++)
					msg.addCustomHeader((String)customSoapHeader[i]);
			}
			currentElement = null;
		}
		
		public override void  characters(char[] ch, int start, int length) {
			log.debug("Character: " + new System.String(ch, start, length));
			if (currentElement == null) {
				for (int i = 0; i < length; i++) {
					if (ch[start + i] > ' '){
						throw new SAXException(de.osci.osci12.common.DialogHandler.text.getString("unexpected_char"));
					}
				}
			}
			else currentElement.Append(ch, start, length);
		}

		static OSCIMessageBuilder() {
			log = LogFactory.getLog(typeof(OSCIMessageBuilder));
		}
	}
}