namespace de.osci.osci12.messagetypes  {

	using	System;
	using	org.apache.commons.logging;
	using	de.osci.osci12.common;
	using de.osci.osci12.dotcommon;
	using	de.osci.osci12.roles;
	using	de.osci.osci12.soapheader;
	using de.osci.helper;

	/// <summary> Diese	Klasse ist die	Superklasse	aller	OSCI-Auftragsnachrichtenobjekte.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary> 
	public abstract class OSCIRequest:OSCIMessage {

		/// <todo>falls ein statisches Parserobjekt unperformant	ist, 
		/// evtl. eine getInstance()-Methode in den	Parser einbauen
		/// </todo>
		internal	static IncomingMSGParser parser;
		new private	static Log log;
		protected internal String uriReceiver;

		/// <summary> Diese Methode liefert	die im DialogHandler	gesetzte	Liste	der gewnschten Sprachen.
		/// </summary>
		/// <value> Liste der Sprachkrzel, getrennt durch Leerzeichen,z.B. "de	en-US	fr"
		/// </value>
		/// <seealso href="de.osci.osci12.common.DialogHandler.LanguageList()">
		/// </seealso>
		virtual public	System.String DesiredLanguages {
			get {
				return dialogHandler.LanguageList;
			}
		}

		internal OSCIRequest() {
		}
		
		internal	OSCIRequest(DialogHandler dh):base(dh) {
			desiredLanguagesH	= new	DesiredLanguagesH(dh.LanguageList);
		}
		
		public override void writeXML(System.IO.Stream	out_Renamed) {
			base.writeXML(out_Renamed);
		}
		
		// --> Storestreams:
		protected internal virtual	OSCIMessage	transmit(System.IO.Stream outp, System.IO.Stream inp) {
			de.osci.osci12.extinterfaces.TransportI transport = dialogHandler.TransportModule.newInstance();
			bool intermed = ((this is AcceptDelivery)	||	(this	is	ProcessDelivery));
			System.Uri uri;

			if	(!intermed) {
				uri =	((de.osci.osci12.roles.Intermed)	dialogHandler.Supplier).Uri;
			} 
			else {
				uri =	new Uri(uriReceiver);
			}

			System.IO.MemoryStream out_Renamed = new System.IO.MemoryStream();				
			StoreOutputStream sos = null;

			if	(dialogHandler.CreateSignatures) {
				sign();
			}
			if	(dialogHandler.Encryption) {
				new SOAPMessageEncrypted(this, outp).writeXML(out_Renamed);
			} else {
				if (outp != null) {
					sos = new StoreOutputStream(out_Renamed, outp);
					writeXML(sos);
				} else {
					writeXML(out_Renamed);
				}
			}

			System.IO.Stream out_Renamed2	= transport.getConnection(uri, out_Renamed.Length);
			out_Renamed.WriteTo(out_Renamed2);

			if (sos != null) sos.Close();
			out_Renamed.Close();
			out_Renamed2.Close();
			System.IO.Stream rspStream = transport.ResponseStream;
			OSCIMessage ret;
			try
			{
				ret = parser.parseStream(rspStream, 
					dialogHandler, 
					false, 
					inp);
			}
			finally
			{
				rspStream.Close();
			}
			return ret;

		}
		// <-- Storestreams:

		/// <summary> Bringt	eine Client-Signatur	an.
		/// </summary>
		internal	virtual void  sign() {
			base.sign(dialogHandler.Client);
		}

		static OSCIRequest() {
			log =	LogFactory.getLog(typeof(OSCIRequest));
			parser =	new PassiveRecipientParser();
		}
	}
}