namespace de.osci.osci12.messagetypes {

	using System;
	using java.security.cert;
	using org.apache.commons.logging;
	using org.xml.sax;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.soapheader;

	/// <summary><H4>ProcessDelivery-Parser</H4>
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class ProcessDeliveryBuilder:OSCIMessageBuilder {

		new private static Log log;
		private ProcessCardBundleBuilder processCardBuilder = null;
		
		/// <summary> Konstruktutor fr den aufrufenden Mime-Parser
		/// </summary>
		/// <param name="xmlReader"> XML-Reader fr den ContentHandler
		/// </param>
		/// <param name="parentBuilder"> Parent Object in den meisten Fllen der OSCIEnvelopeBuilder
		/// </param>
		/// <param name="dh"> DialogHandler zum berprfen der Dialoginformationen
		/// </param>
		public ProcessDeliveryBuilder(OSCIEnvelopeBuilder parentBuilder):base(parentBuilder) {
			log.debug("Konstruktor");
			msg = new de.osci.osci12.messagetypes.ProcessDelivery();
		}
		
		/// <summary> berschreibt die character Methode des DefaultHandler
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <param name="attributes">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			try {
				log.debug("Start-Element: " + localName);
				if (insideBody) {
					if (localName.ToUpper().Equals("ContentPackage".ToUpper())&& uri.Equals(OSCI_XMLNS)) {
						parentBuilder.xmlReader.setContentHandler(new ContentPackageBuilder(this));
					}
				}
				else if (insideHeader) {
					log.debug("inside Header");
					// ### Auswerten des ControlBlock Headers###
					if (localName.ToUpper().Equals("ControlBlock".ToUpper())&& uri.Equals(OSCI_XMLNS)) {
						parentBuilder.xmlReader.setContentHandler(new ControlBlockHBuilder(this, attributes, new int[]{0, 1, 0, 0}));
						// ### Auswerten des ClientSignature Headers###
					}
					else if (localName.ToUpper().Equals("ClientSignature".ToUpper())&& uri.Equals(OSCI_XMLNS)) {
						parentBuilder.xmlReader.setContentHandler(new OSCISignatureBuilder(parentBuilder.xmlReader, this, attributes));
					}
					else if (localName.ToUpper().Equals("DesiredLanguages".ToUpper())&& uri.Equals(OSCI_XMLNS)) {
						parentBuilder.xmlReader.setContentHandler(new DesiredLanguagesHBuilder(this, attributes));
					}
					else if (localName.ToUpper().Equals("processDelivery".ToUpper())&& uri.Equals(OSCI_XMLNS)) {
						; 
					}
					else if (localName.Equals("MessageIdResponse")&& uri.Equals(OSCI_XMLNS)) { 
						currentElement = new System.Text.StringBuilder(); 
					} 
					else if (localName.ToUpper().Equals("ProcessCardBundle".ToUpper())&& uri.Equals(OSCI_XMLNS)) {
						int[] check = new int[]{1, 1, 0, - 1};
						processCardBuilder = new ProcessCardBundleBuilder("ProcessCardBundle", parentBuilder.xmlReader, this, attributes, check);
						parentBuilder.xmlReader.setContentHandler(processCardBuilder);
					}
					else if (localName.ToUpper().Equals("IntermediaryCertificates".ToUpper())&& uri.Equals(OSCI_XMLNS)) {
						int[] check = new int[]{- 1, - 1};
						parentBuilder.xmlReader.setContentHandler(new IntermediaryCertificatesHBuilder(this, attributes, check));
					}
					else if (localName.ToUpper().Equals("NonIntermediaryCertificates".ToUpper())&& uri.Equals(OSCI_XMLNS)) {
						int[] check = new int[]{- 1, - 1, - 1, - 1, - 1, - 1, 0};
						parentBuilder.xmlReader.setContentHandler(new NonIntermediaryCertificatesHBuilder(this, attributes, check));
						log.debug("Accept Delivery Element: " + localName);
					}
					else {
						startCustomSoapHeader(uri, localName, qName, attributes);
					}
				}
				else {
					base.startElement(uri, localName, qName, attributes);
				}
			}
			catch (CertificateException ex) {
				log.error("Fehler beim Endetag: " + localName);
			}
			catch (SAXException ex) {
				log.error("SAX Fehler:", ex);
			}
		}
		
		/// <summary> 
		/// </summary>
		/// <param name="uri"> 
		/// </param>
		/// <param name="localName"> 
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException"> 
		/// </exception>	
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("End-Element: " + localName);
			if (localName.Equals("Header")&& uri.Equals(SOAP_XMLNS)) {
				if (processCardBuilder != null)
					((ProcessDelivery) msg).processCardBundle = processCardBuilder.ProcessCardBundleObject;
				insideHeader = false;
			}
			else if (insideHeader && localName.Equals("MessageIdResponse")&& uri.Equals(OSCI_XMLNS))
				msg.messageId = MoreSupport.NewString(
					de.osci.helper.Base64.decode(currentElement.ToString()),
					de.osci.osci12.common.Constants.CHAR_ENCODING);
			else
				base.endElement(uri, localName, localName);
			currentElement = null;
		}

		static ProcessDeliveryBuilder() {
			log = LogFactory.getLog(typeof(ProcessDeliveryBuilder));
		}
	}
}