namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	
	/// <summary><p><H4>Dialogendeantwort-Nachrichtenobjekt</H4></p>
	/// Dieses Klasse reprsentiert die Antwortnachricht auf die Beendigung
	/// eines expliziten Dialogs dar.
	/// Clients erhalten vom Intermedir eine Instanz dieser Klasse als Antwort auf
	/// eine an den Intermedir gesendeten ExitDialog-Objekt.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.ExitDialog">
	/// </seealso>
	/// <seealso cref="de.osci.osci12.common.DialogHandler">
	/// </seealso>
	
	public class ResponseToExitDialog:OSCIResponseTo
	{
		new private static Log log;
		
		internal ResponseToExitDialog(DialogHandler dh):base(dh)
		{
			messageType = RESPONSE_TO_EXIT_DIALOG;
			originator = ((Originator) dh.Client);
			dialogHandler.Controlblock.Response=dialogHandler.prevChallenge;
		}
		
		internal ResponseToExitDialog():base()
		{
			messageType = RESPONSE_TO_EXIT_DIALOG;
			dialogHandler.explicitDialog = false;
		}

		public override void  compose()
		{
			base.compose();
			if (dialogHandler.Controlblock.Response == null){throw new System.SystemException("Kein Response-Wert in DialogHandler eingestellt.");}
			if (dialogHandler.Controlblock.ConversationID == null){throw new System.SystemException("Keine Conversation-ID in DialogHandler eingestellt.");}
			if (dialogHandler.Controlblock.SequenceNumber == - 1){throw new System.SystemException("Kein Squenznummer in DialogHandler eingestellt.");}
			dialogHandler.Controlblock.Challenge = null;
			dialogHandler.explicitDialog = false;
			if (feedBack == null)
			{
				throw new System.SystemException("Kein Feedback eingestellt.");
			}
			body = new Body("<" +  osciNSPrefix + ":responseToExitDialog>" + 
				writeFeedBack() + "</" +  osciNSPrefix + ":responseToExitDialog>");
			stateOfMsg |= STATE_COMPOSED;
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed)
		{
			base.writeXML(out_Renamed);
			// ClientSignatur
			if (signatureHeader != null)
			{
				signatureHeader.writeXML(out_Renamed);
				intermediaryCertificatesH.writeXML(out_Renamed);
			}
			
			completeMessage(out_Renamed);
		}
		
		static ResponseToExitDialog()
		{
			log = LogFactory.getLog(typeof(ResponseToExitDialog));
		}
	}
}