namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;

	/// <summary><p><H4>MessageId-Anforderungsantwort</H4></p>
	/// Dieses Klasse reprsentiert die Antwort des Intermedirs auf die
	/// Anforderung einer Message-ID.
	/// Clients erhalten vom Intermedir eine Instanz dieser Klasse, die eine Rckmeldung
	/// ber den Erfolg der Operation (getFeedback()) sowie ggf. die angeforderte
	/// Message-Id.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.GetMessageId">
	/// </seealso>
	public class ResponseToGetMessageId:OSCIResponseTo {

		/// <summary> Liefert die angeforderte Message-ID. Wenn der Auftrag nicht
		/// ordnungsgem abgewickelt wurde, wird <b>null</b> zurckgegeben.
		/// In disem Fall sollte der Feedback-Eintrag ausgewertet werden.
		/// </summary>
		/// <value> Message-ID bzw. null im Fehlerfall
		/// </value>
		// <seealso cref="getFeedback()">
		// </seealso>
		virtual public System.String MessageId {
			get {
				return messageId;
			}
//			set {
//				this.messageId = value;
//			}
		}

		new private static Log log;
		
		internal ResponseToGetMessageId(DialogHandler dh, bool parser):base(dh) {
			messageType = RESPONSE_TO_GET_MESSAGE_ID;
			originator = ((Originator) dh.Client);
			
			if (!parser) {
				dialogHandler.Controlblock.Response = dialogHandler.prevChallenge;
				dialogHandler.Controlblock.Challenge = Tools.createRandom(10);
			}
		}

		internal ResponseToGetMessageId(DialogHandler dh):this(dh, false) {
		}

		public override void  compose() 
		{
			base.compose();
			if (dialogHandler.Controlblock.SequenceNumber == - 1) 
			{
				throw new System.SystemException("Kein Squenznummer in DialogHandler eingestellt.");
			}
			if (feedBack == null) {
				throw new System.SystemException("Kein Feedback eingestellt.");
			}
			System.Text.StringBuilder bd = new System.Text.StringBuilder ("<"); 
			bd.Append(osciNSPrefix); 
			bd.Append(":responseToGetMessageId>"); 
			bd.Append(writeFeedBack()); 
	
			if ((messageId != null) && (messageId.Length>0))
			{
				bd.Append("<"); 
				bd.Append(osciNSPrefix); 
				bd.Append(":MessageId>"); 
				bd.Append(de.osci.helper.Base64.encode(MoreSupport.ToByteArray(messageId,Constants.CHAR_ENCODING))); 
				bd.Append("</"); 
				bd.Append(osciNSPrefix); 
				bd.Append(":MessageId>"); 
			} 
			bd.Append("</"); 
			bd.Append(osciNSPrefix); 
			bd.Append(":responseToGetMessageId>"); 
			body = new Body(bd.ToString()); 
			body.setNSPrefixes(this); 

			stateOfMsg |= STATE_COMPOSED;
		}

		public override void  writeXML(System.IO.Stream out_Renamed) {
			base.writeXML(out_Renamed);
			// ClientSignatur
			if (signatureHeader != null) {
				signatureHeader.writeXML(out_Renamed);
				intermediaryCertificatesH.writeXML(out_Renamed);
			}
			completeMessage(out_Renamed);
		}

		static ResponseToGetMessageId() {
			log = LogFactory.getLog(typeof(ResponseToGetMessageId));
		}
	}
}