namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	
	/// <summary><p><H4>Dialoginitialisierungsantwort-Nachrichtenobjekt</H4></p>
	/// Diese Klasse reprsentiert die Antwortnachricht auf die Initialisierung
	/// eines expliziten Dialogs dar.
	/// Clients erhalten vom Intermedir eine Instanz dieser Klasse als Antwort auf
	/// eine an den Intermedir gesendeten InitDialog-Objekt. Da der an den Konstruktor
	/// des InitDialog-Objektes bergebene DialogHandler bereits beim Empfang der
	/// Antwortnachricht mit den empfangenen Parametern aktualisiert wird, ist dieses
	/// Objekt fr Client-Anwendungen normalerweise ohne Bedeutung.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.InitDialog">
	/// </seealso>
	/// <seealso cref="de.osci.osci12.common.DialogHandler">
	/// </seealso>
	
	public class ResponseToInitDialog:OSCIResponseTo
	{
		new private static Log log;
		
		protected internal ResponseToInitDialog(DialogHandler dh):base(dh)
		{
			messageType = RESPONSE_TO_INIT_DIALOG;
			dialogHandler.explicitDialog = true;
		}
		
		protected internal ResponseToInitDialog(InitDialog iD):base(iD.dialogHandler)
		{
			messageType = RESPONSE_TO_INIT_DIALOG;
			dialogHandler.Controlblock.Response = dialogHandler.prevChallenge;
			dialogHandler.Controlblock.Challenge = Tools.createRandom(10);
			dialogHandler.Controlblock.SequenceNumber = - 1;
			setFeedback(new System.String[]{"0801"});
		}
		
		public override void  compose()
		{
			base.compose();
			if (feedBack == null)
			{
				throw new System.SystemException("Kein Feedback eingestellt.");
			}
			//body = new Body("<osci:responseToInitDialog>" + writeFeedBack() + "</osci:responseToInitDialog>");
			System.Text.StringBuilder bd = new System.Text.StringBuilder("<"); 
			bd.Append(osciNSPrefix); 
			bd.Append(":responseToInitDialog>"); 
			bd.Append(writeFeedBack()); 
			bd.Append("</"); 
			bd.Append(osciNSPrefix); 
			bd.Append(":responseToInitDialog>"); 
			body = new Body(bd.ToString()); 
			body.setNSPrefixes(this); 
			dialogHandler.explicitDialog = true;
			stateOfMsg |= STATE_COMPOSED;
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed)
		{
			base.writeXML(out_Renamed);
			// ClientSignatur
			if (signatureHeader != null)
			{
				signatureHeader.writeXML(out_Renamed);
				intermediaryCertificatesH.writeXML(out_Renamed);
			}
			
			completeMessage(out_Renamed);
		}
		
		static ResponseToInitDialog()
		{
			log = LogFactory.getLog(typeof(ResponseToInitDialog));
		}
	}
}