namespace de.osci.osci12.messagetypes{

	using System;
	using java.security.cert;
	using org.apache.commons.logging;
	using org.xml.sax;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.soapheader;
	
	/// <summary><H4>ResponseToInitDialog-Parser</H4>
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class ResponseToInitDialogBuilder:OSCIMessageBuilder {
		new private static Log log;
		/// <summary>  Objekt Feedback fr Feedback Information
		/// </summary>
		private FeedbackBuilder feedbackBuilder = null;
		
		/// <summary>  Constructor for the ResponseToForwardDeliveryBuilder object
		/// </summary>
		/// <param name="parentHandler"> Description of Parameter
		/// </param>
		public ResponseToInitDialogBuilder(OSCIEnvelopeBuilder parentHandler):base(parentHandler) {
			msg = new de.osci.osci12.messagetypes.ResponseToInitDialog(parentHandler.dhHandler);
		}
		
		/// <summary> 
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <param name="attributes">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			log.debug("Start Element RspFwdDel: " + localName);
			if (localName.Equals("Header")&& uri.Equals(SOAP_XMLNS)) {
				insideBody = false;
				insideHeader = true;
			}
			else if (localName.Equals("Body")&& uri.Equals(SOAP_XMLNS)) {
				insideBody = true;
				insideHeader = false;
			}
			else if (insideBody) {
				if (localName.Equals("Feedback")&& uri.Equals(OSCI_XMLNS)) {
					feedbackBuilder = new FeedbackBuilder(this);
					parentBuilder.xmlReader.setContentHandler(feedbackBuilder);
				}
			}
			else if (insideHeader) {
				// ### Auswerten des ControlBlock Headers###
				if (localName.Equals("ControlBlock")&& uri.Equals(OSCI_XMLNS)) {
					parentBuilder.xmlReader.setContentHandler(new ControlBlockHBuilder(this, attributes, new int[]{1, 1, 1, 0}));
				}
					// ### Auswerten des ClientSignature Headers###
				else if (localName.Equals("SupplierSignature")&& uri.Equals(OSCI_XMLNS)) {
					parentBuilder.xmlReader.setContentHandler(new OSCISignatureBuilder(parentBuilder.xmlReader, this, attributes));
				}
				else if (localName.Equals("responseToInitDialog")&& uri.Equals(OSCI_XMLNS)) {
					;
					//nothing to do
				}
				else if (localName.Equals("IntermediaryCertificates")&& uri.Equals(OSCI_XMLNS)) {
					int[] check = new int[]{0, - 1};
					parentBuilder.xmlReader.setContentHandler(new IntermediaryCertificatesHBuilder(this, attributes, check));
				}
				else {
					startCustomSoapHeader(uri, localName, qName, attributes);
				}
			}
			else {
				new org.xml.sax.SAXException("Nicht vorgesehens Element! Element Name:" + localName);
			}
		}
		
		/// <summary> 
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			if (log.DebugEnabled) log.debug("End-Element: " + localName);
			if (localName.Equals("Body")&& uri.Equals(SOAP_XMLNS)) insideBody = false;
			else	base.endElement(uri, localName, qName);
			currentElement = null;
		}
		static ResponseToInitDialogBuilder() {
			log = LogFactory.getLog(typeof(ResponseToForwardDeliveryBuilder));
		}
	}
}