namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.dotcommon; // Storestreams
	using de.osci.osci12.encryption;
	using de.osci.osci12.roles;

	/// <summary>Diese Klasse entspricht einer Verschlsselten OSCI-Nachricht.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class SOAPMessageEncrypted:OSCIMessage {

		new private static Log log;
		new public de.osci.osci12.encryption.EncryptedData encryptedData;
		private OSCIMessage msg;
		private javax.crypto.SecretKey symKey;
		private byte[] encSymKey;
		private System.IO.IOException ioex;
		// --> Storestreams:
		private System.IO.Stream storeStream;
		// <-- Storestreams:

		private static byte[] xml_0;
		private static byte[] xml_1;
		private static byte[] xml_2;
		private static byte[] xml_3;		

		// --> Storestreams:
		public SOAPMessageEncrypted(OSCIMessage msg, System.IO.Stream storeStream) {
            this.storeStream = storeStream;
            // <-- Storestreams:

			messageType = SOAP_MESSAGE_ENCRYPTED;
            this.msg = msg;
            if (msg != null)
			    symKey = de.osci.osci12.encryption.Crypto.createSymKey(msg.DialogHandler.SymmetricCipherAlgorithm);
		}
				
		public override void  compose() 
		{
			if ((msg.stateOfMsg & STATE_COMPOSED) == 0)
				msg.compose();

			if (msg is OSCIRequest) 
			{
				encSymKey = MoreSupport.ToByteArray( Base64.encode(Crypto.doRSAEncryption(msg.DialogHandler.Supplier.CipherCertificate, symKey.Encoded())),Constants.CHAR_ENCODING);
			} else {
				encSymKey = MoreSupport.ToByteArray( Base64.encode(Crypto.doRSAEncryption(msg.DialogHandler.Client.CipherCertificate, symKey.Encoded())),Constants.CHAR_ENCODING);
			}
			stateOfMsg |= STATE_COMPOSED;
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed) 
		{
			if ((stateOfMsg & STATE_COMPOSED) == 0)
				compose();

			setFragm();
			byte[] algo = SupportClass.ToByteArray(msg.DialogHandler.SymmetricCipherAlgorithm);
			long length = xml_0.Length +  algo.Length + xml_1.Length + xml_2.Length + encSymKey.Length + xml_3.Length;
						
			MoreSupport.ToStream(out_Renamed,"MIME-Version: 1.0\r\nContent-Type: Multipart/Related; boundary=" + msg.boundary_string + "; type=text/xml\r\n",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,"\r\n--" + msg.boundary_string + "\r\nContent-Type: text/xml; charset=UTF-8\r\n",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,"Content-Transfer-Encoding: 8bit\r\nContent-ID: <" + contentID + ">\r\n",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,"Content-Length: " + length + "\r\n\r\n",Constants.CHAR_ENCODING);
			
			out_Renamed.Write(xml_0, 0, xml_0.Length);
			out_Renamed.Write(algo, 0, algo.Length);
			out_Renamed.Write(xml_1, 0, xml_1.Length);

			if (msg is OSCIRequest) {
				MoreSupport.ToStream(out_Renamed,de.osci.helper.Base64.encode(msg.DialogHandler.Supplier.CipherCertificate.getEncoded()),Constants.CHAR_ENCODING);
			}
			else {
				MoreSupport.ToStream(out_Renamed, de.osci.helper.Base64.encode(msg.DialogHandler.Client.CipherCertificate.getEncoded()),Constants.CHAR_ENCODING);
			}

			byte[] temp_byteArray6;
			temp_byteArray6 = xml_2;
			out_Renamed.Write(temp_byteArray6, 0, temp_byteArray6.Length);;;
			byte[] temp_byteArray7;
			temp_byteArray7 = encSymKey;
			out_Renamed.Write(temp_byteArray7, 0, temp_byteArray7.Length);;;
			byte[] temp_byteArray8;
			temp_byteArray8 = xml_3;
			out_Renamed.Write(temp_byteArray8, 0, temp_byteArray8.Length);;;			

			MoreSupport.ToStream(out_Renamed, "\r\n\r\n--" + msg.boundary_string + "\r\nContent-Type: ", Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed, msg.Base64Encoding ? "text/base64" : "application/octet-stream", Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed, "\r\nContent-Transfer-Encoding: ",Constants.CHAR_ENCODING);			
			MoreSupport.ToStream(out_Renamed, msg.Base64Encoding ? "7bit" : "binary",Constants.CHAR_ENCODING);			
			MoreSupport.ToStream(out_Renamed, "\r\nContent-ID: <osci_enc>\r\n\r\n",Constants.CHAR_ENCODING);			

		 Base64OutputStream b64out = null;
 	     SymCipherOutputStream tdesOut = null;
 	     if (msg.Base64Encoding)
 	     {
 	       b64out = new Base64OutputStream(out_Renamed, false);
 	       tdesOut = new SymCipherOutputStream(b64out, symKey, true);
 	     }
 	     else
 	       tdesOut = new SymCipherOutputStream(out_Renamed, symKey, true);
 	 
			// --> Storestreams
			if (storeStream == null) 
			{
				msg.writeXML(tdesOut);
				tdesOut.Close();
			}
			else 
			{
				StoreOutputStream sos = new StoreOutputStream(tdesOut, storeStream);
				msg.writeXML(sos);
				sos.Close();
			}

			if (msg.Base64Encoding)
				b64out.Flush();
					
		// <-- Storestreams

			MoreSupport.ToStream(out_Renamed,"\r\n--" + msg.boundary_string + "--\r\n",Constants.CHAR_ENCODING);
		}
		
		SOAPMessageEncrypted() 
		{
			log = LogFactory.getLog(typeof(SOAPMessageEncrypted));
		setFragm();
		}
		protected void setFragm() 
		{
			xml_0 = MoreSupport.ToByteArray("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n\r\n<soap:Envelope xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xenc=\"http://www.w3.org/2001/04/xmlenc#\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://schemas.xmlsoap.org/soap/envelope/ soapMessageEncrypted.xsd http://www.w3.org/2000/09/xmldsig# oscisig.xsd http://www.w3.org/2001/04/xmlenc# oscienc.xsd\"><soap:Body><xenc:EncryptedData Id=\"Attachment\" MimeType=\"text/xml\"><xenc:EncryptionMethod Algorithm=\"",Constants.CHAR_ENCODING);
			xml_1 = MoreSupport.ToByteArray("\"></xenc:EncryptionMethod><ds:KeyInfo><xenc:EncryptedKey Id=\"EncData__0\"><xenc:EncryptionMethod Algorithm=\"http://www.w3.org/2001/04/xmlenc#rsa-1_5\"></xenc:EncryptionMethod><"+dsNSPrefix+":KeyInfo><"+dsNSPrefix+":X509Data><"+dsNSPrefix+":X509Certificate>",Constants.CHAR_ENCODING);
			xml_2 = MoreSupport.ToByteArray("</ds:X509Certificate></ds:X509Data></ds:KeyInfo><xenc:CipherData><xenc:CipherValue>",Constants.CHAR_ENCODING);
			xml_3 = MoreSupport.ToByteArray("</xenc:CipherValue></xenc:CipherData></xenc:EncryptedKey></"+dsNSPrefix+":KeyInfo><xenc:CipherData><xenc:CipherReference URI=\"cid:osci_enc\"><xenc:Transforms><ds:Transform Algorithm=\"http://www.w3.org/2000/09/xmldsig#base64\"></"+dsNSPrefix+":Transform></xenc:Transforms></xenc:CipherReference></xenc:CipherData></xenc:EncryptedData></soap:Body></soap:Envelope>",Constants.CHAR_ENCODING);
		}
	}
}