namespace de.osci.osci12.messagetypes {
	
	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using de.osci.osci12.encryption;

	/// <summary><H4>SOAPMessageEncrypted-Parser</H4>
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class SOAPMessageEncryptedBuilder:OSCIMessageBuilder {

		new private static Log log;
		private EncryptedDataBuilder edb;
		
		/// <summary> Constructor for the SOAPMessageEncryptedBuilder object
		/// </summary>
		/// <param name="eb">
		/// </param>
		public SOAPMessageEncryptedBuilder(OSCIEnvelopeBuilder eb):base(eb) {
			log.debug("Konstruktor");
			msg = new SOAPMessageEncrypted(null, null);
		}
		
		/// <summary> 
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <param name="attributes">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			log.debug("Start-Element: " + localName);
			if (localName.ToUpper().Equals("EncryptedData".ToUpper())&& uri.Equals(XENC_XMLNS)) {
				edb = new EncryptedDataBuilder(parentBuilder.xmlReader, this, attributes);
				parentBuilder.xmlReader.setContentHandler(edb);
			}
			else if (localName.ToLower().Equals("body")&& uri.Equals(SOAP_XMLNS)) {
				// nothing to do
			}
			else {
				throw new org.xml.sax.SAXException("Unerwartetes Element in SOAPMessageEncrypted: " + localName);
			}
		}
		
		public override void  endDocument() {
			log.debug("End-Document SOAP");
			((SOAPMessageEncrypted) msg).encryptedData = edb.EncryptedData;
		}
		
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("End-Element: " + localName);
		}

		static SOAPMessageEncryptedBuilder() {
			log = LogFactory.getLog(typeof(SOAPMessageEncryptedBuilder));
		}
	}
}