namespace de.osci.osci12.messagetypes {

	using System;
	using de.osci.osci12.common;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.soapheader;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	using DefaultHandler = org.xml.sax.helpers.DefaultHandler;
	using java.security.cert;

	/// <summary><H4>StoredEnvelope-Parser</H4>
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>

	class StoredEnvelopeBuilder:OSCIEnvelopeBuilder {

		new private static Log log;
		
		public StoredEnvelopeBuilder(XMLReader xmlReader):base(xmlReader, null) {
		}

		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			if (log.DebugEnabled)
				log.debug("Start-Element: " + qName);
			// SOAP-Namespacedefinition suchen
  if ((soapNSPrefix == null) || !localName.Equals("Envelope")) 
              throw new SAXException(DialogHandler.text.getString("msg_format_error") + ": " + localName); 
				System.String schemaLocation = attributes.getValue("xsi:schemaLocation");
				
				if (log.DebugEnabled)
					log.debug("Nachrichtentyp: " + schemaLocation);
				
				if (schemaLocation == null) {
					childBuilder = new SOAPFaultBuilder(this);
					this.xmlReader.setContentHandler(childBuilder);
				} else if (schemaLocation.Equals(StoredEnvelopeBuilder.XSD_ENCRYPTED_DATA)) {
					throw new SAXException(new System.ArgumentException(DialogHandler.text.getString("encrypted_message")));
				} else {
					int msgType;
					
					if (schemaLocation.Equals(StoredEnvelopeBuilder.XSD_STORE_DELIVERY))
						msgType = OSCIMessage.STORE_DELIVERY;
					else if (schemaLocation.Equals(StoredEnvelopeBuilder.XSD_RSP_STORE_DELIVERY))
						msgType = OSCIMessage.RESPONSE_TO_STORE_DELIVERY;
					else if (schemaLocation.Equals(StoredEnvelopeBuilder.XSD_FETCH_DELIVERY))
						msgType = OSCIMessage.FETCH_DELIVERY;
					else if (schemaLocation.Equals(StoredEnvelopeBuilder.XSD_RSP_FETCH_DELIVERY))
						msgType = OSCIMessage.RESPONSE_TO_FETCH_DELIVERY;
					else if (schemaLocation.Equals(StoredEnvelopeBuilder.XSD_FORWARD_DELIVERY))
						msgType = OSCIMessage.FORWARD_DELIVERY;
					else if (schemaLocation.Equals(StoredEnvelopeBuilder.XSD_ACCEPT_DELIVERY))
						msgType = OSCIMessage.ACCEPT_DELIVERY;
					else if (schemaLocation.Equals(StoredEnvelopeBuilder.XSD_RSP_ACCEPT_DELIVERY))
						msgType = OSCIMessage.RESPONSE_TO_ACCEPT_DELIVERY;
					else if (schemaLocation.Equals(StoredEnvelopeBuilder.XSD_RSP_FORWARD_DELIVERY))
						msgType = OSCIMessage.RESPONSE_TO_FORWARD_DELIVERY;
					else if (schemaLocation.Equals(StoredEnvelopeBuilder.XSD_MEDIATE_DELIVERY))
						msgType = OSCIMessage.MEDIATE_DELIVERY;
					else if (schemaLocation.Equals(StoredEnvelopeBuilder.XSD_PROCESS_DELIVERY))
						msgType = OSCIMessage.PROCESS_DELIVERY;
					else if (schemaLocation.Equals(StoredEnvelopeBuilder.XSD_RSP_PROCESS_DELIVERY))
						msgType = OSCIMessage.RESPONSE_TO_PROCESS_DELIVERY;
					else if (schemaLocation.Equals(StoredEnvelopeBuilder.XSD_RSP_MEDIATE_DELIVERY))
						msgType = OSCIMessage.RESPONSE_TO_MEDIATE_DELIVERY;
					else if (schemaLocation.Equals(StoredEnvelopeBuilder.XSD_FETCH_PROCESS_CARD))
						msgType = OSCIMessage.FETCH_PROCESS_CARD;
					else if (schemaLocation.Equals(StoredEnvelopeBuilder.XSD_RSP_FETCH_PROCESS_CARD))
						msgType = OSCIMessage.RESPONSE_TO_FETCH_PROCESS_CARD;
					else {
						log.error("Nicht erlaubter Nachrichtentyp ! Nachrichtentypt ist:\n" + schemaLocation + "\n" + (StoredEnvelopeBuilder.XSD_RSP_FETCH_PROCESS_CARD));
						throw new SAXException(DialogHandler.text.getString("sax_exception_msgtype"));
					}
					
					childBuilder = new StoredMessageBuilder(this, msgType);
					this.xmlReader.setContentHandler(childBuilder);
			}
			childBuilder.OSCIMessage.soapNSPrefix = soapNSPrefix; 
			childBuilder.OSCIMessage.osciNSPrefix = osciNSPrefix; 
			childBuilder.OSCIMessage.dsNSPrefix = dsNSPrefix; 
			childBuilder.OSCIMessage.xencNSPrefix = xencNSPrefix; 
			log.debug("\n\n\nBUFF" +ns.ToString()); 
			childBuilder.OSCIMessage.ns = ns.ToString(); 
		}

		static StoredEnvelopeBuilder() {
			log = LogFactory.getLog(typeof(StoredEnvelopeBuilder));
		}
	}
}