namespace de.osci.osci12.messagetypes {

	using System;
	using de.osci.osci12.common;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.soapheader;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	using DefaultHandler = org.xml.sax.helpers.DefaultHandler;
	using java.security.cert;

	/// <summary><H4>StoredMessage-Parser</H4>
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	class StoredMessageBuilder:OSCIMessageBuilder {

		new private static Log log;
		
		/// <summary> Objekt ProcessCardBundle fr ProcessCard Information
		/// </summary>
		private ProcessCardBundleBuilder     processCardBuilder = null;

		private System.Collections.ArrayList processCardBundles;
		private ProcessCardBundleBuilder     replyProcessCardBuilder = null;
		
		private void  InitBlock() {
			processCardBundles = new System.Collections.ArrayList();
		}

		/// <summary> Constructor for the ResponseToForwardDeliveryBuilder object
		/// </summary>
		public StoredMessageBuilder(OSCIEnvelopeBuilder parentHandler, int msgType):base(parentHandler) {
			InitBlock();
			msg = new de.osci.osci12.messagetypes.StoredMessage(msgType);
		}
		
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			if (log.DebugEnabled)
				log.debug("Start-Element: " + localName);
			
			if (localName.Equals("MessageId")&& uri.Equals(OSCI_XMLNS))
				currentElement = new System.Text.StringBuilder();
	       else if (insideHeader && localName.Equals("MessageIdResponse")
				&& uri.Equals(OSCI_XMLNS) &&
				msg.MessageType==OSCIMessage.PROCESS_DELIVERY) 
				currentElement = new System.Text.StringBuilder();
			else if (localName.Equals("SelectionRule")&& uri.Equals(OSCI_XMLNS)) {
			} else if (localName.Equals("Quantity")&& uri.Equals(OSCI_XMLNS))
				((StoredMessage) msg).quantityLimit = System.Int64.Parse(attributes.getValue("Limit"));

			else if (localName.Equals("Feedback")&& uri.Equals(OSCI_XMLNS)) {
				FeedbackBuilder feedbackBuilder = new FeedbackBuilder(this);
				parentBuilder.xmlReader.setContentHandler(feedbackBuilder);

			} else if (localName.Equals("ProcessCardBundle")&& uri.Equals(OSCI_XMLNS)) {
				if (processCardBuilder != null) {
					processCardBundles.Add(processCardBuilder.ProcessCardBundleObject);
				}
				int[] check = new int[]{1, - 1, - 1, - 1};
				processCardBuilder = new ProcessCardBundleBuilder("ProcessCardBundle", parentBuilder.xmlReader, this, attributes, check);
				parentBuilder.xmlReader.setContentHandler(processCardBuilder);

			} else if (insideBody) {
				if (localName.Equals("ContentPackage")&& uri.Equals(OSCI_XMLNS))
					parentBuilder.xmlReader.setContentHandler(MessagePartsFactory.createContentPackageBuilder(this));
			
			} else if (insideHeader) {
				// ### Auswerten des ControlBlock Headers###
				if (localName.Equals("ControlBlock")&& uri.Equals(OSCI_XMLNS))
					parentBuilder.xmlReader.setContentHandler(new ControlBlockHBuilder(this, attributes, new int[]{- 1, - 1, - 1, - 1}));
				
					// ### Auswerten des ClientSignature Headers###
				else if (localName.Equals("SupplierSignature") && uri.Equals(OSCI_XMLNS)
					|| localName.Equals("ClientSignature")&& uri.Equals(OSCI_XMLNS))
					parentBuilder.xmlReader.setContentHandler(MessagePartsFactory.createOsciSignatureBuilder(parentBuilder.xmlReader, this, attributes));
				else if (localName.Equals("DesiredLanguages")&& uri.Equals(OSCI_XMLNS)) {
					msg.desiredLanguagesH = new DesiredLanguagesH(attributes.getValue("LanguagesList"));
					msg.dialogHandler.LanguageList = attributes.getValue("LanguagesList");
					msg.desiredLanguagesH.RefID=attributes.getValue("Id"); 
					msg.desiredLanguagesH.setNSPrefixes(msg); 
				} 
				else if (localName.Equals("QualityOfTimestamp")&& uri.Equals(OSCI_XMLNS))
					parentBuilder.xmlReader.setContentHandler(new QualityOfTimestampHBuilder(this, attributes));

				else if (localName.Equals("RequestProcessCardBundle")&& uri.Equals(OSCI_XMLNS)) {
					int[] check = new int[]{- 1, - 1, - 1, - 1};
					processCardBuilder = new ProcessCardBundleBuilder("RequestProcessCardBundle", parentBuilder.xmlReader, this, attributes, check);
					parentBuilder.xmlReader.setContentHandler(processCardBuilder);
				} else if (localName.Equals("ReplyProcessCardBundle")&& uri.Equals(OSCI_XMLNS)) {
					int[] check = new int[]{- 1, - 1, - 1, - 1};
					replyProcessCardBuilder = new ProcessCardBundleBuilder("ReplyProcessCardBundle", parentBuilder.xmlReader, this, attributes, check);
					parentBuilder.xmlReader.setContentHandler(replyProcessCardBuilder);
				} else if (localName.Equals("IntermediaryCertificates")&& uri.Equals(OSCI_XMLNS)) {
					int[] check = new int[]{- 1, - 1};
					
					try {
						parentBuilder.xmlReader.setContentHandler(new IntermediaryCertificatesHBuilder(this, attributes, check));
					} catch (CertificateException ex) {
						throw new SAXException(DialogHandler.text.getString("cert_gen_error"), ex);
					}
				} else if (localName.Equals("NonIntermediaryCertificates")&& uri.Equals(OSCI_XMLNS)) {
					int[] check = new int[]{- 1, - 1, - 1, - 1, - 1, - 1, - 1};
					
					try {
						parentBuilder.xmlReader.setContentHandler(new NonIntermediaryCertificatesHBuilder(this, attributes, check));
					} catch (CertificateException ex) {
						throw new SAXException(DialogHandler.text.getString("cert_gen_error"), ex);
					}
				} else if (localName.Equals("Subject")&& uri.Equals(OSCI_XMLNS))
					currentElement = new System.Text.StringBuilder();
				else if (localName.Equals("ContentReceiver")&& uri.Equals(OSCI_XMLNS)) {
					try {
						((StoredMessage) msg).uriReceiver = new System.Uri(attributes.getValue("URI"));
					} catch (System.Exception ex) {
						throw new SAXException(ex);
					}
				} else if (localName.Equals("QualityOfTimestamp")&& uri.Equals(OSCI_XMLNS))
					parentBuilder.xmlReader.setContentHandler(new QualityOfTimestampHBuilder(this, attributes));
				else if (localName.Equals("mediateDelivery")&& uri.Equals(OSCI_XMLNS)
					|| localName.Equals("responseToMediateDelivery")&& uri.Equals(OSCI_XMLNS) 
					|| localName.Equals("storeDelivery")&& uri.Equals(OSCI_XMLNS)
					|| localName.Equals("responseToStoreDelivery")&& uri.Equals(OSCI_XMLNS)
					|| localName.Equals("fetchDelivery")&& uri.Equals(OSCI_XMLNS)
					|| localName.Equals("responseToFetchDelivery")&& uri.Equals(OSCI_XMLNS)
					|| localName.Equals("fetchProcessCard")&& uri.Equals(OSCI_XMLNS)
					|| localName.Equals("responseToFetchProcessCard")&& uri.Equals(OSCI_XMLNS)
					|| localName.Equals("acceptDelivery")&& uri.Equals(OSCI_XMLNS)
					|| localName.Equals("responseToAcceptDelivery")&& uri.Equals(OSCI_XMLNS)
					|| localName.Equals("processDelivery")&& uri.Equals(OSCI_XMLNS)
					|| localName.Equals("responseToProcessDelivery")&& uri.Equals(OSCI_XMLNS)
					) {
				} else {
					startCustomSoapHeader(uri, localName, qName, attributes);
				}
			} else
				base.startElement(uri, localName, qName, attributes);
		}
		
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			if (log.DebugEnabled)
				log.debug("End-Element: " + localName);
			
			if (localName.Equals("Envelope")&& uri.Equals(SOAP_XMLNS)) {
				StoredMessage sm = ((StoredMessage) msg);
								
				if (processCardBuilder != null) {
					processCardBundles.Add(processCardBuilder.ProcessCardBundleObject);
					sm.processCardBundles = (ProcessCardBundle[])processCardBundles.ToArray(typeof(ProcessCardBundle));
					
					if (replyProcessCardBuilder != null)
						sm.processCardBundleReply = replyProcessCardBuilder.ProcessCardBundleObject;
				}
				
				for (int i = 0; i < customSoapHeader.Count; i++)
					msg.addCustomHeader((String) customSoapHeader[i]);
				
				msg.dialogHandler.CheckSignatures = false;
				parentBuilder.xmlReader.setContentHandler(parentBuilder);
			} else if (localName.Equals("MessageId")&& uri.Equals(OSCI_XMLNS)) {
				try {
					if ((msg.MessageType == OSCIMessage.FETCH_DELIVERY) || (msg.MessageType == OSCIMessage.RESPONSE_TO_FETCH_DELIVERY) || (msg.MessageType == OSCIMessage.FETCH_PROCESS_CARD) || (msg.MessageType == OSCIMessage.RESPONSE_TO_FETCH_PROCESS_CARD)) {
						((StoredMessage) msg).selectionMode = OSCIMessage.SELECT_BY_MESSAGE_ID;
						((StoredMessage) msg).selectionRule = MoreSupport.NewString(de.osci.helper.Base64.decode(currentElement.ToString()), Constants.CHAR_ENCODING);
					} else {
						msg.messageId = MoreSupport.NewString(de.osci.helper.Base64.decode(currentElement.ToString()), Constants.CHAR_ENCODING);
					}
				} catch (System.Exception ex) {
					throw new SAXException(ex);
				}
			} else if (localName.Equals("ReceptionOfDelivery")&& uri.Equals(OSCI_XMLNS)) {
				((StoredMessage) msg).selectionMode = OSCIMessage.SELECT_BY_DATE_OF_RECEPTION;
				((StoredMessage) msg).selectionRule = currentElement.ToString();
			} else if (localName.Equals("Subject")&& uri.Equals(OSCI_XMLNS))
				((StoredMessage) msg).subject = currentElement.ToString();
			else
				base.endElement(uri, localName, qName);
			
			currentElement = null;
		}
		
		static StoredMessageBuilder() {
			log = LogFactory.getLog(typeof(StoredMessageBuilder));
		}
	}
}