namespace de.osci.osci12.roles {

	using System;
	using java.security.cert;
	using org.apache.commons.logging;
	using de.osci.osci12.extinterfaces.crypto;

	/// <summary> Diese Klasse stellt einen OSCI-Leser dar.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class Reader:Role {

		virtual public X509Certificate SignatureCertificate {
			set {
				throw new UnsupportedOperationException("Readerobjekte haben kein Signaturzertifikat.");
			}
		}
		
		new private static Log log;
		private static int idNr = - 1;
		
		/// <summary>  Konstruktor fr den Empfang einer Nachricht. Wird als Parameter
		/// null bergeben, weil keine Verschlsselung der Rckantwort gewnscht wird,
		/// mu trotzdem mit CipherCertificate(X509Certificate) ein
		/// Verschlsselungszertifikat gesetzt werden, weil dies Voraussetzung fr
		/// die Teilnahme an OSCI ist.
		/// </summary>
		/// <param name="decrypter">Decrypter-Objekt, welches den Inhalt der Nachricht entschlsseln
		/// soll.
		/// </param>
		public Reader(Decrypter decrypter) {
			this.signer = signer;
			this.decrypter = decrypter;
			idNr++;
			id += idNr;
		}
		
		/// <summary> Konstruktor fr das Versenden einer Nachricht.
		/// </summary>
		/// <param name="cipherCertificate">Zertifikat, mit dem die Nachricht verschlsselt werden soll.
		/// </param>
		public Reader(X509Certificate cipherCertificate) {
			this.cipherCertificate = cipherCertificate;
			idNr++;
			id += idNr;
		}
		
		static Reader() {
			log = LogFactory.getLog(typeof(Reader));
		}
	}
}