using System;
using java.security.cert;
using de.osci.osci12.roles;
using de.osci.osci12.messagetypes;
using de.osci.osci12.messageparts;
using de.osci.osci12.common;
using de.osci.osci12.dotcommon;
using de.osci.osci12.samples.impl.crypto;
namespace de.osci.osci12.samples
{
	/// <summary>
	/// Zusammenfassung fr OneWayMessage_ActiveRecipient.
	/// </summary>
	public class OneWayMessage_ActiveRecipient {
		Intermed intermed;

		public OneWayMessage_ActiveRecipient(String intermedURL) {//  throws Exception
			// First we need the Intermediarys Cipher-Certificate
			//    System.Security.Cryptography.X509Certificates.X509Certificate intermedCipherCert = X509Certificate.CreateFromCertFile(@"..\..\Daten\Zertifikate\Intermed.cer");
			X509Certificate intermedCipherCert;
			SupportClass.PrivateKeySupport pKeyInfo;
			RSAEncryption.LoadFromStore(out intermedCipherCert,@"..\..\Daten\Zertifikate\Intermed.cer","123456",false, out pKeyInfo);

			// Create the Intermed-role object
			intermed = new Intermed(null, intermedCipherCert, new java.net.URI(intermedURL));
		}

		public ResponseToStoreDelivery sendStoreDelivery() {// throws Exception
			// Create the Originator-object. Since we have the private key(s), we use this
			// Constructor. We need a Decrypter-object for decryption of response
			Originator user_1 = new Originator(new PKCS12Signer(@"..\..\Daten\Zertifikate\Sender1.p12","123456"), new PKCS12Decrypter(@"..\..\Daten\Zertifikate\Sender1.p12","123456"));
			// Create a DialogHandler
			DialogHandler clientDialog = new DialogHandler(user_1, intermed, new de.osci.osci12.samples.impl.HttpTransport());

			// Default configuration is transport encryption and signature activated
			clientDialog.CreateSignatures=true;
			clientDialog.Encryption=true;

			// Create the GetMessageId-Request
			GetMessageId getMsgID = new GetMessageId(clientDialog);

			// Transmit the request and receive the response
			ResponseToGetMessageId rsp2GetMsgID = getMsgID.send();

			// Error handling
			if (!rsp2GetMsgID.Feedback[0][1].StartsWith("0")) {
				// do error handling
			}

			// Now we have a message-Id, we are ready to send the StoreDelivery-request
			// First we need to create the Addressee-object representing the user 2.
			// The signature certificate is not needed since we do not
			// expect a signed response from the addressee in this secenario
			// (it would be included in a signed response anyway).
			java.security.cert.X509Certificate addresseecert;
			SupportClass.PrivateKeySupport pKeyInfo;
			RSAEncryption.LoadFromStore(out addresseecert, @"../../Daten/Zertifikate/Empfaenger.cer","123456",false, out pKeyInfo); //Intermed.cer
			Addressee user_2 = new Addressee(null,addresseecert);

			// Create the StoreDelivery-message object
            StoreDelivery storeDel = new StoreDelivery(clientDialog, user_2, rsp2GetMsgID.MessageId);//, rsp2GetMsgID.MessageId);
			storeDel.Subject="Subject";

			// We trust the intermediary's time stamp
			storeDel.QualityOfTimeStampCreation=false;
			storeDel.QualityOfTimeStampReception=false;

			// Add not encrypted Content-Data
			ContentContainer not_encrypted_container = new ContentContainer();
			not_encrypted_container.addContent(new Content("Any content data."));

			System.IO.FileStream fis=new System.IO.FileStream(@"../../Daten/Testdaten/a.txt",System.IO.FileMode.Open,System.IO.FileAccess.Read);
			Attachment att =new Attachment(fis,"9999");
			Content co=new Content(att);
			not_encrypted_container.addContent(co);
			storeDel.addContentContainer(not_encrypted_container);
			

			// Add encrypted Content-Data
			java.security.cert.X509Certificate readercert;
			RSAEncryption.LoadFromStore(out readercert, @"../../Daten/Zertifikate/Leser1.cer","123456",false, out pKeyInfo); //Intermed.cer
			Reader reader=new Reader(readercert);
			ContentContainer encrypted_container = new ContentContainer();
			encrypted_container.addContent(new Content("Any encrypted content data."));

			fis=new System.IO.FileStream(@"../../Daten/Testdaten/a.txt",System.IO.FileMode.Open,System.IO.FileAccess.Read);
			Attachment atten =new Attachment(fis,"9998", null);
			Content coen=new Content(atten);
			encrypted_container.addContent(coen);

			EncryptedDataOSCI encryptedData = new EncryptedDataOSCI(null, Constants.SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC, encrypted_container);
			encryptedData.encrypt(reader);
			storeDel.addEncryptedData(encryptedData);

			// Transmit the request and receive the response
			ResponseToStoreDelivery rsp2StoreDel = storeDel.send();

			if (!rsp2StoreDel.Feedback[0][1].StartsWith("0")) {
				// Error handling
			}

			// Optional: Check process card bundle (result of certificate check etc.)
			Inspection[] insp=rsp2StoreDel.Inspections;
			Timestamp ts=rsp2StoreDel.TimestampCreation;
			String rm=rsp2StoreDel.RecentModification;

			// ....
			return rsp2StoreDel;
		}

		public ResponseToFetchDelivery sendFetchDelivery(String messageId) {//  throws Exception
			// Create the Originator-object. Since we have the private key(s), we use this
			// Constructor. We need a Decrypter-object for decryption of response
            Originator user_2 = new Originator(new PKCS12Signer(@"..\..\Daten\Zertifikate\Empfaenger.p12", "123456"), new PKCS12Decrypter(@"..\..\Daten\Zertifikate\Empfaenger.p12", "123456"));

			// Create a DialogHandler
			// Default configuration is transport encryption and signature activated
			DialogHandler clientDialog = new DialogHandler(user_2, intermed, new de.osci.osci12.samples.impl.HttpTransport());

			// Default configuration is transport encryption and signature activated
			clientDialog.CreateSignatures=true;
			clientDialog.Encryption=true;

			// Create the InitDialog-Request
			InitDialog id = new InitDialog(clientDialog);

			// Transmit the request and receive the response
			ResponseToInitDialog rsp2InitDialog = id.send();

			// Error handling
			if (!rsp2InitDialog.Feedback[0][1].StartsWith("0")) {
				// Error handling
			}

			// Now the dialog is initialized. We can authenticate ourselves by
			// returning the challenge, so we are ready to send the FetchDelivery-request
			// Create the FetchDelivery-message object
			FetchDelivery fetchDel = new FetchDelivery(clientDialog);
			fetchDel.SelectionMode=OSCIMessage.SELECT_BY_MESSAGE_ID;
			fetchDel.SelectionRule=messageId;

			// Transmit the request and receive the response
			ResponseToFetchDelivery rsp2FetchDel = fetchDel.send();

			if (!rsp2FetchDel.Feedback[0][1].StartsWith("0")) {
				// Error handling
			}

			// Optional: Check process card bundle (result of certificate check etc.)
			//    rsp2FetchDel.getInspections();
			//    rsp2FetchDel.getTimestampCreation();
			//    rsp2FetchDel.getRecentModification();
			//    rsp2FetchDel.getContentContainer();
			//    rsp2FetchDel.getAttachments();
			// ....
			// Cleanup - Create the ExitDialog-Request
			ExitDialog ed = new ExitDialog(clientDialog);

			// Transmit the request and receive the response
			ResponseToExitDialog rsp2ExitDialog = ed.send();

			return rsp2FetchDel;
		}

		public ResponseToFetchProcessCard sendFetchProcessCard(String messageId) {// throws Exception
			// Create the Originator-object.
			Originator user_1 = new Originator(new PKCS12Signer(@"..\..\Daten\Zertifikate\Empfaenger.p12","123456"),new PKCS12Decrypter(@"..\..\Daten\Zertifikate\Empfaenger.p12","123456"));

			// Create a DialogHandler
			DialogHandler clientDialog = new DialogHandler(user_1, intermed, new de.osci.osci12.samples.impl.HttpTransport());
			clientDialog.Encryption=true;

			// Create the InitDialog-Request
			InitDialog id = new InitDialog(clientDialog);

			// Transmit the request and receive the response
			ResponseToInitDialog rsp2InitDialog = id.send();

			// Error handling
			if (!rsp2InitDialog.Feedback[0][1].StartsWith("0")) {
				// do error handling
			}

			// Now the dialog is initialized. We can authenticate ourselves by
			// returning the challenge, so we are ready to send the FetchProcessCard-request
			// Create the FetchProcessCard-message object
			FetchProcessCard fetchProcCard = new FetchProcessCard(clientDialog);
			fetchProcCard.SelectionMode=OSCIMessage.SELECT_BY_MESSAGE_ID;
			fetchProcCard.SelectionRule=messageId;

			// Transmit the request and receive the response
			ResponseToFetchProcessCard rsp2FetchProcCard = fetchProcCard.send();

			if (!rsp2FetchProcCard.Feedback[0][1].StartsWith("0")) {
				// Error handling
			}

			// Optional: Check process card bundle (result of certificate check etc.)
			ProcessCardBundle [] pcb=rsp2FetchProcCard.ProcessCardBundles;

			// ....
			// Cleanup - Create the ExitDialog-Request
			ExitDialog ed = new ExitDialog(clientDialog);

			// Transmit the request and receive the response
			ResponseToExitDialog rsp2ExitDialog = ed.send();

			return rsp2FetchProcCard;
		}

		public static void main(String[] args) {
			try {
				OneWayMessage_ActiveRecipient scenario_1 = new OneWayMessage_ActiveRecipient(args[0]);

				ResponseToStoreDelivery responseStore = scenario_1.sendStoreDelivery();
				System.Diagnostics.Debug.WriteLine("\nResponseToStoreDelivery:\n" + responseStore.ToString());

				ResponseToFetchDelivery responseFetchDel = scenario_1.sendFetchDelivery(responseStore.MessageId);
		
				Reader reader = new Reader(new PKCS12Decrypter(@"..\..\Daten\Zertifikate\Leser.p12","123456"));
				java.security.cert.X509Certificate cert;
				SupportClass.PrivateKeySupport pKeyInfo;
				RSAEncryption.LoadFromStore(out cert, @"Leser1.p12", "123456",true,out pKeyInfo); //Intermed.cer
		
				System.Diagnostics.Debug.WriteLine("\nResponseToFetchDelivery:\n" + responseFetchDel.ToString());

				System.Diagnostics.Debug.WriteLine("\nCONTENT DATA:\n" +
					responseFetchDel.ContentContainer[0].Contents[0].ContentData);

				System.Diagnostics.Debug.WriteLine("\nENCRYPTED CONTENT DATA:\n" +
					responseFetchDel.EncryptedData[0].decrypt(reader).Contents[0].ContentData);

				ResponseToFetchProcessCard responseFetchProcCard = scenario_1.sendFetchProcessCard(responseStore.MessageId);
				System.Diagnostics.Debug.WriteLine("\nResponseToFetchProcessCard:\n"+responseFetchProcCard.ToString());
			} catch (Exception ex) 
            {
                System.Diagnostics.Debug.WriteLine(ex.StackTrace);
			}
		}
	}
}