using System;
using java.security.cert;
using de.osci.osci12.roles;
using de.osci.osci12.messagetypes;
using de.osci.osci12.messageparts;
using de.osci.osci12.common;
using de.osci.osci12.dotcommon;
using de.osci.osci12.samples.impl.crypto;

using java.util;

namespace de.osci.osci12.samples
{
	/// <summary>
	/// Zusammenfassung fr OneWayMessage_PassiveRecipient.
	/// </summary>
public class OneWayMessage_PassiveRecipient
{
  Intermed intermed;

  public OneWayMessage_PassiveRecipient(String intermedURL)// throws Exception
  {
	  // First we need the Intermediarys Cipher-Certificate
	  X509Certificate intermedCipherCert;
	  SupportClass.PrivateKeySupport pKeyInfo;
	  RSAEncryption.LoadFromStore(out intermedCipherCert,@"..\..\Daten\Zertifikate\Intermed.cer","123456",false, out pKeyInfo);

	  // Create the Intermed-role object
	  intermed = new Intermed(null, intermedCipherCert, new java.net.URI(intermedURL));
  }

  public ResponseToForwardDelivery sendForwardDelivery(String urlRecipient)//  throws Exception
  {
    // Create the Originator-object. Since we have the private key(s), we use this
    // Constructor. We need a Decrypter-object for decryption of response
	  Originator user_1 = new Originator(new PKCS12Signer(@"..\..\Daten\Zertifikate\Sender1.p12","123456"),new PKCS12Decrypter(@"..\..\Daten\Zertifikate\Sender1.p12","123456"));

    // Create a DialogHandler
    // Default configuration is transport encryption and signature activated
	  DialogHandler clientDialog = new DialogHandler(user_1, intermed, new de.osci.osci12.samples.impl.HttpTransport());

    // Create the GetMessageId-Request
    GetMessageId getMsgID = new GetMessageId(clientDialog);

    // Transmit the request and receive the response
    ResponseToGetMessageId rsp2GetMsgID = getMsgID.send();

    // Error handling
    if (!rsp2GetMsgID.Feedback[0][1].StartsWith("0"))
    {
      // do error handling
    }

    // Now we have a message-Id, we are ready to send the MediateDelivery-request
    // First we create the Addressee-object representing the user 2.
    // The signature certificate is not needed since we do not
    // expect a signed response from the addressee in this secenario
    // (it would be included in a signed response anyway).
	  java.security.cert.X509Certificate addresseecert;
	  SupportClass.PrivateKeySupport pKeyInfo;
	  RSAEncryption.LoadFromStore(out addresseecert, @"../../Daten/Zertifikate/Empfaenger.cer","123456",false, out pKeyInfo); //Intermed.cer
	  Addressee user_2 = new Addressee(null,addresseecert);

    // Create the MediateDelivery-message object
    // The constructor's second parameter (Addressee-object) may be null.
    // In this case transport encryption must be disabled by calling
    // clientDialog.setEncryption(false);
    // This is usually not recommended. As a side effect, the recipient will not
    // be able to fetch the process card later from the intermediary.
    ForwardDelivery forwardDel = new ForwardDelivery(clientDialog, user_2, urlRecipient, rsp2GetMsgID.MessageId);
    forwardDel.Subject="Subject";
    // We trust the intermediary's time stamp
    forwardDel.QualityOfTimeStampCreation=false;
    forwardDel.QualityOfTimeStampReception=false;

    // Add Content-Data
    ContentContainer data = new ContentContainer();
    data.addContent(new Content("Any content data."));
    forwardDel.addContentContainer(data);

    // Transmit the request and receive the response
    ResponseToForwardDelivery rsp2FwdDel = forwardDel.send();

    if (!rsp2FwdDel.Feedback[0][1].StartsWith("0"))
    {
      // Error handling
    }

    // Optional: Check process card bundle (result of certificate check etc.)
    //    rsp2FwdDel.getProcessCardBundleRequest();
    //    rsp2FwdDel.getProcessCardBundleReply();
    // ....
    return rsp2FwdDel;
  }

}
}
