namespace de.osci.osci12.samples.impl {

	using System;
	using org.apache.commons.logging;
	using System.Net;
	using System.IO;
	using de.osci.osci12.extinterfaces;
	using de.osci.osci12.dotcommon;
	using System.Xml;

	/// <summary>Beispiel-Implementierung eines Transport-Interfaces.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.extinterfaces.TransportI">
	/// </seealso>
	public class HttpTransport : TransportI {
		private static Log log = LogFactory.getLog(typeof(HttpTransport)); 

		InputStream inStream = null;
		HttpWebRequest hrequest;
		private static int j=0, k=0;
		public HttpTransport() {
		}

		/// <summary> Liefert den Namen des Herstellers.
		/// </summary>
		/// <value> Herstellername
		/// </value>
		public String Vendor {
			get {
				return "BOS";
			}
		}

		/// <summary> Liefert die Versionsnummer.
		/// </summary>
		/// <value> Versionsnummer
		/// </value>
		public String Version {
			get {
				return "0.9";
			}
		}

		public System.IO.Stream ResponseStream 
		{
			get {
				Stream s;
				try{
					WebResponse wr = hrequest.GetResponse();
					s = new WebResponseClosingStream(wr);
				}
				catch(Exception x){
					throw x;
				}
			 return s;				
			}
		}
		public TransportI newInstance() 
		{
			return new HttpTransport();
		}

		public bool isOnline(Uri uri) 
		{
			return true;
		}

		public long ContentLength 
		{
			get 
			{
				/**@todo Diese de.osci.osci12.extinterfaces.transport.TransportI-Methode implementieren*/
				throw new ApplicationException(
					"Methode getContentLength() noch nicht implementiert.");
			}
		}

		public System.IO.Stream getConnection(Uri uri, long laenge) 
		{
			hrequest=(HttpWebRequest)WebRequest.Create(uri); //url als String

			/*
				 * default proxy Einstellungen vom Browser
				 * WebProxy myProxy=new WebProxy();
				 * myProxy=(WebProxy)hrequest.Proxy;
				*/
			hrequest.ProtocolVersion = HttpVersion.Version10;
			hrequest.Method = "POST";
			hrequest.ContentLength = laenge;
			Stream rqst=hrequest.GetRequestStream();
			return rqst;
		}

		public static void main0() {
			HttpTransport http = new HttpTransport();
			http.outp();
		}

		public void outp() {
			System.Uri u=new System.Uri("http://localhost/osci-recipient");
			XmlTextWriter tw=new XmlTextWriter(this.getConnection(u,0),System.Text.Encoding.GetEncoding("UTF-8"));
			XmlDocument xmlDoc = new XmlDocument();
			xmlDoc.LoadXml("<test>weiuewhw</test>");
			xmlDoc.WriteTo(tw);
			tw.Flush();
			tw.Close();
			XmlDocument resultXmlDoc = new XmlDocument();
			resultXmlDoc.Load(ResponseStream);
			string s = resultXmlDoc.InnerXml;
		}
	}

	/// <summary>
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	/// 
// Notwendig, damit die Schnittstelle zum Schlieen des WebRequests nicht gendert werden musste.
	public class WebResponseClosingStream:de.osci.osci12.dotcommon.InputStream 
	{
		Stream respStream;
		WebResponse wr;
		public WebResponseClosingStream(WebResponse _wr)
		{
			wr = _wr;
			respStream = wr.GetResponseStream();
		}

		public override int ReadByte() 
		{
			return respStream.ReadByte();
		}

		public override int Read(byte[] b, int off, int len) 
		{
			return respStream.Read(b, off, len);
		}

		public override void Close() 
		{
			wr.Close();
		}

		public override void Flush() 
		{
			respStream.Flush();
		}
	}
	
}