namespace de.osci.osci12.samples.impl.crypto {
	
	using System;
	using de.osci.osci12.extinterfaces.crypto;
	using de.osci.osci12.dotcommon;
	using System.Security.Cryptography;
	using java.security.cert;
/*
    using Org.BouncyCastle.Crypto;
    using Org.BouncyCastle.Crypto.Encodings;
    using Org.BouncyCastle.Crypto.Parameters;
    using Org.BouncyCastle.Math;
 */ 
    using org.apache.commons.logging;

	/// <summary><p>Diese Klasse ist eine Beispiel-Implementierung der abstrakten Decrypter-Klasse.
	/// Fr die Verwendung wird ein PKCS#12-Keystore in Form einer *.p12-Datei bentigt.
	/// Die Implementierung ist fr Testzwecke bestimmt, sie greift auf den ersten
	/// verfgbaren Alias zu. Die PIN fr dessen Privatschlssel muss die gleiche sein
	/// wie die des Keystores.</p>
	/// <p>Diese einfache Implementierung hlt die PIN des Keystores als Character-Array
	/// im Arbeitsspeicher, sie wird als String bergeben. Es wird Anwendern empfohlen,
	/// eigene Implementierungen zu schreiben, die die PIN in der Methode decrypt(...)
	/// abfragen und nach Gebrauch wieder lschen oder anderweitig fr ein sicheres
	/// Pin-Cashing zu sorgen.</p>
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.extinterfaces.crypto.Decrypter">
	/// </seealso>
	public class PKCS12Decrypter : de.osci.osci12.extinterfaces.crypto.Decrypter {

		// private static Log log = LogFactory.getLog(PKCS12Decrypter.class);
        private static Log log;
        private X509Certificate cert;
		private String alias;
		private String pin_ = null;
		private KeyStore keyStore = null;
		private SupportClass.PrivateKeySupport pKeyInfo;
        private String file_;

        static PKCS12Decrypter()
        {
            log = LogFactory.getLog(typeof(PKCS12Decrypter));
		}

		/// <summary> Liefert den Namen des Herstellers.
		/// </summary>
		/// <value> Herstellername
		/// </value>
		public override String Vendor { 
			get {
				return "BOS";
			}
		}

		/// <summary> Liefert die Versionsnummer.
		/// </summary>
		/// <value> Versionsnummer
		/// </value>
		public override String Version { 
			get { 
				return "1.5";
			}
		}

		/// <summary> Liefert das Verschlsselungszertifikat.
		/// </summary>
		/// <value> Verschlsselungszertifikat
		/// </value>
		public override java.security.cert.X509Certificate Certificate 
        {
			get 
            {
				return cert;
			}
		}

		public PKCS12Decrypter(String p12_fileName, String pin) 
		{
            SupportClass.PrivateKeySupport pKeyInfo_dummy;
            file_ = p12_fileName;
            pin_ = pin;
            RSAEncryption.LoadFromStore(out cert, file_, pin_, false, out pKeyInfo_dummy);  
		}

		/// <summary> Entschlsselt das bergebene Byte-Array.
		/// </summary>
		/// <param name="data"> die zu entschlsselnden Daten
		/// </param>
		/// <returns> die verschlsselten Daten
		/// </returns>
		public override byte[] decrypt(byte[] data) 
		{
            RSAEncryption.LoadFromStore(out cert, file_, pin_, true, out pKeyInfo);
			return de.osci.osci12.encryption.Crypto.doRSADecryption(pKeyInfo, data);
        }
    }
}