namespace de.osci.osci12.signature {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	using de.osci.osci12.common;

	/// <exclude/>
	/// <summary>
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p>
	/// <p>Version: 1.5</p>
	/// </summary>
	public class KeyInfo 
	{

		private void  InitBlock() { 
			encryptedKey = new System.Collections.ArrayList();
		}

		virtual public System.String KeyName {
			get {
				return keyName;
			}
			set {
				this.keyName = value;
			}	
		}

		virtual public RetrievalMethod RetrievalMethod {
			get {
				return retrievalMethod;
			}
			set {
				if (typeOfKey > - 1) {
					throw new System.Exception("KeyInfo wurde schon anders Instanziert (X509Data, oder EncryptedKey)");
				}
				typeOfKey = 1;
				this.retrievalMethod = value;
			}
		}

		virtual public System.String KeyValue {
			get {
				return keyValue;
			}
			set {
				this.keyValue = value;
			}
		}

		virtual public de.osci.osci12.encryption.EncryptedKey[] EncryptedKeys {
			get {
				return (de.osci.osci12.encryption.EncryptedKey[]) encryptedKey.ToArray(typeof( de.osci.osci12.encryption.EncryptedKey));
			}
		}

		virtual public System.String AgreementMethod {
			get {
				return agreementMethod;
			}
			set {
				this.agreementMethod = value;
			}
		}

		virtual public System.String Id {
			get {
				return id;
			}
			set {
				this.id = value;
			}
		}

		virtual public int TypeOfKey {
			get {
				return typeOfKey;
			}	
		}

		virtual public X509Data X509Data {
			get {
				return x509Data;
			}	
		}

		virtual public System.String MgmtData {
			get {
				return mgmtData;
			}
			set {
				this.mgmtData = value;
			}			
		}

		private static Log log;
		
		/// <summary>Bezeichner dieses KeyInfo Elements. Kann statt ID als Referenz verwendet werden. 
		/// </summary>
   
		private System.String keyName;
		private static int KEY_IS_ENCRYPTEDKEY = 0;
		private static int KEY_IS_RETRIEVALMETHOD = 1;
		private static int KEY_IS_X509DATA = 2;
		private int typeOfKey = - 1;
		private RetrievalMethod retrievalMethod;
		internal X509Data x509Data = null;
		private System.String mgmtData = null;
		private System.String keyValue;
		
		/// <summary>Liste enthaltener Encryptedkey Elemente. 
		/// </summary>
		private System.Collections.ArrayList encryptedKey;
		private System.String agreementMethod;
		private System.String id;

		/// <summary>Zertifikat, welches direkt in KeyInfo eingestellt wird. 
		/// </summary>
		private java.security.cert.X509Certificate cert;

		/// <summary>DefaultHandler auf den zurckgesprungen wird. (SAX) 
		/// </summary>
		private DefaultHandler defaultHandler = null;
		
		/// <summary>SAX Parser. 
		/// </summary>
		private XMLReader xmlReader = null;

		/// <summary>Element, welches gerade geparsed wird. 
		/// </summary>
		private System.String currentElement = null;

		/// <summary>Stringbuffer fr Characters. 
		/// </summary>
		private System.Text.StringBuilder tempStrBuffer;
		
		/// <summary> Creates a new KeyInfo object.
		/// </summary>
		/// <param name="id">
		/// </param>
		/// <param name="uRI">
		/// </param>
		public KeyInfo(System.String uRI, System.String id) {
			InitBlock();
			this.id = id;
			this.typeOfKey = 1;
			log.debug("Konstruktortype: RetrievelMethod");
			RetrievalMethod retrievalMethod = new RetrievalMethod();
			retrievalMethod.URI = uRI;
			this.retrievalMethod = retrievalMethod;
		}
		
		public KeyInfo(System.String uRI) {
			InitBlock();
			this.typeOfKey = 1;
			log.debug("Konstruktortype: RetrievelMethod");
			RetrievalMethod retrievalMethod = new RetrievalMethod();
			retrievalMethod.URI = uRI;
			this.retrievalMethod = retrievalMethod;
		}
		
		/// <summary> Creates a new KeyInfo object.
		/// </summary>
		public KeyInfo() {
			InitBlock();
			log.debug("KeyInfo Konstruktortype noch nicht definiert.");
		}
		
		/// <summary>Konstruktor. Transportverschluesselung.
		/// </summary>
		public KeyInfo(java.security.cert.X509Certificate cert) {
			InitBlock();
			typeOfKey = 2;
			log.debug("Konstruktortype: X509Certificate Data");
			x509Data = new X509Data();
			x509Data.setX509Certificate(cert);
			this.cert = cert;
		}		
		
		public virtual void  addEncryptedKey(de.osci.osci12.encryption.EncryptedKey encryptedKey) {
			this.encryptedKey.Add(encryptedKey);
		}

		public virtual void  writeXML(System.IO.Stream out_Renamed, String ds, String xenc) {
			MoreSupport.ToStream(out_Renamed,"<" + ds + ":KeyInfo",Constants.CHAR_ENCODING);
			//MoreSupport.ToStream(out_Renamed,"<" + ds + ":KeyInfo xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\"",Constants.CHAR_ENCODING);
			if (this.id != null) {
				MoreSupport.ToStream(out_Renamed," Id=\"" + this.id + "\">",Constants.CHAR_ENCODING);
			} else {
				//MoreSupport.ToStream(out_Renamed,">",Constants.CHAR_ENCODING);
				byte[] b=new byte[1];  b[0]=0x3e;
				out_Renamed.Write(b,0,1);
			}
			if (this.encryptedKey.Count > 0) {
				for (int i = 0; i < this.encryptedKey.Count; i++) {
					//((de.osci.osci12.encryption.EncryptedKey) this.encryptedKey[i]).writeXML(out_Renamed);
					((de.osci.osci12.encryption.EncryptedKey) this.encryptedKey[i]).writeXML(out_Renamed, ds, xenc);
				}
			}
			if (this.retrievalMethod != null) {
				MoreSupport.ToStream(out_Renamed,"<" + ds + ":RetrievalMethod Type=\"" + this.RetrievalMethod.Type + "\" URI=\"" + this.RetrievalMethod.URI + "\"></" + ds + ":RetrievalMethod>",Constants.CHAR_ENCODING);
			}
			if (x509Data != null) {
				x509Data.writeXML(out_Renamed, ds);
			} else if (mgmtData != null) {
				MoreSupport.ToStream(out_Renamed,"<" + ds + ":MgmtData>" + mgmtData + "</ds:MgmtData>","UTF-8");
			}
			MoreSupport.ToStream(out_Renamed,"</ds:KeyInfo>","UTF-8");
		}
		
	/*	public override System.String ToString() {
			System.Text.StringBuilder buffer = new System.Text.StringBuilder();
			buffer.Append("<KeyInfo xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\" Id=\"" + this.id + "\">");
			if (this.encryptedKey.Count > 0) {
				for (int i = 0; i < this.encryptedKey.Count; i++) {
					buffer.Append(((de.osci.osci12.encryption.EncryptedKey) this.encryptedKey[i]).ToString());
				}
			}
			if (this.retrievalMethod != null) {
				buffer.Append("<ds:RetrievalMethod URI=\"" + this.RetrievalMethod.URI + "\" Type=\"" + this.RetrievalMethod.Type + "\" />");
			}
			if (this.cert != null) {
				buffer.Append("<" + ds + ":X509Data><" + ds + ":X509Certificate>");
				System.IO.MemoryStream bytes = new System.IO.MemoryStream();
				de.osci.osci12.common.Base64OutputStream base64Out = new de.osci.osci12.common.Base64OutputStream(bytes, true);
				base64Out.Write(this.cert.getEncoded(), 0, this.cert.getEncoded().Length);
				base64Out.Flush();
				base64Out.Close();
				buffer.Append(MoreSupport.ToByteArray(bytes.ToString(), Constants.CHAR_ENCODING).ToString());
				buffer.Append("</" + ds + ":X509Certificate></" + ds + ":X509Data>");
			}
			buffer.Append("</KeyInfo>");
				
			return buffer.ToString();
		}*/
		
		static KeyInfo() {
			log = LogFactory.getLog(typeof(KeyInfo));
		}
	}
}