namespace de.osci.osci12.signature {

	using System;
	using java.security.cert;
	using org.apache.commons.logging;
	using de.osci.osci12.common;

	/// <exclude/>
	/// <summary>
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class X509Data 
	{

		private X509Certificate x509Cert = null;

		virtual public X509Certificate X509Certificate {
			get {
				return x509Cert;
			}
			set {
				if (dataState > - 1) {
					throw new System.Exception("X509Data wurde bereits anders Instanziert");
				}
				dataState = STATE_CERTIFICATE;
				this.x509Cert = value;
			}
		}

		public void setX509Certificate(byte[] x509Certificate) {
			if (dataState > - 1) {
				throw new System.Exception("X509Data wurde bereits anders Instanziert");
			} else {
				dataState = STATE_CERTIFICATE;
			}
			this.x509Cert = de.osci.helper.Tools.createCertificate(x509Certificate);
		}

		public void setX509Certificate(java.security.cert.X509Certificate x509Certificate) {
			if(dataState>-1) {
				throw new Exception("X509Data wurde bereits anders Instanziert");
			}
			dataState=STATE_CERTIFICATE;
			this.x509Cert = x509Certificate; 
		}

		virtual public System.String X509CRL {
			get {
				return x509CRL;
			}
			set {
				this.x509CRL = value;
			}			
		}

		virtual public System.String X509SKI {
			get {
				return x509SKI;
			}			
			set {
				if (dataState > - 1) {
					throw new System.Exception("X509Data wurde bereits anders Instanziert");
				} else {
					dataState = STATE_SKI;
				}
				this.x509SKI = value;
			}	
		}

		virtual public System.String X509SubjectName {
			get {
				return x509SubjectName;
			}			
			set {
				if (dataState > - 1) {
					throw new System.Exception("X509Data wurde bereits anders Instanziert");
				} else {
					dataState = STATE_SUBJECT_NAME;
				}
				this.x509SubjectName = value;
			}	
		}

		virtual public System.String IssuerName {
			set {
				if (dataState > - 1 && dataState != STATE_ISSUER_SERIAL) {
					throw new System.Exception("X509Data wurde bereits anders Instanziert");
				} else {
					dataState = STATE_ISSUER_SERIAL;
				}
				if (issuerSerial == null) {
					issuerSerial = new IssuerSerial(this);
				}
				issuerSerial.x509IssuerName = value;
			}	
		}

		virtual public System.String SerialNumber {
			set {
				if (dataState > - 1 && dataState != STATE_ISSUER_SERIAL) {
					throw new System.Exception("X509Data wurde bereits anders Instanziert");
				} else {
					dataState = STATE_ISSUER_SERIAL;
				}
				if (issuerSerial == null) {
					issuerSerial = new IssuerSerial(this);
				}
				issuerSerial.x509SerialNumber = value;
			}	
		}

		private static Log log;
		private System.String x509SKI = null;
		private System.String x509SubjectName = null;
		private System.String x509CRL = null;
		private IssuerSerial issuerSerial = null;
		private int dataState = - 1;
		private const int STATE_ISSUER_SERIAL = 0;
		private const int STATE_SKI = 1;
		private const int STATE_SUBJECT_NAME = 2;
		private const int STATE_CERTIFICATE = 3;
		private const int STATE_CRL = 4;
		
		public X509Data() {
		}
		
		public class IssuerSerial {
			private void  InitBlock(X509Data enclosingInstance) {
				this.enclosingInstance = enclosingInstance;
			}

			private X509Data enclosingInstance;
			virtual public System.String X509IssuerName {
				get {
					return x509IssuerName;
				}	
			}

			virtual public System.String X509SerialNumber {
				get {
					return x509SerialNumber;
				}	
			}

			public X509Data Enclosing_Instance {
				get {
					return enclosingInstance;
				}	
			}

			internal System.String x509IssuerName = null;
			internal System.String x509SerialNumber = null;
			internal IssuerSerial(X509Data enclosingInstance) {
				InitBlock(enclosingInstance);
			}	
		}
		
		public virtual void  writeXML(System.IO.Stream out_Renamed, String ds) {
			MoreSupport.ToStream(out_Renamed,"<" + ds + ":X509Data>",Constants.CHAR_ENCODING);
			if (dataState == STATE_CERTIFICATE) {
				MoreSupport.ToStream(out_Renamed,"<" + ds + ":X509Certificate>",Constants.CHAR_ENCODING);
				de.osci.osci12.common.Base64OutputStream myOut = new de.osci.osci12.common.Base64OutputStream(out_Renamed, true);
				myOut.Write(x509Cert.getEncoded(), 0, x509Cert.getEncoded().Length);
				myOut.Flush();
				MoreSupport.ToStream(out_Renamed,"</" + ds + ":X509Certificate>",Constants.CHAR_ENCODING);
			} else if (dataState == STATE_CRL) {
				MoreSupport.ToStream(out_Renamed,"<" + ds + ":X509CRL>" + x509CRL + "</" + ds + ":X509CRL>",Constants.CHAR_ENCODING);
			} else if (dataState == STATE_ISSUER_SERIAL) {
				MoreSupport.ToStream(out_Renamed,"<" + ds + ":X509IssuerSerial><" + ds + ":X509IssuerName>",Constants.CHAR_ENCODING);
				MoreSupport.ToStream(out_Renamed,issuerSerial.X509IssuerName + "</" + ds + ":X509IssuerName><" + ds + ":X509SerisalNumber>" + issuerSerial.X509SerialNumber,Constants.CHAR_ENCODING);
				MoreSupport.ToStream(out_Renamed,"</" + ds + ":X509SerisalNumber></" + ds + ":X509IssuerSerial>",Constants.CHAR_ENCODING);
			} else if (dataState == STATE_SKI) {
				MoreSupport.ToStream(out_Renamed,"<" + ds + ":X509SKI>" + x509SKI + "</" + ds + ":X509SKI>",Constants.CHAR_ENCODING);
			} else if (dataState == STATE_SUBJECT_NAME) {
				MoreSupport.ToStream(out_Renamed,"<" + ds + ":X509SubjectName>" + x509SubjectName + "</" + ds + ":X509SubjectName>",Constants.CHAR_ENCODING);
			}
			MoreSupport.ToStream(out_Renamed,"</" + ds + ":X509Data>",Constants.CHAR_ENCODING);
		}
		
		static X509Data() {
			log = LogFactory.getLog(typeof(X509Data));
		}
	}
}