namespace de.osci.osci12.signature {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;

	/// <exclude/>
	/// <summary> Builder, der ein Element xdsig:X509Data bearbeitet.
	/// 
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: PPI Financial Systems GmbH</p> 
	/// </summary>
	class X509DataBuilder:DefaultHandler 
	{
		protected const String DS_XMLNS = "http://www.w3.org/2000/09/xmldsig#"; 
		protected const String XENC_XMLNS = "http://www.w3.org/2001/04/xmlenc#"; 

		private void  InitBlock() 
		{
			x509Data = new X509Data();
			buffer = new System.Text.StringBuilder();
		}

		/// <summary> Ruft den Inhalt des inneren Elements xdsig:X509Certificate ab.
		/// </summary>
		/// <value> StringBuffer.
		/// </value>
		virtual public System.Text.StringBuilder X509DataBuffer {
			get {
				return buffer;
			}
			
		}

		internal DefaultHandler parentHandler = null;
		private System.String currentElement = null;
		private X509Data x509Data;
		internal XMLReader xmlReader = null;
		protected internal static Log log;
		private System.Text.StringBuilder buffer;
		
		/// <summary> Konstruktor.
		/// </summary>
		/// <param name="parentHandler">DefaultCursorHandler, der diesen Builder erzeugt hat.
		/// </param>
		/// <param name="xmlReader">Aktueller CursorXMLReader.
		/// </param>
		/// <param name="attributes">Attribute des Elements xdsig:X509Data.
		/// </param>
		public X509DataBuilder(XMLReader xmlReader, DefaultHandler parentHandler, Attributes attributes) {
			InitBlock();
			this.xmlReader = xmlReader;
			this.parentHandler = parentHandler;
			if (attributes == null) {
				throw new System.ArgumentException("Dem Konstruktor der Klasse X509DataBuilder wird null bergeben !");
			}
		}
		
		/// <summary> 
		/// <element name="X509IssuerSerial" type="ds:X509IssuerSerialType"/>
		/// <element name="X509SKI" type="base64Binary"/>
		/// <element name="X509SubjectName" type="string"/>
		/// <element name="X509Certificate" type="base64Binary"/>
		/// <element name="X509CRL" type="base64Binary"/>
		/// <any namespace="##other" processContents="lax"/>
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="attributes">
		/// </param>
		/// <exception cref="SAXException">
		/// </exception>
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			log.debug("Start-Element: " + localName + "localName: " + localName);
			if (localName.Equals("X509Certificate")&& uri.Equals(DS_XMLNS)) {
				currentElement = "";
			}
			else if (localName.Equals("X509IssuerSerial")&& uri.Equals(DS_XMLNS)) {
				throw new SAXException("X509IssuerSerial Element wird nicht unterstzt");
			}
			else if (localName.Equals("X509SKI")&& uri.Equals(DS_XMLNS)) {
				currentElement = "";
			}
			else if (localName.Equals("X509SubjectName")&& uri.Equals(DS_XMLNS)) {
				currentElement = "";
			}
			else if (localName.Equals("X509CRL")&& uri.Equals(DS_XMLNS)) {
				currentElement = "";
			}
			else {
				throw new SAXParseException("Nicht vorgesehenes Element: " + localName, null);
			}
		}
		
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("End-Element: " + localName);
			if (localName.Equals("X509Data")&& uri.Equals(DS_XMLNS)) {
				if (parentHandler is KeyInfoBuilder) {
					((KeyInfoBuilder) parentHandler).KeyInfo.x509Data = x509Data;
					log.debug("Parent ist KeyInfo");
				}
				xmlReader.setContentHandler(parentHandler);
			}
			else if (localName.Equals("X509Certificate")&& uri.Equals(DS_XMLNS)) {
				System.IO.MemoryStream bytesStream = new System.IO.MemoryStream(MoreSupport.ToByteArray(currentElement,de.osci.osci12.common.Constants.CHAR_ENCODING));
				de.osci.osci12.common.Base64InputStream base64In = new de.osci.osci12.common.Base64InputStream(bytesStream);
				x509Data.setX509Certificate(de.osci.helper.Tools.createCertificate(base64In));
			}
			else if (localName.Equals("X509IssuerSerial")&& uri.Equals(DS_XMLNS)) {
				throw new SAXException("X509IssuerSerial Element wird nicht unterstzt");
			}
			else if (localName.Equals("X509SKI")&& uri.Equals(DS_XMLNS)) {
				x509Data.X509SKI = currentElement;
			}
			else if (localName.Equals("X509SubjectName")&& uri.Equals(DS_XMLNS)) {
				x509Data.X509SubjectName = currentElement;
			}
			else if (localName.Equals("X509CRL")&& uri.Equals(DS_XMLNS)) {
				x509Data.X509CRL = currentElement;
			}
			else {
				throw new SAXParseException("Nicht vorgesehenes Element: " + localName, null);
			}
			currentElement = null;
		}
		
		
		public override void  characters(char[] ch, int start, int length) {
			log.debug("Character: " + new System.String(ch, start, length));
			if (currentElement == null) {
				for (int i = 0; i < length; i++) {
  if (ch[start + i] > ' '){
	  //if ((ch[start + i] == '\r') || (ch[start + i] == '\n') || (ch[start + i] == ' '))
		//				i++;
		//			else {
						throw new SAXException(de.osci.osci12.common.DialogHandler.text.getString("unexpected_char"));
					}
				}
			}
			else currentElement += new System.String(ch, start, length);
		}

		static X509DataBuilder() {
			log = LogFactory.getLog(typeof(X509DataBuilder));
		}
	}
}