namespace de.osci.osci12.soapheader {
	
	using System;
	using java.security.cert;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	using de.osci.osci12.roles;

	/// <exclude/>
	/// <summary>
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public abstract class CertificateH:HeaderEntry 
	{

		public CertificateH() {
			InitBlock();
		}
		
		private void  InitBlock() {
			certificates = new System.Collections.Hashtable();
		}

		virtual public System.Collections.Hashtable Certificates {
			get {
				return certificates;
			}	
		}

		new private static Log log;
	
		protected internal System.Collections.Hashtable certificates;
		
		protected internal virtual void  addCipherCertificate(Role role, System.IO.Stream out_Renamed) {
			if (role == null) {
				throw new OSCIRoleException("Role-Objekt darf nicht null sein.", "");
			}
			System.String name = role.GetType().FullName.Substring(role.GetType().FullName.LastIndexOf((System.Char) '.') + 1);
			if (role.CipherCertificate == null) {
				throw new OSCIRoleException("Kein Verschlsselungszertifikat fr " + name + " eingestellt.", null);
			}
			if (role is Intermed) name = "Intermediary";
			else if (role is Originator)
				name = "Originator";
			else if (role is Addressee)
				name = "Addressee";
			else if (role is de.osci.osci12.roles.Reader)
				name = "OtherReader";
			else if (role is Author)
				name = "OtherAuthor";
			else
				throw new System.ArgumentException(DialogHandler.text.getString("invalid_firstargument") + role);
			//throw new IllegalArgumentException(DialogHandler.text.getString("invalid_firstargument") + role);
/*			if (name.Equals("Reader") || name.Equals("Author")) {
				name = "Other" + name;
			}
			else if (name.Equals("Intermed")) {
				name = "Intermediary";
			}
*/			MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix +":CipherCertificate" + name + " Id=\"" + role.CipherCertificateId + "\"><" + dsNSPrefix + ":X509Data><" + dsNSPrefix + ":X509Certificate>",Constants.CHAR_ENCODING);
			de.osci.osci12.common.Base64OutputStream base64Out = new de.osci.osci12.common.Base64OutputStream(out_Renamed, true);
			base64Out.Write(role.CipherCertificate.getEncoded(), 0, role.CipherCertificate.getEncoded().Length);
			base64Out.Flush();
			MoreSupport.ToStream(out_Renamed,"</" + dsNSPrefix + ":X509Certificate></" + dsNSPrefix + ":X509Data></" + osciNSPrefix +":CipherCertificate" + name + ">",Constants.CHAR_ENCODING);
		//	out_Renamed.Flush();
		}

		protected internal virtual void  addSignatureCertificate(Role role, System.IO.Stream out_Renamed) {
			System.String name = role.GetType().FullName.Substring(role.GetType().FullName.LastIndexOf((System.Char) '.') + 1);
			if ((role == null) || (role.SignatureCertificate == null)) {
				throw new OSCIRoleException("Kein Signaturzertifikat fr " + name + "eingestellt.", null);
			}
/*			if (name.Equals("Reader") || name.Equals("Author")) {
				name = "Other" + name;
			} else if (name.Equals("Intermed")) {
				name = "Intermediary";
			}
*/
			    if (role is Intermed)
					name = "Intermediary";
				else if (role is Originator)
					name = "Originator";
				else if (role is Addressee)
					name = "Addressee";
				else if (role is de.osci.osci12.roles.Reader)
					name = "OtherReader";
				else if (role is Author)
					name = "OtherAuthor";
				else
					 throw new System.ArgumentException(DialogHandler.text.getString("invalid_firstargument") + role);
			//throw new IllegalArgumentException(DialogHandler.text.getString("invalid_firstargument") + role);

			MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix+":SignatureCertificate" + name + " Id=\"" + role.SignatureCertificateId + "\"><" + dsNSPrefix + ":X509Data><" + dsNSPrefix + ":X509Certificate>",Constants.CHAR_ENCODING);
			try {
				de.osci.osci12.common.Base64OutputStream base64Out = new de.osci.osci12.common.Base64OutputStream(out_Renamed, true);
				base64Out.Write(role.SignatureCertificate.getEncoded(), 0, role.SignatureCertificate.getEncoded().Length);
				base64Out.Flush();
			} catch (CertificateEncodingException ex) {
				throw new OSCIRoleException("Problem beim Codieren des Zertifikats: " + ex.ToString() + ". " + ex.Message, null);
			}
			
			MoreSupport.ToStream(out_Renamed,"</" + dsNSPrefix + ":X509Certificate></" + dsNSPrefix + ":X509Data></" + osciNSPrefix +":SignatureCertificate" + name + ">",Constants.CHAR_ENCODING);
		}

		static CertificateH() {
			log = LogFactory.getLog(typeof(CertificateH));
		}
	}
}