namespace de.osci.osci12.soapheader {

	using System;
	using System.Collections;
	using java.security.cert;
	using org.apache.commons.logging;
	using org.xml.sax;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.messagetypes;
	using de.osci.osci12.roles;

	/// <exclude/>
	/// <summary>
	/// <p>Copyright  2011 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von bremen online services Entwicklungs- und
	/// Betriebsgesellschaft GmbH &amp; Co. KG</p>
	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
	/// Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
	/// 28015 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.5</p>
	/// </summary>
	public class NonIntermediaryCertificatesHBuilder:de.osci.osci12.messageparts.MessagePartParser 
	{
		private void  InitBlock() {
			reader = new System.Collections.ArrayList();
			authors = new System.Collections.ArrayList();
		}
		new private static Log log;
		private const int CIPHER_CERTIFICATE_ORIGINATOR = 0;
		private const int CIPHER_CERTIFICATE_OTHER_AUTHOR = 1;
		private const int CIPHER_CERTIFICATE_ADDRESSEE = 2;
		private const int CIPHER_CERTIFICATE_OTHER_READER = 3;
		private const int SIGNATURE_CERTIFICATE_ORIGINATOR = 4;
		private const int SIGNATURE_CERTIFICATE_OTHER_AUTHOR = 5;
		private const int SIGNATURE_CERTIFICATE_ADDRESSEE = 6;

		// fr das Array relevant sind die oben aufgefhrten statischen Variablen wie z.B. CIPHER_CERTIFICATE_ORIGINATOR
		private int[] check;
		private NonIntermediaryCertificatesH nic = null;
		private int typ;
		private Originator originator;
		private Addressee addressee;
		private System.Collections.ArrayList reader;
		private System.Collections.ArrayList authors;
		private String tmpId, refId;
		private int msgTyp;
		private bool changeOrgsAndAdds = false;
		
		/// <summary> Constructor for the NonIntermediaryCertificatesHBuilder object
		/// </summary>
		/// <param name="xmlReader">
		/// </param>
		/// <param name="parentHandler">
		/// </param>
		/// <param name="atts">
		/// </param>
		/// <param name="check">
		/// </param>

		public NonIntermediaryCertificatesHBuilder(OSCIMessageBuilder parentHandler, Attributes atts, int[] check):base(parentHandler) {
			InitBlock();
			this.check = check;
			nic = new NonIntermediaryCertificatesH();
			if (atts.getValue("Id") != null) {
				nic.RefID=atts.getValue("Id");
			}

			msgTyp = msg.MessageType;

			if ((msgTyp == OSCIMessage.RESPONSE_TO_FETCH_DELIVERY) ||
				(msgTyp == OSCIMessage.RESPONSE_TO_PROCESS_DELIVERY) ||
				(msgTyp == OSCIMessage.RESPONSE_TO_MEDIATE_DELIVERY))
				changeOrgsAndAdds = true;
			nic.setNSPrefixes(msg); 

		}
		
		/// <summary> 
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <param name="attributes">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			log.debug("Start-Element: " + qName);
			if (localName.Equals("CipherCertificateOriginator")&& uri.Equals(OSCI_XMLNS)) {
				typ = CIPHER_CERTIFICATE_ORIGINATOR;
			} else if (localName.Equals("CipherCertificateOtherAuthor")&& uri.Equals(OSCI_XMLNS)) {
				typ = CIPHER_CERTIFICATE_OTHER_AUTHOR;
			} else if (localName.Equals("CipherCertificateAddressee")&& uri.Equals(OSCI_XMLNS)) {
				typ = CIPHER_CERTIFICATE_ADDRESSEE;
			} else if (localName.Equals("CipherCertificateOtherReader")&& uri.Equals(OSCI_XMLNS)) {
				typ = CIPHER_CERTIFICATE_OTHER_READER;
			} else if (localName.Equals("SignatureCertificateOriginator")&& uri.Equals(OSCI_XMLNS)) {
				typ = SIGNATURE_CERTIFICATE_ORIGINATOR;
			} else if (localName.Equals("SignatureCertificateOtherAuthor")&& uri.Equals(OSCI_XMLNS)) {
				typ = SIGNATURE_CERTIFICATE_OTHER_AUTHOR;
			} else if (localName.Equals("SignatureCertificateAddressee")&& uri.Equals(OSCI_XMLNS)) {
				typ = SIGNATURE_CERTIFICATE_ADDRESSEE;
			} else if (localName.Equals("X509Certificate")&& uri.Equals(DS_XMLNS)) {
				currentElement = new System.Text.StringBuilder();
			} else if (!localName.Equals("X509Data")&& uri.Equals(DS_XMLNS)) {
				throw new org.xml.sax.SAXException("Unerwartetes Element im NonIntermediaryCertificate-Header: " + qName);
			}
			if (attributes.getValue("Id") != null) {
				refId = attributes.getValue("Id");
				if (refId.IndexOf("_") >= 0)
					tmpId = refId.Substring(0, refId.IndexOf('_'));
			}			
			if (check[typ] == 0) {
				throw new org.xml.sax.SAXException("Unerlaubtes Zertifikat im NonIntermediaryCertificate-Header, Typ: " + typ);
			}
		}
		
		/// <summary>
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("End-Element: " + qName);
			if (localName.Equals("NonIntermediaryCertificates")&& uri.Equals(OSCI_XMLNS)) {
				for (int i = 0; i < check.Length; i++) {
					if (check[i] > 0) {
						throw new org.xml.sax.SAXException("Fehlendes Zertifikat im NonIntermediaryCertificate-Header, Typ: " + i);
					}
				}				
				nic.CipherCertificatesOtherReaders = (de.osci.osci12.roles.Reader[]) reader.ToArray(typeof(de.osci.osci12.roles.Reader));
					
				System.Collections.ArrayList vec = new System.Collections.ArrayList();
				for (int i = 0; i < authors.Count; i++) {
					if (((Role) authors[i]).hasCipherCertificate()){
						Object dumm=((Role) authors[i]).CipherCertificate;
						vec.Add(authors[i]);
					}
				}
				nic.CipherCertificatesOtherAuthors = (Author[]) vec.ToArray(typeof(Author));
					
				vec = new System.Collections.ArrayList();
				for (int i = 0; i < authors.Count; i++) {
					if (((Role) authors[i]).hasSignatureCertificate()) {
						Object dumm=((Role) authors[i]).SignatureCertificate;
						vec.Add(authors[i]);
					} 
				}
				nic.SignatureCertificatesOtherAuthors = (Author[]) vec.ToArray(typeof(Author));
				msg.addRole(originator);
				msg.addRole(addressee);
					
				if (msg is ResponseToAcceptDelivery) {
					if (addressee != null) {
						msg.dialogHandler.supplier = addressee;
					}
				} else if (msg is ResponseToProcessDelivery) {
					if (originator != null){
						if (originator.hasSignatureCertificate())
							msg.dialogHandler.Supplier.SignatureCertificate=originator.SignatureCertificate;
						if (originator.hasCipherCertificate())
							msg.dialogHandler.Supplier.CipherCertificate=originator.CipherCertificate;
					}
				} 
	         else if (!changeOrgsAndAdds &&
                   !(msg is AcceptDelivery) &&
                   !(msg is ProcessDelivery) &&
                   (originator != null))
					msg.dialogHandler.client = originator;

				for (int i = 0; i < authors.Count; i++) {
					msg.addRole((Author) authors[i]);
				}
				for (int i = 0; i < reader.Count; i++) {
					msg.addRole((de.osci.osci12.roles.Reader) reader[i]);
				}
				msg.nonIntermediaryCertificatesH = nic;
					
				this.xmlReader.setContentHandler(this.parentHandler);
			} else if (localName.Equals("X509Certificate")&& uri.Equals(DS_XMLNS)) {
				int i;
				X509Certificate cert;
				System.IO.MemoryStream byteStream = new System.IO.MemoryStream(MoreSupport.ToByteArray(currentElement.ToString(), de.osci.osci12.common.Constants.CHAR_ENCODING));
				de.osci.osci12.common.Base64InputStream base64In = new de.osci.osci12.common.Base64InputStream(byteStream);
				cert = de.osci.helper.Tools.createCertificate(base64In);
				if (typ == CIPHER_CERTIFICATE_ORIGINATOR) {

					if (!changeOrgsAndAdds && (msg.dialogHandler.Client is Originator))
						originator = (Originator)msg.dialogHandler.Client;
					if (originator != null) {
						if (!originator.hasCipherCertificate())
						{ originator.CipherCertificate=cert;
						}if (!originator.CipherCertificate.Equals(cert)){
								  throw new System.ArgumentException("Nicht passendes Zertifikat gefunden fr Originator.");}
						
					}
					else originator = new Originator((X509Certificate) null, cert);
					if (tmpId != null) 
						originator.id = tmpId; 
					originator.cipherRefId = refId; 

					nic.CipherCertificateOriginator = originator;
					check[CIPHER_CERTIFICATE_ORIGINATOR] = 0;
				} else if (typ == CIPHER_CERTIFICATE_OTHER_AUTHOR) {
					Author au = new Author((X509Certificate) null, cert);
					if (tmpId != null) 
						au.id = tmpId;
					au.cipherRefId = refId; 
					authors.Add(au);
					msg.addRole(au);
				} else if (typ == CIPHER_CERTIFICATE_ADDRESSEE) {

					if (!changeOrgsAndAdds && (msg.dialogHandler.Client is Addressee))
						addressee = (Addressee)msg.dialogHandler.Supplier;

					if (addressee != null) {
						if (addressee.hasCipherCertificate())
						{
							addressee.CipherCertificate=cert;
						}
						else if (!addressee.CipherCertificate.Equals(cert)){
								  throw new System.ArgumentException("Nicht apssendes Zertifikat gefunden fr Addressee.");}

						
					}
					else addressee = new Addressee((X509Certificate) null, cert);

					if (tmpId != null) 
						addressee.id = tmpId;
					addressee.cipherRefId = refId; 
					nic.CipherCertificateAddressee = addressee;
					check[CIPHER_CERTIFICATE_ADDRESSEE] = 0;
				} else if (typ == CIPHER_CERTIFICATE_OTHER_READER) {
					de.osci.osci12.roles.Reader rd = new de.osci.osci12.roles.Reader(cert);
					if (tmpId != null) 
						rd.id = tmpId;
					rd.cipherRefId = refId; 
					reader.Add(rd);
					msg.addRole(rd);
				} else if (typ == SIGNATURE_CERTIFICATE_ORIGINATOR) {
					if (originator == null) {
						if (!changeOrgsAndAdds && (msg.dialogHandler.Client  is Originator))
							originator = (Originator)msg.dialogHandler.Client;
						else originator = new Originator(cert, (X509Certificate) null);
					}
					else originator.SignatureCertificate = cert;
					if (tmpId != null) 
						originator.id = tmpId;
					originator.signatureRefId = refId; 
					nic.SignatureCertificateOriginator = originator;
					check[SIGNATURE_CERTIFICATE_ORIGINATOR] = 0;
				} else if (typ == SIGNATURE_CERTIFICATE_OTHER_AUTHOR) {
					for (i = 0; i < authors.Count; i++) {

						if (((Role) authors[i]).id.Equals(tmpId)) {
							try { 
								// Test, ob bereits ein Author-Objekt mit dieser Vorwahl mit einem Signaturzertifikat 
								// versehen wurde (sollte nur beim Import von Rollen aus anderen Nachrichten vorkommen) 
								X509Certificate x5=((Author) authors[i]).SignatureCertificate;

								// Role.id wird nur noch intern verwendet 
								if (tmpId != null) 
									tmpId += '0'; 
							} 
							catch (OSCIRoleException ex) { 
								((Author) authors[i]).SignatureCertificate = cert;
								((Author) authors[i]).signatureRefId = refId;
								break;
							}
						}
					}
					if (i == authors.Count) {
						Author au = new Author(cert, (X509Certificate) null);
						if (tmpId != null) 
							au.id = tmpId;
						au.signatureRefId = refId; 
						authors.Add(au);
						msg.addRole(au);
					}
				} else if (typ == SIGNATURE_CERTIFICATE_ADDRESSEE) {
					if (addressee == null) {
						if (!changeOrgsAndAdds && (msg.dialogHandler.Supplier is Addressee)) {
							addressee = (Addressee)msg.dialogHandler.Supplier;
							addressee.SignatureCertificate = cert;
						}
						addressee = new Addressee(cert, (X509Certificate) null);
					}
					else addressee.SignatureCertificate = cert;

					if (tmpId != null) 
						addressee.id = tmpId;
					addressee.signatureRefId = refId; 
					nic.SignatureCertificateAddressee = addressee;
					check[SIGNATURE_CERTIFICATE_ADDRESSEE] = 0;
				}
				 tmpId = null;
			}
			typ = - 1;
			currentElement = null;
		}

		static NonIntermediaryCertificatesHBuilder() {
			log = LogFactory.getLog(typeof(NonIntermediaryCertificatesHBuilder));
		}
	}
}