using System;
using System.Globalization;
using  System.Resources;
using  System.Collections;
using System.IO;

namespace java.util
{
	/// <summary>
	/// Die Untersttzung der mehrsprachigen Ausgaben wurde vereinfacht.
    /// Sprachdateien knnen nun als einfache Java-properties-Dateien in
    /// das Verzeichnis der DLL osci-bib.dll kopiert werden. Die Dateinamen
    /// mssen aus dem String "Text_" bestehen, dem der CultureInfo.TwoLetterISOLanguageName
    /// sowie die Erweiterung ".properties" angehngt ist, also z.B. "Text_de.properties".
    /// Als default-Datei fr unbekannte Sprachen wird die Datei "Text.properties"
    /// geladen.
    /// 
	/// </summary>
	public class ResourceBundle
	{
		private ResourceManager rm;	
		private CultureInfo ci;
        private Hashtable props;

		ResourceBundle(String s1, CultureInfo cultureInfo)
		{
            try
            {
                ci = cultureInfo;
                props = new Hashtable();
                String suffix = "_" + ci.TwoLetterISOLanguageName;
                String file = this.GetType().Assembly.CodeBase;
                file = file.Substring(8, file.LastIndexOf('/') - 7);
                if (File.Exists(file + s1 + suffix + ".properties"))
                    file += s1 + suffix + ".properties";
                else
                    file += s1 + ".properties";
                FileStream input = new FileStream(file, FileMode.Open, FileAccess.Read);
                StreamReader sr = new StreamReader(input);
                String line;
                String[] vals;
                while ((line = sr.ReadLine()) != null)
                {
                    line = line.Trim();
                    if (line.Length == 0)
                        continue;
                    if (line.StartsWith("#"))
                        continue;
                    vals = line.Split(new Char[] { '=' }, 2);
                    props.Add(vals[0].Trim(), vals[1].Trim());
                }
            }
            catch (Exception e)
            {
                ResourceManager rm = new ResourceManager(s1, this.GetType().Assembly);
                IDictionaryEnumerator en = rm.GetResourceSet(cultureInfo, true, true).GetEnumerator();
                props = new Hashtable();
                while (en.MoveNext())
                    props.Add(en.Key, en.Value);
            }
		}

		public String getString(String s)
		{
            return (String)props[s];
		}

		static public ResourceBundle getBundle(String s1, CultureInfo cultureInfo)
		{
			if (s1.IndexOf('.') > -1) s1 = s1.Substring(s1.LastIndexOf('.')+1);
			return new ResourceBundle(s1, cultureInfo);
		}
 
	}
}
