namespace org.xml.sax {
	
	using System;
	using System.Collections;
	using System.Xml;
	using org.xml.sax.helpers;
	using org.apache.commons.logging;

	/// <exclude/>
	/// <summary>
	/// Zusammenfassung fr XMLReader.
	/// </summary>
	public class XMLReader 
	{
		private DefaultHandler contentHandler;
		private static Log log;

		public XMLReader() 
		{
			log = LogFactory.getLog(typeof(XMLReader));
		}

		public void setContentHandler(DefaultHandler ch) {
			contentHandler = ch;
		}
		public DefaultHandler getContentHandler() {
			return contentHandler ;
		}


		public void setDTDHandler(Object o) {
			throw new InvalidOperationException("Nicht implementiert");
		}
		
		public void setEntityResolver(Object o) {
			throw new InvalidOperationException("Nicht implementiert");
		}
		
		public void setErrorResolver(Object o) {
			throw new InvalidOperationException("Nicht implementiert");
		}
		
		public void setErrorHandler(Object o) {
		}
		
		public void setFeature(Object o) {
		}
		
		public void setFeature(String s , bool b) {
		}
		
		public void setProperty(String s,bool b) {
			throw new InvalidOperationException("Nicht implementiert");
		}

		public void parse(System.IO.Stream input) {
			parse(input, true);
		}

		public void parse(System.IO.Stream input, bool ns) 
		{		
			XmlTextReader reader = new XmlTextReader(input);
			reader.Namespaces = ns;
			String startElement=null;
			bool empty;

			while (reader.Read()) 
			{
				switch (reader.NodeType) 
				{
					case XmlNodeType.Whitespace:
						if (startElement == null) break;
						char[] ws = reader.Value.ToCharArray();
						contentHandler.characters(ws,0,ws.Length);
					break;

					case XmlNodeType.Attribute:
					break;
				
					case XmlNodeType.Element:
						string localName=reader.LocalName;
						string namespaceURI = reader.NamespaceURI;
						string name = reader.Name;
						empty = reader.IsEmptyElement;
//						if (log.DebugEnabled)
//							log.debug("START ELEM "+reader.Name+":"+contentHandler);

						if (startElement == null) 
						{
							startElement = name;
							contentHandler.startDocument();
						}
						ArrayList attributes = new ArrayList();
						if (reader.HasAttributes) 
						{
							for (int i = 0; i < reader.AttributeCount; i++) 
							{	
								reader.MoveToAttribute(i);
								String[] props = new String[4];
								props[0] = reader.Prefix;
								props[1] = reader.LocalName;
								props[2] = reader.NamespaceURI;
								props[3] = reader.Value;

								if (reader.Prefix.ToLower().Equals("xmlns"))
									contentHandler.startPrefixMapping(reader.LocalName, reader.Value);
		
								attributes.Add(props);
							}
						}
						Attributes atts = new Attributes(attributes);
						contentHandler.startElement(namespaceURI, localName, name, atts);
						if (empty)
							contentHandler.endElement(namespaceURI, localName,  name);
					break;
					
					case XmlNodeType.EndElement:
//						if (log.DebugEnabled)
//							log.debug("END ELEM "+reader.Name+":"+contentHandler);							
						contentHandler.endElement(reader.NamespaceURI, reader.LocalName, reader.Name);
						if (reader.Name.Equals(startElement)) 
						{
							contentHandler.endDocument();
							input.Close();
							return;
						}
					break;

					case XmlNodeType.Text:
						contentHandler.characters(reader.Value.ToCharArray(),0,reader.Value.Length);
					break;
				// There are many other types of nodes, but
				// we are not interested in them
				}
			}
		}
	}
}
